@@ -2,9 +2,10 @@
 
 #include "analyze.h"
 #include "analyze-capability.h"
-#include "cap-list.h"
+#include "capability-list.h"
 #include "capability-util.h"
 #include "format-table.h"
+#include "log.h"
 #include "parse-util.h"
 
 static int table_add_capability(Table *table, int c) {
@@ -83,5 +84,5 @@ int verb_capabilities(int argc, char *argv[], void *userdata) {
         if (r < 0)
                 return r;
 
-        return EXIT_SUCCESS;
+        return 0;
 }
