@@ -1,7 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include "sd-bus.h"
+
 #include "analyze.h"
 #include "analyze-log-control.h"
+#include "bus-util.h"
+#include "runtime-scope.h"
 #include "verb-log-control.h"
 
 int verb_log_control(int argc, char *argv[], void *userdata) {
@@ -18,5 +22,5 @@ int verb_log_control(int argc, char *argv[], void *userdata) {
         if (r < 0)
                 return r;
 
-        return EXIT_SUCCESS;
+        return 0;
 }
