@@ -1,21 +1,17 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
-#include <errno.h>
-#include <limits.h>
-#include <stdarg.h>
 #include <stdlib.h>
 #include <unistd.h>
 
 #include "alloc-util.h"
 #include "env-util.h"
 #include "errno-util.h"
-#include "escape.h"
 #include "extract-word.h"
-#include "macro.h"
+#include "format-util.h"
+#include "log.h"
 #include "parse-util.h"
 #include "path-util.h"
 #include "process-util.h"
-#include "stdio-util.h"
 #include "string-util.h"
 #include "strv.h"
 #include "syslog-util.h"
@@ -26,12 +22,18 @@
         DIGITS LETTERS                          \
         "_"
 
+size_t sc_arg_max(void) {
+        long l = sysconf(_SC_ARG_MAX);
+        assert(l > 0);
+        return (size_t) l;
+}
+
 static bool env_name_is_valid_n(const char *e, size_t n) {
 
         if (n == SIZE_MAX)
                 n = strlen_ptr(e);
 
-        if (n <= 0)
+        if (n == 0)
                 return false;
 
         assert(e);
@@ -42,7 +44,7 @@ static bool env_name_is_valid_n(const char *e, size_t n) {
         /* POSIX says the overall size of the environment block cannot be > ARG_MAX, an individual assignment
          * hence cannot be either. Discounting the equal sign and trailing NUL this hence leaves ARG_MAX-2 as
          * longest possible variable name. */
-        if (n > (size_t) sysconf(_SC_ARG_MAX) - 2)
+        if (_unlikely_(n > sc_arg_max() - 2))
                 return false;
 
         for (const char *p = e; p < e + n; p++)
@@ -53,7 +55,7 @@ static bool env_name_is_valid_n(const char *e, size_t n) {
 }
 
 bool env_name_is_valid(const char *e) {
-        return env_name_is_valid_n(e, strlen_ptr(e));
+        return env_name_is_valid_n(e, SIZE_MAX);
 }
 
 bool env_value_is_valid(const char *e) {
@@ -70,7 +72,7 @@ bool env_value_is_valid(const char *e) {
         /* POSIX says the overall size of the environment block cannot be > ARG_MAX, an individual assignment
          * hence cannot be either. Discounting the shortest possible variable name of length 1, the equal
          * sign and trailing NUL this hence leaves ARG_MAX-3 as longest possible variable value. */
-        if (strlen(e) > sc_arg_max() - 3)
+        if (_unlikely_(strlen(e) > sc_arg_max() - 3))
                 return false;
 
         return true;
@@ -79,6 +81,8 @@ bool env_value_is_valid(const char *e) {
 bool env_assignment_is_valid(const char *e) {
         const char *eq;
 
+        assert(e);
+
         eq = strchr(e, '=');
         if (!eq)
                 return false;
@@ -91,13 +95,13 @@ bool env_assignment_is_valid(const char *e) {
 
         /* POSIX says the overall size of the environment block cannot be > ARG_MAX, hence the individual
          * variable assignments cannot be either, but let's leave room for one trailing NUL byte. */
-        if (strlen(e) > sc_arg_max() - 1)
+        if (_unlikely_(strlen(e) > sc_arg_max() - 1))
                 return false;
 
         return true;
 }
 
-bool strv_env_is_valid(char **e) {
+bool strv_env_is_valid(char * const *e) {
         STRV_FOREACH(p, e) {
                 size_t k;
 
@@ -114,7 +118,7 @@ bool strv_env_is_valid(char **e) {
         return true;
 }
 
-bool strv_env_name_is_valid(char **l) {
+bool strv_env_name_is_valid(char * const *l) {
         STRV_FOREACH(p, l) {
                 if (!env_name_is_valid(*p))
                         return false;
@@ -126,7 +130,7 @@ bool strv_env_name_is_valid(char **l) {
         return true;
 }
 
-bool strv_env_name_or_assignment_is_valid(char **l) {
+bool strv_env_name_or_assignment_is_valid(char * const *l) {
         STRV_FOREACH(p, l) {
                 if (!env_assignment_is_valid(*p) && !env_name_is_valid(*p))
                         return false;
@@ -266,7 +270,7 @@ static bool env_entry_has_name(const char *entry, const char *name) {
         return *t == '=';
 }
 
-char **strv_env_delete(char **x, size_t n_lists, ...) {
+char** strv_env_delete(char **x, size_t n_lists, ...) {
         size_t n, i = 0;
         _cleanup_strv_free_ char **t = NULL;
         va_list ap;
@@ -371,6 +375,7 @@ int strv_env_replace_consume(char ***l, char *p) {
         const char *t, *name;
         int r;
 
+        assert(l);
         assert(p);
 
         /* Replace first occurrence of the env var or add a new one in the string list. Drop other
@@ -404,6 +409,9 @@ int strv_env_replace_consume(char ***l, char *p) {
 int strv_env_replace_strdup(char ***l, const char *assignment) {
         /* Like strv_env_replace_consume(), but copies the argument. */
 
+        assert(l);
+        assert(assignment);
+
         char *p = strdup(assignment);
         if (!p)
                 return -ENOMEM;
@@ -412,11 +420,14 @@ int strv_env_replace_strdup(char ***l, const char *assignment) {
 }
 
 int strv_env_replace_strdup_passthrough(char ***l, const char *assignment) {
-        /* Like strv_env_replace_strdup(), but pulls the variable from the environment of
-         * the calling program, if a variable name without value is specified.
-         */
         char *p;
 
+        /* Like strv_env_replace_strdup(), but pulls the variable from the environment of
+         * the calling program, if a variable name without value is specified. */
+
+        assert(l);
+        assert(assignment);
+
         if (strchr(assignment, '=')) {
                 if (!env_assignment_is_valid(assignment))
                         return -EINVAL;
@@ -438,6 +449,8 @@ int strv_env_replace_strdup_passthrough(char ***l, const char *assignment) {
 }
 
 int strv_env_assign(char ***l, const char *key, const char *value) {
+        assert(l);
+
         if (!env_name_is_valid(key))
                 return -EINVAL;
 
@@ -546,13 +559,13 @@ char* strv_env_get_n(char * const *l, const char *name, size_t k, ReplaceEnvFlag
                         return NULL;
 
                 t = strndupa_safe(name, k);
-                return getenv(t);
+                return secure_getenv(t);
         };
 
         return NULL;
 }
 
-char *strv_env_pairs_get(char **l, const char *name) {
+char* strv_env_pairs_get(char **l, const char *name) {
         char *result = NULL;
 
         assert(name);
@@ -564,10 +577,38 @@ char *strv_env_pairs_get(char **l, const char *name) {
         return result;
 }
 
-char **strv_env_clean_with_callback(char **e, void (*invalid_callback)(const char *p, void *userdata), void *userdata) {
+int strv_env_get_merged(char **l, char ***ret) {
+        _cleanup_strv_free_ char **v = NULL;
+        size_t n = 0;
+        int r;
+
+        assert(ret);
+
+        /* This converts a strv with pairs of environment variable name + value into a strv of name and
+         * value concatenated with a "=" separator. E.g.
+         * input  : { "NAME", "value", "FOO", "var" }
+         * output : { "NAME=value", "FOO=var" } */
+
+        STRV_FOREACH_PAIR(key, value, l) {
+                char *s;
+
+                s = strjoin(*key, "=", *value);
+                if (!s)
+                        return -ENOMEM;
+
+                r = strv_consume_with_size(&v, &n, s);
+                if (r < 0)
+                        return r;
+        }
+
+        *ret = TAKE_PTR(v);
+        return 0;
+}
+
+char** strv_env_clean_with_callback(char **l, void (*invalid_callback)(const char *p, void *userdata), void *userdata) {
         int k = 0;
 
-        STRV_FOREACH(p, e) {
+        STRV_FOREACH(p, l) {
                 size_t n;
                 bool duplicate = false;
 
@@ -590,13 +631,13 @@ char **strv_env_clean_with_callback(char **e, void (*invalid_callback)(const cha
                         continue;
                 }
 
-                e[k++] = *p;
+                l[k++] = *p;
         }
 
-        if (e)
-                e[k] = NULL;
+        if (l)
+                l[k] = NULL;
 
-        return e;
+        return l;
 }
 
 static int strv_extend_with_length(char ***l, const char *s, size_t n) {
@@ -647,7 +688,7 @@ static int strv_env_get_n_validated(
 
 int replace_env_full(
                 const char *format,
-                size_t n,
+                size_t length,
                 char **env,
                 ReplaceEnvFlags flags,
                 char **ret,
@@ -667,19 +708,19 @@ int replace_env_full(
         _cleanup_strv_free_ char **unset_variables = NULL, **bad_variables = NULL;
         const char *e, *word = format, *test_value = NULL; /* test_value is initialized to appease gcc */
         _cleanup_free_ char *s = NULL;
-        char ***pu, ***pb, *k;
+        char ***pu, ***pb;
         size_t i, len = 0; /* len is initialized to appease gcc */
         int nest = 0, r;
 
         assert(format);
 
-        if (n == SIZE_MAX)
-                n = strlen(format);
+        if (length == SIZE_MAX)
+                length = strlen(format);
 
         pu = ret_unset_variables ? &unset_variables : NULL;
         pb = ret_bad_variables ? &bad_variables : NULL;
 
-        for (e = format, i = 0; *e && i < n; e++, i++)
+        for (e = format, i = 0; *e && i < length; e++, i++)
                 switch (state) {
 
                 case WORD:
@@ -689,33 +730,24 @@ int replace_env_full(
 
                 case CURLY:
                         if (*e == '{') {
-                                k = strnappend(s, word, e-word-1);
-                                if (!k)
+                                if (!strextendn(&s, word, e-word-1))
                                         return -ENOMEM;
 
-                                free_and_replace(s, k);
-
                                 word = e-1;
                                 state = VARIABLE;
                                 nest++;
 
                         } else if (*e == '$') {
-                                k = strnappend(s, word, e-word);
-                                if (!k)
+                                if (!strextendn(&s, word, e-word))
                                         return -ENOMEM;
 
-                                free_and_replace(s, k);
-
                                 word = e+1;
                                 state = WORD;
 
                         } else if (FLAGS_SET(flags, REPLACE_ENV_ALLOW_BRACELESS) && strchr(VALID_BASH_ENV_NAME_CHARS, *e)) {
-                                k = strnappend(s, word, e-word-1);
-                                if (!k)
+                                if (!strextendn(&s, word, e-word-1))
                                         return -ENOMEM;
 
-                                free_and_replace(s, k);
-
                                 word = e-1;
                                 state = VARIABLE_RAW;
 
@@ -796,10 +828,10 @@ int replace_env_full(
                                         t = v;
                                 }
 
-                                r = strv_extend_strv(&unset_variables, u, /* filter_duplicates= */ true);
+                                r = strv_extend_strv_consume(&unset_variables, TAKE_PTR(u), /* filter_duplicates= */ true);
                                 if (r < 0)
                                         return r;
-                                r = strv_extend_strv(&bad_variables, b, /* filter_duplicates= */ true);
+                                r = strv_extend_strv_consume(&bad_variables, TAKE_PTR(b), /* filter_duplicates= */ true);
                                 if (r < 0)
                                         return r;
 
@@ -865,9 +897,12 @@ int replace_env_argv(
                 char ***ret_bad_variables) {
 
         _cleanup_strv_free_ char **n = NULL, **unset_variables = NULL, **bad_variables = NULL;
-        size_t k = 0, l = 0;
+        size_t k = 0, l;
         int r;
 
+        assert(!strv_isempty(argv));
+        assert(ret);
+
         l = strv_length(argv);
 
         n = new(char*, l+1);
@@ -931,21 +966,21 @@ int replace_env_argv(
                         return r;
                 n[++k] = NULL;
 
-                r = strv_extend_strv(&unset_variables, u, /* filter_duplicates= */ true);
+                r = strv_extend_strv_consume(&unset_variables, TAKE_PTR(u), /* filter_duplicates= */ true);
                 if (r < 0)
                         return r;
 
-                r = strv_extend_strv(&bad_variables, b, /*filter_duplicates= */ true);
+                r = strv_extend_strv_consume(&bad_variables, TAKE_PTR(b), /* filter_duplicates= */ true);
                 if (r < 0)
                         return r;
         }
 
         if (ret_unset_variables) {
-                strv_uniq(strv_sort(unset_variables));
+                strv_sort_uniq(unset_variables);
                 *ret_unset_variables = TAKE_PTR(unset_variables);
         }
         if (ret_bad_variables) {
-                strv_uniq(strv_sort(bad_variables));
+                strv_sort_uniq(bad_variables);
                 *ret_bad_variables = TAKE_PTR(bad_variables);
         }
 
@@ -1086,7 +1121,7 @@ int getenv_steal_erase(const char *name, char **ret) {
          * it from there. Usecase: reading passwords from the env block (which is a bad idea, but useful for
          * testing, and given that people are likely going to misuse this, be thorough) */
 
-        e = getenv(name);
+        e = secure_getenv(name);
         if (!e) {
                 if (ret)
                         *ret = NULL;
@@ -1110,25 +1145,6 @@ int getenv_steal_erase(const char *name, char **ret) {
         return 1;
 }
 
-int set_full_environment(char **env) {
-        int r;
-
-        clearenv();
-
-        STRV_FOREACH(e, env) {
-                _cleanup_free_ char *k = NULL, *v = NULL;
-
-                r = split_pair(*e, "=", &k, &v);
-                if (r < 0)
-                        return r;
-
-                if (setenv(k, v, /* overwrite= */ true) < 0)
-                        return -errno;
-        }
-
-        return 0;
-}
-
 int setenvf(const char *name, bool overwrite, const char *valuef, ...) {
         _cleanup_free_ char *value = NULL;
         va_list ap;
@@ -1146,5 +1162,10 @@ int setenvf(const char *name, bool overwrite, const char *valuef, ...) {
         if (r < 0)
                 return -ENOMEM;
 
+        /* Try to suppress writes if the value is already set correctly (simply because memory management of
+         * environment variables sucks a bit. */
+        if (streq_ptr(getenv(name), value))
+                return 0;
+
         return RET_NERRNO(setenv(name, value, overwrite));
 }
