@@ -144,32 +144,32 @@ char ascii_toupper(char x) {
         return x;
 }
 
-char* ascii_strlower(char *t) {
-        assert(t);
+char* ascii_strlower(char *s) {
+        assert(s);
 
-        for (char *p = t; *p; p++)
+        for (char *p = s; *p; p++)
                 *p = ascii_tolower(*p);
 
-        return t;
+        return s;
 }
 
-char* ascii_strupper(char *t) {
-        assert(t);
+char* ascii_strupper(char *s) {
+        assert(s);
 
-        for (char *p = t; *p; p++)
+        for (char *p = s; *p; p++)
                 *p = ascii_toupper(*p);
 
-        return t;
+        return s;
 }
 
-char* ascii_strlower_n(char *t, size_t n) {
+char* ascii_strlower_n(char *s, size_t n) {
         if (n <= 0)
-                return t;
+                return s;
 
         for (size_t i = 0; i < n; i++)
-                t[i] = ascii_tolower(t[i]);
+                s[i] = ascii_tolower(s[i]);
 
-        return t;
+        return s;
 }
 
 int ascii_strcasecmp_n(const char *a, const char *b, size_t n) {
@@ -332,7 +332,7 @@ static char *ascii_ellipsize_mem(const char *s, size_t old_length, size_t new_le
         x = ((new_length - need_space) * percent + 50) / 100;
         assert(x <= new_length - need_space);
 
-        write_ellipsis(mempcpy(t, s, x), /* unicode = */ false);
+        write_ellipsis(mempcpy(t, s, x), /* unicode= */ false);
         suffix_len = new_length - x - need_space;
         memcpy(t + x + 3, s + old_length - suffix_len, suffix_len);
         *(t + x + 3 + suffix_len) = '\0';
@@ -454,7 +454,7 @@ char* ellipsize_mem(const char *s, size_t old_length, size_t new_length, unsigne
                 return NULL;
 
         memcpy_safe(e, s, len);
-        write_ellipsis(e + len, /* unicode = */ true);
+        write_ellipsis(e + len, /* unicode= */ true);
 
         char *dst = e + len + 3;
 
@@ -533,7 +533,7 @@ char* cellescape(char *buf, size_t len, const char *s) {
         }
 
         if (i + 4 <= len) /* yay, enough space */
-                i += write_ellipsis(buf + i, /* unicode = */ false);
+                i += write_ellipsis(buf + i, /* unicode= */ false);
         else if (i + 3 <= len) { /* only space for ".." */
                 buf[i++] = '.';
                 buf[i++] = '.';
@@ -735,7 +735,7 @@ char* strip_tab_ansi(char **ibuf, size_t *_isz, size_t highlight[2]) {
                 case STATE_CSI:
                         assert(n_carriage_returns == 0);
 
-                        if (eot || !strchr("01234567890;m", *i)) { /* EOT or invalid chars in sequence */
+                        if (eot || !strchr(DIGITS ";:m", *i)) { /* EOT or invalid chars in sequence */
                                 fputc('\x1B', f);
                                 fputc('[', f);
                                 advance_offsets(i - *ibuf, highlight, shift, 2);
@@ -793,12 +793,11 @@ char* strip_tab_ansi(char **ibuf, size_t *_isz, size_t highlight[2]) {
         return *ibuf;
 }
 
-char* strextend_with_separator_internal(char **x, const char *separator, ...) {
+char* strextendv_with_separator(char **x, const char *separator, va_list ap) {
         _cleanup_free_ char *buffer = NULL;
         size_t f, l, l_separator;
         bool need_separator;
         char *nr, *p;
-        va_list ap;
 
         if (!x)
                 x = &buffer;
@@ -808,11 +807,12 @@ char* strextend_with_separator_internal(char **x, const char *separator, ...) {
         need_separator = !isempty(*x);
         l_separator = strlen_ptr(separator);
 
-        va_start(ap, separator);
+        va_list aq;
+        va_copy(aq, ap);
         for (const char *t;;) {
                 size_t n;
 
-                t = va_arg(ap, const char *);
+                t = va_arg(aq, const char *);
                 if (!t)
                         break;
                 if (t == POINTER_MAX)
@@ -824,14 +824,14 @@ char* strextend_with_separator_internal(char **x, const char *separator, ...) {
                         n += l_separator;
 
                 if (n >= SIZE_MAX - l) {
-                        va_end(ap);
+                        va_end(aq);
                         return NULL;
                 }
 
                 l += n;
                 need_separator = true;
         }
-        va_end(ap);
+        va_end(aq);
 
         need_separator = !isempty(*x);
 
@@ -842,7 +842,6 @@ char* strextend_with_separator_internal(char **x, const char *separator, ...) {
         *x = nr;
         p = nr + f;
 
-        va_start(ap, separator);
         for (;;) {
                 const char *t;
 
@@ -859,7 +858,6 @@ char* strextend_with_separator_internal(char **x, const char *separator, ...) {
 
                 need_separator = true;
         }
-        va_end(ap);
 
         assert(p == nr + l);
         *p = 0;
@@ -872,6 +870,17 @@ char* strextend_with_separator_internal(char **x, const char *separator, ...) {
         return p;
 }
 
+char* strextend_with_separator_internal(char **x, const char *separator, ...) {
+        va_list ap;
+        char *ret;
+
+        va_start(ap, separator);
+        ret = strextendv_with_separator(x, separator, ap);
+        va_end(ap);
+
+        return ret;
+}
+
 int strextendf_with_separator(char **x, const char *separator, const char *format, ...) {
         size_t m, a, l_separator;
         va_list ap;
