@@ -1,15 +1,12 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
-#include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
-#include <unistd.h>
 
 #include "af-list.h"
-#include "fd-util.h"
+#include "alloc-util.h"
 #include "fileio.h"
+#include "hashmap.h"
 #include "log.h"
-#include "macro.h"
 #include "parse-util.h"
 #include "path-util.h"
 #include "socket-util.h"
@@ -115,26 +112,37 @@ int sysctl_writef(const char *property, const char *format, ...) {
         return sysctl_write(property, v);
 }
 
+static const char* af_to_sysctl_dir(int af) {
+        if (af == AF_MPLS)
+                return "mpls";
+
+        return af_to_ipv4_ipv6(af);
+}
+
 int sysctl_write_ip_property(int af, const char *ifname, const char *property, const char *value, Hashmap **shadow) {
         const char *p;
 
         assert(property);
         assert(value);
 
-        if (!IN_SET(af, AF_INET, AF_INET6))
+        if (!IN_SET(af, AF_INET, AF_INET6, AF_MPLS))
                 return -EAFNOSUPPORT;
 
         if (ifname) {
                 if (!ifname_valid_full(ifname, IFNAME_VALID_SPECIAL))
                         return -EINVAL;
 
-                p = strjoina("net/", af_to_ipv4_ipv6(af), "/conf/", ifname, "/", property);
+                p = strjoina("net/", af_to_sysctl_dir(af), "/conf/", ifname, "/", property);
         } else
-                p = strjoina("net/", af_to_ipv4_ipv6(af), "/", property);
+                p = strjoina("net/", af_to_sysctl_dir(af), "/", property);
 
         return sysctl_write_full(p, value, shadow);
 }
 
+int sysctl_write_ip_property_boolean(int af, const char *ifname, const char *property, bool value, Hashmap **shadow) {
+        return sysctl_write_ip_property(af, ifname, property, one_zero(value), shadow);
+}
+
 int sysctl_write_ip_neighbor_property(int af, const char *ifname, const char *property, const char *value, Hashmap **shadow) {
         const char *p;
 
@@ -155,6 +163,12 @@ int sysctl_write_ip_neighbor_property(int af, const char *ifname, const char *pr
         return sysctl_write_full(p, value, shadow);
 }
 
+int sysctl_write_ip_neighbor_property_uint32(int af, const char *ifname, const char *property, uint32_t value, Hashmap **shadow) {
+        char buf[DECIMAL_STR_MAX(uint32_t)];
+        xsprintf(buf, "%u", value);
+        return sysctl_write_ip_neighbor_property(af, ifname, property, buf, shadow);
+}
+
 int sysctl_read(const char *property, char **ret) {
         char *p;
         int r;
@@ -181,16 +195,16 @@ int sysctl_read_ip_property(int af, const char *ifname, const char *property, ch
 
         assert(property);
 
-        if (!IN_SET(af, AF_INET, AF_INET6))
+        if (!IN_SET(af, AF_INET, AF_INET6, AF_MPLS))
                 return -EAFNOSUPPORT;
 
         if (ifname) {
                 if (!ifname_valid_full(ifname, IFNAME_VALID_SPECIAL))
                         return -EINVAL;
 
-                p = strjoina("net/", af_to_ipv4_ipv6(af), "/conf/", ifname, "/", property);
+                p = strjoina("net/", af_to_sysctl_dir(af), "/conf/", ifname, "/", property);
         } else
-                p = strjoina("net/", af_to_ipv4_ipv6(af), "/", property);
+                p = strjoina("net/", af_to_sysctl_dir(af), "/", property);
 
         return sysctl_read(p, ret);
 }
