@@ -7,6 +7,7 @@
 #  include "util.h"
 #else
 #  include <stdlib.h>
+
 #  include "alloc-util.h"
 #  define xnew(t, n) ASSERT_SE_PTR(new(t, n))
 #  define xmalloc(n) ASSERT_SE_PTR(malloc(n))
@@ -188,8 +189,10 @@ static unsigned utf8_to_unichar(const char *utf8, size_t n, char32_t *c) {
 
 /* Convert UTF-8 to UCS-2, skipping any invalid or short byte sequences. */
 char16_t *xstrn8_to_16(const char *str8, size_t n) {
-        if (!str8 || n == 0)
-                return NULL;
+        assert(str8 || n == 0);
+
+        if (n == SIZE_MAX)
+                n = strlen8(str8);
 
         size_t i = 0;
         char16_t *str16 = xnew(char16_t, n + 1);
@@ -209,10 +212,34 @@ char16_t *xstrn8_to_16(const char *str8, size_t n) {
                 }
         }
 
-        str16[i] = '\0';
+        str16[i] = u'\0';
         return str16;
 }
 
+char *xstrn16_to_ascii(const char16_t *str16, size_t n) {
+        assert(str16 || n == 0);
+
+        if (n == SIZE_MAX)
+                n = strlen16(str16);
+
+        _cleanup_free_ char *str8 = xnew(char, n + 1);
+
+        size_t i = 0;
+        while (n > 0 && *str16 != u'\0') {
+
+                if ((uint16_t) *str16 > 127U) /* Not ASCII? Fail! */
+                        return NULL;
+
+                str8[i++] = (char) (uint16_t) *str16;
+
+                str16++;
+                n--;
+        }
+
+        str8[i] = '\0';
+        return TAKE_PTR(str8);
+}
+
 char* startswith8(const char *s, const char *prefix) {
         size_t l;
 
@@ -470,7 +497,8 @@ char* line_get_key_value(char *s, const char *sep, size_t *pos, char **ret_key,
                         value++;
 
                 /* unquote */
-                if (value[0] == '"' && line[linelen - 1] == '"') {
+                if ((value[0] == '"' && line[linelen - 1] == '"') ||
+                    (value[0] == '\'' && line[linelen - 1] == '\'')) {
                         value++;
                         line[linelen - 1] = '\0';
                 }
@@ -726,7 +754,7 @@ static bool handle_format_specifier(FormatContext *ctx, SpecifierContext *sp) {
          * otherwise warn about fetching smaller types. */
         assert_cc(sizeof(int) == 4);
         assert_cc(sizeof(wchar_t) <= sizeof(int));
-        assert_cc(sizeof(intmax_t) <= sizeof(long long));
+        assert_cc(sizeof(long long) == sizeof(intmax_t));
 
         assert(ctx);
         assert(sp);
@@ -890,6 +918,37 @@ static bool handle_format_specifier(FormatContext *ctx, SpecifierContext *sp) {
         }
 }
 
+#if SD_BOOT
+static void output_string_safe(EFI_SIMPLE_TEXT_OUTPUT_PROTOCOL *this, const char16_t *s) {
+        assert(this);
+        assert(s);
+
+        /* This is a color-conscious version of ST->ConOut->OutputString(). Whenever it encounters a newline
+         * character, it will reset the color to our default, because some UEFI implementations/terminals
+         * reset the color in that case, and we want our default color to remain in effect */
+
+        int32_t saved_attribute = ST->ConOut->Mode->Attribute;
+        for (;;) {
+                const char16_t *nl = strchr16(s, '\n');
+                if (!nl) /* No further newline */
+                        return (void) ST->ConOut->OutputString(ST->ConOut, (char16_t*) s);
+
+                if (nl[1] == 0) { /* Newline is at the end of the string */
+                        (void) ST->ConOut->OutputString(ST->ConOut, (char16_t*) s);
+                        set_attribute_safe(saved_attribute);
+                        return;
+                }
+
+                /* newline is in the middle of the string */
+                _cleanup_free_ char16_t *x = xstrndup16(s, nl - s + 1);
+                (void) ST->ConOut->OutputString(ST->ConOut, x);
+                set_attribute_safe(saved_attribute);
+
+                s = nl + 1;
+        }
+}
+#endif
+
 /* printf_internal is largely compatible to userspace vasprintf. Any features omitted should trigger asserts.
  *
  * Supported:
@@ -956,7 +1015,7 @@ _printf_(2, 0) static char16_t *printf_internal(EFI_STATUS status, const char *f
         }
 
 #if SD_BOOT
-        ST->ConOut->OutputString(ST->ConOut, ctx.buf);
+        output_string_safe(ST->ConOut, ctx.buf);
 #endif
 
         return mfree(ctx.dyn_buf);
@@ -997,6 +1056,7 @@ _used_ void *memcpy(void * restrict dest, const void * restrict src, size_t n);
 _used_ void *memset(void *p, int c, size_t n);
 #else
 /* And for userspace unit testing we need to give them an efi_ prefix. */
+#  undef memchr
 #  define memchr efi_memchr
 #  define memcmp efi_memcmp
 #  define memcpy efi_memcpy
@@ -1083,3 +1143,27 @@ void *memset(void *p, int c, size_t n) {
 
         return p;
 }
+
+size_t strspn16(const char16_t *p, const char16_t *good) {
+        assert(p);
+        assert(good);
+
+        const char16_t *i = p;
+        for (; *i != 0; i++)
+                if (!strchr16(good, *i))
+                        break;
+
+        return i - p;
+}
+
+size_t strcspn16(const char16_t *p, const char16_t *bad) {
+        assert(p);
+        assert(bad);
+
+        const char16_t *i = p;
+        for (; *i != 0; i++)
+                if (strchr16(bad, *i))
+                        break;
+
+        return i - p;
+}
