@@ -211,10 +211,10 @@ EFI_STATUS process_random_seed(EFI_FILE *root_dir) {
 
         size = info->FileSize;
         if (size < RANDOM_MAX_SIZE_MIN)
-                return log_error("Random seed file is too short.");
+                return log_error_status(EFI_INVALID_PARAMETER, "Random seed file is too short.");
 
         if (size > RANDOM_MAX_SIZE_MAX)
-                return log_error("Random seed file is too large.");
+                return log_error_status(EFI_INVALID_PARAMETER, "Random seed file is too large.");
 
         seed = xmalloc(size);
         rsize = size;
