@@ -1,24 +1,28 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include <grp.h>
 #include <sys/file.h>
 #include <sys/stat.h>
-#include <sys/types.h>
+#include <unistd.h>
 
 #include "clean-ipc.h"
 #include "dynamic-user.h"
+#include "errno-list.h"
+#include "extract-word.h"
 #include "fd-util.h"
+#include "fdset.h"
 #include "fileio.h"
 #include "format-util.h"
-#include "fs-util.h"
+#include "hashmap.h"
 #include "iovec-util.h"
 #include "lock-util.h"
-#include "parse-util.h"
+#include "manager.h"
 #include "random-util.h"
 #include "serialize.h"
+#include "siphash24.h"
 #include "socket-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
-#include "strv.h"
 #include "uid-classification.h"
 #include "user-util.h"
 
@@ -732,11 +736,20 @@ int dynamic_user_lookup_name(Manager *m, const char *name, uid_t *ret) {
         if (!d)
                 return -ESRCH;
 
-        r = dynamic_user_current(d, ret);
+        uid_t uid;
+        r = dynamic_user_current(d, &uid);
         if (r == -EAGAIN) /* not realized yet? */
                 return -ESRCH;
+        if (r < 0)
+                return r;
 
-        return r;
+        if (!uid_is_dynamic(uid))
+                return -ESRCH;
+
+        if (ret)
+                *ret = uid;
+
+        return 0;
 }
 
 int dynamic_creds_make(Manager *m, const char *user, const char *group, DynamicCreds **ret) {
