@@ -3,9 +3,12 @@
 #include <getopt.h>
 #include <sys/mman.h>
 
-#include "ask-password-api.h"
+#include "sd-device.h"
+
+#include "blockdev-list.h"
 #include "blockdev-util.h"
 #include "build.h"
+#include "cryptenroll.h"
 #include "cryptenroll-fido2.h"
 #include "cryptenroll-list.h"
 #include "cryptenroll-password.h"
@@ -13,24 +16,22 @@
 #include "cryptenroll-recovery.h"
 #include "cryptenroll-tpm2.h"
 #include "cryptenroll-wipe.h"
-#include "cryptenroll.h"
 #include "cryptsetup-util.h"
-#include "devnum-util.h"
-#include "env-util.h"
-#include "escape.h"
+#include "extract-word.h"
 #include "fileio.h"
 #include "libfido2-util.h"
+#include "log.h"
 #include "main-func.h"
-#include "memory-util.h"
+#include "pager.h"
 #include "parse-argument.h"
 #include "parse-util.h"
-#include "path-util.h"
 #include "pkcs11-util.h"
 #include "pretty-print.h"
+#include "process-util.h"
 #include "string-table.h"
-#include "strv.h"
-#include "terminal-util.h"
+#include "string-util.h"
 #include "tpm2-pcr.h"
+#include "tpm2-util.h"
 
 static EnrollType arg_enroll_type = _ENROLL_TYPE_INVALID;
 static char *arg_unlock_keyfile = NULL;
@@ -39,6 +40,8 @@ static char *arg_unlock_fido2_device = NULL;
 static char *arg_unlock_tpm2_device = NULL;
 static char *arg_pkcs11_token_uri = NULL;
 static char *arg_fido2_device = NULL;
+static char *arg_fido2_salt_file = NULL;
+static bool arg_fido2_parameters_in_header = true;
 static char *arg_tpm2_device = NULL;
 static uint32_t arg_tpm2_seal_key_handle = 0;
 static char *arg_tpm2_device_key = NULL;
@@ -51,6 +54,7 @@ static uint32_t arg_tpm2_public_key_pcr_mask = 0;
 static char *arg_tpm2_signature = NULL;
 static char *arg_tpm2_pcrlock = NULL;
 static char *arg_node = NULL;
+PagerFlags arg_pager_flags = 0;
 static int *arg_wipe_slots = NULL;
 static size_t arg_n_wipe_slots = 0;
 static WipeScope arg_wipe_slots_scope = WIPE_EXPLICIT;
@@ -69,6 +73,7 @@ STATIC_DESTRUCTOR_REGISTER(arg_unlock_fido2_device, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_unlock_tpm2_device, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_pkcs11_token_uri, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_fido2_device, freep);
+STATIC_DESTRUCTOR_REGISTER(arg_fido2_salt_file, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_tpm2_device, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_tpm2_device_key, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_tpm2_hash_pcr_values, freep);
@@ -147,7 +152,7 @@ static int determine_default_node(void) {
                 return log_error_errno(SYNTHETIC_ERRNO(ENXIO), "Block device backing /var/ is not a LUKS2 device.");
 
         _cleanup_(sd_device_unrefp) sd_device *origin = NULL;
-        r = block_device_get_originating(dev, &origin);
+        r = block_device_get_originating(dev, &origin, /* recursive= */ false);
         if (r < 0)
                 return log_error_errno(r, "Failed to get originating device of LUKS2 device backing /var/: %m");
 
@@ -168,6 +173,8 @@ static int help(void) {
         _cleanup_free_ char *link = NULL;
         int r;
 
+        pager_open(arg_pager_flags);
+
         r = terminal_urlify_man("systemd-cryptenroll", "1", &link);
         if (r < 0)
                 return log_oom();
@@ -176,6 +183,8 @@ static int help(void) {
                "%5$sEnroll a security token or authentication credential to a LUKS volume.%6$s\n\n"
                "  -h --help            Show this help\n"
                "     --version         Show package version\n"
+               "     --no-pager        Do not spawn a pager\n"
+               "     --list-devices    List candidate block devices to operate on\n"
                "     --wipe-slot=SLOT1,SLOT2,…\n"
                "                       Wipe specified slots\n"
                "\n%3$sUnlocking:%4$s\n"
@@ -189,11 +198,15 @@ static int help(void) {
                "     --password        Enroll a user-supplied password\n"
                "     --recovery-key    Enroll a recovery key\n"
                "\n%3$sPKCS#11 Enrollment:%4$s\n"
-               "     --pkcs11-token-uri=URI\n"
-               "                       Specify PKCS#11 security token URI\n"
+               "     --pkcs11-token-uri=URI|auto|list\n"
+               "                       Enroll a PKCS#11 security token or list them\n"
                "\n%3$sFIDO2 Enrollment:%4$s\n"
-               "     --fido2-device=PATH\n"
-               "                       Enroll a FIDO2-HMAC security token\n"
+               "     --fido2-device=PATH|auto|list\n"
+               "                       Enroll a FIDO2-HMAC security token or list them\n"
+               "     --fido2-salt-file=PATH\n"
+               "                       Use salt from a file instead of generating one\n"
+               "     --fido2-parameters-in-header=BOOL\n"
+               "                       Whether to store FIDO2 parameters in the LUKS2 header\n"
                "     --fido2-credential-algorithm=STRING\n"
                "                       Specify COSE algorithm for FIDO2 credential\n"
                "     --fido2-with-client-pin=BOOL\n"
@@ -203,8 +216,8 @@ static int help(void) {
                "     --fido2-with-user-verification=BOOL\n"
                "                       Whether to require user verification to unlock the volume\n"
                "\n%3$sTPM2 Enrollment:%4$s\n"
-               "     --tpm2-device=PATH\n"
-               "                       Enroll a TPM2 device\n"
+               "     --tpm2-device=PATH|auto|list\n"
+               "                       Enroll a TPM2 device or list them\n"
                "     --tpm2-device-key=PATH\n"
                "                       Enroll a TPM2 device using its public key\n"
                "     --tpm2-seal-key-handle=HANDLE\n"
@@ -236,6 +249,7 @@ static int help(void) {
 static int parse_argv(int argc, char *argv[]) {
         enum {
                 ARG_VERSION = 0x100,
+                ARG_NO_PAGER,
                 ARG_PASSWORD,
                 ARG_RECOVERY_KEY,
                 ARG_UNLOCK_KEYFILE,
@@ -243,6 +257,8 @@ static int parse_argv(int argc, char *argv[]) {
                 ARG_UNLOCK_TPM2_DEVICE,
                 ARG_PKCS11_TOKEN_URI,
                 ARG_FIDO2_DEVICE,
+                ARG_FIDO2_SALT_FILE,
+                ARG_FIDO2_PARAMETERS_IN_HEADER,
                 ARG_TPM2_DEVICE,
                 ARG_TPM2_DEVICE_KEY,
                 ARG_TPM2_SEAL_KEY_HANDLE,
@@ -257,42 +273,47 @@ static int parse_argv(int argc, char *argv[]) {
                 ARG_FIDO2_WITH_UP,
                 ARG_FIDO2_WITH_UV,
                 ARG_FIDO2_CRED_ALG,
+                ARG_LIST_DEVICES,
         };
 
         static const struct option options[] = {
-                { "help",                         no_argument,       NULL, 'h'                       },
-                { "version",                      no_argument,       NULL, ARG_VERSION               },
-                { "password",                     no_argument,       NULL, ARG_PASSWORD              },
-                { "recovery-key",                 no_argument,       NULL, ARG_RECOVERY_KEY          },
-                { "unlock-key-file",              required_argument, NULL, ARG_UNLOCK_KEYFILE        },
-                { "unlock-fido2-device",          required_argument, NULL, ARG_UNLOCK_FIDO2_DEVICE   },
-                { "unlock-tpm2-device",           required_argument, NULL, ARG_UNLOCK_TPM2_DEVICE    },
-                { "pkcs11-token-uri",             required_argument, NULL, ARG_PKCS11_TOKEN_URI      },
-                { "fido2-credential-algorithm",   required_argument, NULL, ARG_FIDO2_CRED_ALG        },
-                { "fido2-device",                 required_argument, NULL, ARG_FIDO2_DEVICE          },
-                { "fido2-with-client-pin",        required_argument, NULL, ARG_FIDO2_WITH_PIN        },
-                { "fido2-with-user-presence",     required_argument, NULL, ARG_FIDO2_WITH_UP         },
-                { "fido2-with-user-verification", required_argument, NULL, ARG_FIDO2_WITH_UV         },
-                { "tpm2-device",                  required_argument, NULL, ARG_TPM2_DEVICE           },
-                { "tpm2-device-key",              required_argument, NULL, ARG_TPM2_DEVICE_KEY       },
-                { "tpm2-seal-key-handle",         required_argument, NULL, ARG_TPM2_SEAL_KEY_HANDLE  },
-                { "tpm2-pcrs",                    required_argument, NULL, ARG_TPM2_PCRS             },
-                { "tpm2-public-key",              required_argument, NULL, ARG_TPM2_PUBLIC_KEY       },
-                { "tpm2-public-key-pcrs",         required_argument, NULL, ARG_TPM2_PUBLIC_KEY_PCRS  },
-                { "tpm2-signature",               required_argument, NULL, ARG_TPM2_SIGNATURE        },
-                { "tpm2-pcrlock",                 required_argument, NULL, ARG_TPM2_PCRLOCK          },
-                { "tpm2-with-pin",                required_argument, NULL, ARG_TPM2_WITH_PIN         },
-                { "wipe-slot",                    required_argument, NULL, ARG_WIPE_SLOT             },
+                { "help",                          no_argument,       NULL, 'h'                            },
+                { "version",                       no_argument,       NULL, ARG_VERSION                    },
+                { "no-pager",                      no_argument,       NULL, ARG_NO_PAGER                   },
+                { "password",                      no_argument,       NULL, ARG_PASSWORD                   },
+                { "recovery-key",                  no_argument,       NULL, ARG_RECOVERY_KEY               },
+                { "unlock-key-file",               required_argument, NULL, ARG_UNLOCK_KEYFILE             },
+                { "unlock-fido2-device",           required_argument, NULL, ARG_UNLOCK_FIDO2_DEVICE        },
+                { "unlock-tpm2-device",            required_argument, NULL, ARG_UNLOCK_TPM2_DEVICE         },
+                { "pkcs11-token-uri",              required_argument, NULL, ARG_PKCS11_TOKEN_URI           },
+                { "fido2-credential-algorithm",    required_argument, NULL, ARG_FIDO2_CRED_ALG             },
+                { "fido2-device",                  required_argument, NULL, ARG_FIDO2_DEVICE               },
+                { "fido2-salt-file",               required_argument, NULL, ARG_FIDO2_SALT_FILE            },
+                { "fido2-parameters-in-header",    required_argument, NULL, ARG_FIDO2_PARAMETERS_IN_HEADER },
+                { "fido2-with-client-pin",         required_argument, NULL, ARG_FIDO2_WITH_PIN             },
+                { "fido2-with-user-presence",      required_argument, NULL, ARG_FIDO2_WITH_UP              },
+                { "fido2-with-user-verification",  required_argument, NULL, ARG_FIDO2_WITH_UV              },
+                { "tpm2-device",                   required_argument, NULL, ARG_TPM2_DEVICE                },
+                { "tpm2-device-key",               required_argument, NULL, ARG_TPM2_DEVICE_KEY            },
+                { "tpm2-seal-key-handle",          required_argument, NULL, ARG_TPM2_SEAL_KEY_HANDLE       },
+                { "tpm2-pcrs",                     required_argument, NULL, ARG_TPM2_PCRS                  },
+                { "tpm2-public-key",               required_argument, NULL, ARG_TPM2_PUBLIC_KEY            },
+                { "tpm2-public-key-pcrs",          required_argument, NULL, ARG_TPM2_PUBLIC_KEY_PCRS       },
+                { "tpm2-signature",                required_argument, NULL, ARG_TPM2_SIGNATURE             },
+                { "tpm2-pcrlock",                  required_argument, NULL, ARG_TPM2_PCRLOCK               },
+                { "tpm2-with-pin",                 required_argument, NULL, ARG_TPM2_WITH_PIN              },
+                { "wipe-slot",                     required_argument, NULL, ARG_WIPE_SLOT                  },
+                { "list-devices",                  no_argument,       NULL, ARG_LIST_DEVICES               },
                 {}
         };
 
-        bool auto_hash_pcr_values = true, auto_public_key_pcr_mask = true, auto_pcrlock = true;
+        bool auto_public_key_pcr_mask = true, auto_pcrlock = true;
         int c, r;
 
         assert(argc >= 0);
         assert(argv);
 
-        while ((c = getopt_long(argc, argv, "h", options, NULL)) >= 0) {
+        while ((c = getopt_long(argc, argv, "h", options, NULL)) >= 0)
 
                 switch (c) {
 
@@ -302,6 +323,10 @@ static int parse_argv(int argc, char *argv[]) {
                 case ARG_VERSION:
                         return version();
 
+                case ARG_NO_PAGER:
+                        arg_pager_flags |= PAGER_DISABLE;
+                        break;
+
                 case ARG_FIDO2_WITH_PIN:
                         r = parse_boolean_argument("--fido2-with-client-pin=", optarg, NULL);
                         if (r < 0)
@@ -449,11 +474,25 @@ static int parse_argv(int argc, char *argv[]) {
                         break;
                 }
 
+                case ARG_FIDO2_SALT_FILE:
+                        r = parse_path_argument(optarg, /* suppress_root= */ true, &arg_fido2_salt_file);
+                        if (r < 0)
+                                return r;
+
+                        break;
+
+                case ARG_FIDO2_PARAMETERS_IN_HEADER:
+                        r = parse_boolean_argument("--fido2-parameters-in-header=", optarg, &arg_fido2_parameters_in_header);
+                        if (r < 0)
+                                return r;
+
+                        break;
+
                 case ARG_TPM2_DEVICE: {
                         _cleanup_free_ char *device = NULL;
 
                         if (streq(optarg, "list"))
-                                return tpm2_list_devices();
+                                return tpm2_list_devices(/* legend= */ true, /* quiet= */ false);
 
                         if (arg_enroll_type >= 0 || arg_tpm2_device)
                                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
@@ -475,7 +514,6 @@ static int parse_argv(int argc, char *argv[]) {
                                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
                                                        "Multiple operations specified at once, refusing.");
 
-
                         r = parse_path_argument(optarg, /* suppress_root= */ false, &arg_tpm2_device_key);
                         if (r < 0)
                                 return r;
@@ -491,7 +529,6 @@ static int parse_argv(int argc, char *argv[]) {
                         break;
 
                 case ARG_TPM2_PCRS:
-                        auto_hash_pcr_values = false;
                         r = tpm2_parse_pcr_argument_append(optarg, &arg_tpm2_hash_pcr_values, &arg_tpm2_n_hash_pcr_values);
                         if (r < 0)
                                 return r;
@@ -578,32 +615,34 @@ static int parse_argv(int argc, char *argv[]) {
                                 else if (streq(slot, "tpm2"))
                                         arg_wipe_slots_mask |= 1U << ENROLL_TPM2;
                                 else {
-                                        int *a;
-
                                         r = safe_atou(slot, &n);
                                         if (r < 0)
                                                 return log_error_errno(r, "Failed to parse slot index: %s", slot);
                                         if (n > INT_MAX)
                                                 return log_error_errno(SYNTHETIC_ERRNO(ERANGE), "Slot index out of range: %u", n);
 
-                                        a = reallocarray(arg_wipe_slots, arg_n_wipe_slots + 1, sizeof(int));
-                                        if (!a)
+                                        if (!GREEDY_REALLOC(arg_wipe_slots, arg_n_wipe_slots + 1))
                                                 return log_oom();
 
-                                        arg_wipe_slots = a;
                                         arg_wipe_slots[arg_n_wipe_slots++] = (int) n;
                                 }
                         }
                         break;
                 }
 
+                case ARG_LIST_DEVICES:
+                        r = blockdev_list(BLOCKDEV_LIST_SHOW_SYMLINKS|BLOCKDEV_LIST_REQUIRE_LUKS, /* ret_devices= */ NULL, /* ret_n_devices= */ NULL);
+                        if (r < 0)
+                                return r;
+
+                        return 0;
+
                 case '?':
                         return -EINVAL;
 
                 default:
                         assert_not_reached();
                 }
-        }
 
         if (argc > optind+1)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
@@ -630,6 +669,10 @@ static int parse_argv(int argc, char *argv[]) {
                                                "When both enrolling and unlocking with FIDO2 tokens, automatic discovery is unsupported. "
                                                "Please specify device paths for enrolling and unlocking respectively.");
 
+                if (!arg_fido2_parameters_in_header && !arg_fido2_salt_file)
+                        return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
+                                               "FIDO2 parameters' storage in the LUKS2 header was disabled, but no salt file provided, refusing.");
+
                 if (!arg_fido2_device) {
                         r = fido2_find_device_auto(&arg_fido2_device);
                         if (r < 0)
@@ -654,16 +697,12 @@ static int parse_argv(int argc, char *argv[]) {
                         arg_tpm2_public_key_pcr_mask = INDEX_TO_MASK(uint32_t, TPM2_PCR_KERNEL_BOOT);
                 }
 
-                if (auto_hash_pcr_values && !arg_tpm2_pcrlock) { /* Only lock to PCR 7 by default if no pcrlock policy is around (which is a better replacement) */
-                        assert(arg_tpm2_n_hash_pcr_values == 0);
-
-                        if (!GREEDY_REALLOC_APPEND(
-                                            arg_tpm2_hash_pcr_values,
-                                            arg_tpm2_n_hash_pcr_values,
-                                            &TPM2_PCR_VALUE_MAKE(TPM2_PCR_INDEX_DEFAULT, /* hash= */ 0, /* value= */ {}),
-                                            1))
-                                return log_oom();
-                }
+                if (arg_tpm2_n_hash_pcr_values == 0 &&
+                    !arg_tpm2_pin &&
+                    arg_tpm2_public_key_pcr_mask == 0 &&
+                    !arg_tpm2_pcrlock)
+                        log_notice("Notice: enrolling TPM2 with an empty policy, i.e. without any state or access restrictions.\n"
+                                   "Use --tpm2-public-key=, --tpm2-pcrlock=, --tpm2-with-pin= or --tpm2-pcrs= to enable one or more restrictions.");
         }
 
         return 1;
@@ -731,16 +770,12 @@ static int load_volume_key_keyfile(
 
 static int prepare_luks(
                 struct crypt_device **ret_cd,
-                void **ret_volume_key,
-                size_t *ret_volume_key_size) {
+                struct iovec *ret_volume_key) {
 
         _cleanup_(crypt_freep) struct crypt_device *cd = NULL;
-        _cleanup_(erase_and_freep) void *vk = NULL;
-        size_t vks;
         int r;
 
         assert(ret_cd);
-        assert(!ret_volume_key == !ret_volume_key_size);
 
         r = crypt_init(&cd, arg_node);
         if (r < 0)
@@ -764,28 +799,31 @@ static int prepare_luks(
         r = crypt_get_volume_key_size(cd);
         if (r <= 0)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL), "Failed to determine LUKS volume key size");
-        vks = (size_t) r;
 
-        vk = malloc(vks);
-        if (!vk)
+        _cleanup_(iovec_done_erase) struct iovec vk = {};
+
+        vk.iov_base = malloc(r);
+        if (!vk.iov_base)
                 return log_oom();
 
+        vk.iov_len = (size_t) r;
+
         switch (arg_unlock_type) {
 
         case UNLOCK_PASSWORD:
-                r = load_volume_key_password(cd, arg_node, vk, &vks);
+                r = load_volume_key_password(cd, arg_node, vk.iov_base, &vk.iov_len);
                 break;
 
         case UNLOCK_KEYFILE:
-                r = load_volume_key_keyfile(cd, vk, &vks);
+                r = load_volume_key_keyfile(cd, vk.iov_base, &vk.iov_len);
                 break;
 
         case UNLOCK_FIDO2:
-                r = load_volume_key_fido2(cd, arg_node, arg_unlock_fido2_device, vk, &vks);
+                r = load_volume_key_fido2(cd, arg_node, arg_unlock_fido2_device, vk.iov_base, &vk.iov_len);
                 break;
 
         case UNLOCK_TPM2:
-                r = load_volume_key_tpm2(cd, arg_node, arg_unlock_tpm2_device, vk, &vks);
+                r = load_volume_key_tpm2(cd, arg_node, arg_unlock_tpm2_device, vk.iov_base, &vk.iov_len);
                 break;
 
         default:
@@ -796,16 +834,14 @@ static int prepare_luks(
                 return r;
 
         *ret_cd = TAKE_PTR(cd);
-        *ret_volume_key = TAKE_PTR(vk);
-        *ret_volume_key_size = vks;
+        *ret_volume_key = TAKE_STRUCT(vk);
 
         return 0;
 }
 
 static int run(int argc, char *argv[]) {
         _cleanup_(crypt_freep) struct crypt_device *cd = NULL;
-        _cleanup_(erase_and_freep) void *vk = NULL;
-        size_t vks;
+        _cleanup_(iovec_done_erase) struct iovec vk = {};
         int slot, slot_to_wipe, r;
 
         log_setup();
@@ -815,39 +851,41 @@ static int run(int argc, char *argv[]) {
                 return r;
 
         /* A delicious drop of snake oil */
-        (void) mlockall(MCL_FUTURE);
+        (void) safe_mlockall(MCL_CURRENT|MCL_FUTURE|MCL_ONFAULT);
 
         cryptsetup_enable_logging(NULL);
 
         if (arg_enroll_type < 0)
-                r = prepare_luks(&cd, NULL, NULL); /* No need to unlock device if we don't need the volume key because we don't need to enroll anything */
+                r = prepare_luks(&cd, /* ret_volume_key= */ NULL); /* No need to unlock device if we don't need the volume key because we don't need to enroll anything */
         else
-                r = prepare_luks(&cd, &vk, &vks);
+                r = prepare_luks(&cd, &vk);
         if (r < 0)
                 return r;
 
         switch (arg_enroll_type) {
 
         case ENROLL_PASSWORD:
-                slot = enroll_password(cd, vk, vks);
+                slot = enroll_password(cd, &vk);
                 break;
 
         case ENROLL_RECOVERY:
-                slot = enroll_recovery(cd, vk, vks);
+                slot = enroll_recovery(cd, &vk);
                 break;
 
         case ENROLL_PKCS11:
-                slot = enroll_pkcs11(cd, vk, vks, arg_pkcs11_token_uri);
+                slot = enroll_pkcs11(cd, &vk, arg_pkcs11_token_uri);
                 break;
 
         case ENROLL_FIDO2:
-                slot = enroll_fido2(cd, vk, vks, arg_fido2_device, arg_fido2_lock_with, arg_fido2_cred_alg);
+                slot = enroll_fido2(cd, &vk, arg_fido2_device, arg_fido2_lock_with, arg_fido2_cred_alg, arg_fido2_salt_file, arg_fido2_parameters_in_header);
                 break;
 
         case ENROLL_TPM2:
-                slot = enroll_tpm2(cd, vk, vks, arg_tpm2_device, arg_tpm2_seal_key_handle, arg_tpm2_device_key, arg_tpm2_hash_pcr_values, arg_tpm2_n_hash_pcr_values, arg_tpm2_public_key, arg_tpm2_load_public_key, arg_tpm2_public_key_pcr_mask, arg_tpm2_signature, arg_tpm2_pin, arg_tpm2_pcrlock, &slot_to_wipe);
+                slot = enroll_tpm2(cd, &vk, arg_tpm2_device, arg_tpm2_seal_key_handle, arg_tpm2_device_key, arg_tpm2_hash_pcr_values, arg_tpm2_n_hash_pcr_values, arg_tpm2_public_key, arg_tpm2_load_public_key, arg_tpm2_public_key_pcr_mask, arg_tpm2_signature, arg_tpm2_pin, arg_tpm2_pcrlock, &slot_to_wipe);
 
                 if (slot >= 0 && slot_to_wipe >= 0) {
+                        assert(slot != slot_to_wipe);
+
                         /* Updating PIN on an existing enrollment */
                         r = wipe_slots(
                                         cd,
