@@ -16,8 +16,10 @@ int acquire_luks2_key(
                 const char *pin,
                 const char *pcrlock_path,
                 uint16_t primary_alg,
-                const struct iovec *key_data,
-                const struct iovec *policy_hash,
+                const struct iovec blobs[],
+                size_t n_blobs,
+                const struct iovec policy_hash[],
+                size_t n_policy_hash,
                 const struct iovec *salt,
                 const struct iovec *srk,
                 const struct iovec *pcrlock_nv,
