@@ -1,13 +1,15 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include "sd-json.h"
+
 #include "errno-util.h"
-#include "format-table.h"
 #include "hexdecoct.h"
 #include "homectl-pkcs11.h"
 #include "libcrypt-util.h"
-#include "memory-util.h"
+#include "log.h"
 #include "openssl-util.h"
 #include "pkcs11-util.h"
+#include "string-util.h"
 #include "strv.h"
 
 int identity_add_token_pin(sd_json_variant **v, const char *pin) {
@@ -122,9 +124,9 @@ static int add_pkcs11_encrypted_key(
                 return log_error_errno(errno_or_else(EINVAL), "Failed to UNIX hash secret key: %m");
 
         r = sd_json_buildo(&e,
-                           SD_JSON_BUILD_PAIR("uri", SD_JSON_BUILD_STRING(uri)),
-                           SD_JSON_BUILD_PAIR("data", SD_JSON_BUILD_BASE64(encrypted_key, encrypted_key_size)),
-                           SD_JSON_BUILD_PAIR("hashedPassword", SD_JSON_BUILD_STRING(hashed)));
+                           SD_JSON_BUILD_PAIR_STRING("uri", uri),
+                           SD_JSON_BUILD_PAIR_BASE64("data", encrypted_key, encrypted_key_size),
+                           SD_JSON_BUILD_PAIR_STRING("hashedPassword", hashed));
         if (r < 0)
                 return log_error_errno(r, "Failed to build encrypted JSON key object: %m");
 
@@ -133,7 +135,7 @@ static int add_pkcs11_encrypted_key(
 
         r = sd_json_variant_append_array(&l, e);
         if (r < 0)
-                return log_error_errno(r, "Failed append PKCS#11 encrypted key: %m");
+                return log_error_errno(r, "Failed to append PKCS#11 encrypted key: %m");
 
         r = sd_json_variant_set_field(&w, "pkcs11EncryptedKey", l);
         if (r < 0)
@@ -146,7 +148,10 @@ static int add_pkcs11_encrypted_key(
         return 0;
 }
 
+#endif
+
 int identity_add_pkcs11_key_data(sd_json_variant **v, const char *uri) {
+#if HAVE_P11KIT
         _cleanup_(erase_and_freep) void *decrypted_key = NULL, *saved_key = NULL;
         _cleanup_(erase_and_freep) char *pin = NULL;
         size_t decrypted_key_size, saved_key_size;
@@ -192,6 +197,7 @@ int identity_add_pkcs11_key_data(sd_json_variant **v, const char *uri) {
                 return r;
 
         return 0;
-}
-
+#else
+        return log_error_errno(SYNTHETIC_ERRNO(EOPNOTSUPP), "PKCS#11 tokens not supported on this build.");
 #endif
+}
