@@ -1,12 +1,12 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "fuzz.h"
-#include "fuzz-journald.h"
+#include "fuzz-journald-util.h"
 #include "journald-native.h"
 
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         fuzz_setup_logging();
 
-        fuzz_journald_processing_function(data, size, server_process_native_message);
+        fuzz_journald_processing_function(data, size, manager_process_native_message);
         return 0;
 }
