@@ -1,33 +1,35 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <fcntl.h>
-#include <sys/epoll.h>
 #include <sys/mman.h>
 #include <sys/socket.h>
 #include <unistd.h>
 
 #include "sd-device.h"
+#include "sd-event.h"
 #include "sd-messages.h"
 
 #include "alloc-util.h"
 #include "device-util.h"
+#include "errno-util.h"
 #include "escape.h"
 #include "fd-util.h"
 #include "format-util.h"
-#include "fs-util.h"
 #include "iovec-util.h"
 #include "journal-internal.h"
 #include "journald-kmsg.h"
-#include "journald-server.h"
+#include "journald-manager.h"
+#include "journald-sync.h"
 #include "journald-syslog.h"
 #include "log.h"
+#include "log-ratelimit.h"
 #include "parse-util.h"
 #include "process-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
 
-void server_forward_kmsg(
-                Server *s,
+void manager_forward_kmsg(
+                Manager *m,
                 int priority,
                 const char *identifier,
                 const char *message,
@@ -39,15 +41,15 @@ void server_forward_kmsg(
              header_pid[STRLEN("[]: ") + DECIMAL_STR_MAX(pid_t) + 1];
         size_t n = 0;
 
-        assert(s);
+        assert(m);
         assert(priority >= 0);
         assert(priority <= 999);
         assert(message);
 
-        if (_unlikely_(LOG_PRI(priority) > s->max_level_kmsg))
+        if (_unlikely_(LOG_PRI(priority) > m->config.max_level_kmsg))
                 return;
 
-        if (_unlikely_(s->dev_kmsg_fd < 0))
+        if (_unlikely_(m->dev_kmsg_fd < 0))
                 return;
 
         /* Never allow messages with kernel facility to be written to
@@ -79,43 +81,39 @@ void server_forward_kmsg(
         iovec[n++] = IOVEC_MAKE_STRING(message);
         iovec[n++] = IOVEC_MAKE_STRING("\n");
 
-        if (writev(s->dev_kmsg_fd, iovec, n) < 0)
+        if (writev(m->dev_kmsg_fd, iovec, n) < 0)
                 log_debug_errno(errno, "Failed to write to /dev/kmsg for logging, ignoring: %m");
 }
 
-static bool is_us(const char *identifier, const char *pid) {
-        pid_t pid_num;
-
-        if (!identifier || !pid)
+static bool is_us(const char *identifier, pid_t pid) {
+        if (!identifier || !pid_is_valid(pid))
                 return false;
 
-        if (parse_pid(pid, &pid_num) < 0)
-                return false;
-
-        return pid_num == getpid_cached() &&
+        return pid == getpid_cached() &&
                streq(identifier, program_invocation_short_name);
 }
 
-void dev_kmsg_record(Server *s, char *p, size_t l) {
+void dev_kmsg_record(Manager *m, char *p, size_t l) {
 
-        _cleanup_free_ char *message = NULL, *syslog_pid = NULL, *syslog_identifier = NULL, *identifier = NULL, *pid = NULL;
+        _cleanup_free_ char *message = NULL, *syslog_identifier = NULL;
         struct iovec iovec[N_IOVEC_META_FIELDS + 7 + N_IOVEC_KERNEL_FIELDS + 2 + N_IOVEC_UDEV_FIELDS];
         char *kernel_device = NULL;
         unsigned long long usec;
         size_t n = 0, z = 0, j;
         int priority, r;
-        char *e, *f, *k;
+        char *e, *k, syslog_pid[STRLEN("SYSLOG_PID=") + DECIMAL_STR_MAX(pid_t)];
         uint64_t serial;
         size_t pl;
         int saved_log_max_level = INT_MAX;
         ClientContext *c = NULL;
 
-        assert(s);
+        assert(m);
         assert(p);
 
         if (l <= 0)
                 return;
 
+        /* syslog prefix including priority and facility */
         e = memchr(p, ',', l);
         if (!e)
                 return;
@@ -125,9 +123,10 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
         if (r < 0 || priority < 0 || priority > 999)
                 return;
 
-        if (s->forward_to_kmsg && LOG_FAC(priority) != LOG_KERN)
+        if (m->config.forward_to_kmsg && LOG_FAC(priority) != LOG_KERN)
                 return;
 
+        /* seqnum */
         l -= (e - p) + 1;
         p = e + 1;
         e = memchr(p, ',', l);
@@ -139,43 +138,47 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
         if (r < 0)
                 return;
 
-        if (s->kernel_seqnum) {
+        if (m->kernel_seqnum) {
                 /* We already read this one? */
-                if (serial < *s->kernel_seqnum)
+                if (serial < *m->kernel_seqnum)
                         return;
 
                 /* Did we lose any? */
-                if (serial > *s->kernel_seqnum)
-                        server_driver_message(s, 0,
-                                              "MESSAGE_ID=" SD_MESSAGE_JOURNAL_MISSED_STR,
-                                              LOG_MESSAGE("Missed %"PRIu64" kernel messages",
-                                                          serial - *s->kernel_seqnum),
-                                              NULL);
+                if (serial > *m->kernel_seqnum)
+                        manager_driver_message(m, 0,
+                                               LOG_MESSAGE_ID(SD_MESSAGE_JOURNAL_MISSED_STR),
+                                               LOG_MESSAGE("Missed %"PRIu64" kernel messages",
+                                                           serial - *m->kernel_seqnum));
 
                 /* Make sure we never read this one again. Note that
                  * we always store the next message serial we expect
                  * here, simply because this makes handling the first
                  * message with serial 0 easy. */
-                *s->kernel_seqnum = serial + 1;
+                *m->kernel_seqnum = serial + 1;
         }
 
+        /* CLOCK_BOOTTIME timestamp */
         l -= (e - p) + 1;
         p = e + 1;
-        f = memchr(p, ';', l);
-        if (!f)
-                return;
-        /* Kernel 3.6 has the flags field, kernel 3.5 lacks that */
         e = memchr(p, ',', l);
-        if (!e || f < e)
-                e = f;
+        if (!e)
+                return;
         *e = 0;
 
         r = safe_atollu(p, &usec);
         if (r < 0)
                 return;
 
-        l -= (f - p) + 1;
-        p = f + 1;
+        /* ignore flags and any other fields, and find the beginning of the message */
+        l -= (e - p) + 1;
+        p = e + 1;
+        e = memchr(p, ';', l);
+        if (!e)
+                return;
+
+        /* find the end of the message */
+        l -= (e - p) + 1;
+        p = e + 1;
         e = memchr(p, '\n', l);
         if (!e)
                 return;
@@ -186,7 +189,7 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
         k = e + 1;
 
         for (j = 0; l > 0 && j < N_IOVEC_KERNEL_FIELDS; j++) {
-                char *m;
+                char *mm;
                 /* Metadata fields attached */
 
                 if (*k != ' ')
@@ -200,13 +203,13 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
 
                 *e = 0;
 
-                if (cunescape_length_with_prefix(k, e - k, "_KERNEL_", UNESCAPE_RELAX, &m) < 0)
+                if (cunescape_length_with_prefix(k, e - k, "_KERNEL_", UNESCAPE_RELAX, &mm) < 0)
                         break;
 
-                if (startswith(m, "_KERNEL_DEVICE="))
-                        kernel_device = m + 15;
+                if (startswith(mm, "_KERNEL_DEVICE="))
+                        kernel_device = mm + 15;
 
-                iovec[n++] = IOVEC_MAKE_STRING(m);
+                iovec[n++] = IOVEC_MAKE_STRING(mm);
                 z++;
 
                 l -= (e - k) + 1;
@@ -276,16 +279,19 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
         if (LOG_FAC(priority) == LOG_KERN)
                 iovec[n++] = IOVEC_MAKE_STRING("SYSLOG_IDENTIFIER=kernel");
         else {
+                _cleanup_free_ char *identifier = NULL;
+                pid_t pid;
+
                 pl -= syslog_parse_identifier((const char**) &p, &identifier, &pid);
 
                 /* Avoid logging any new messages when we're processing messages generated by ourselves via
                  * log_info() and friends to avoid infinite loops. */
                 if (is_us(identifier, pid)) {
-                        if (!ratelimit_below(&s->kmsg_own_ratelimit))
+                        if (!ratelimit_below(&m->kmsg_own_ratelimit))
                                 return;
 
                         saved_log_max_level = log_get_max_level();
-                        c = s->my_context;
+                        c = m->my_context;
                         log_set_max_level(LOG_NULL);
                 }
 
@@ -295,85 +301,77 @@ void dev_kmsg_record(Server *s, char *p, size_t l) {
                                 iovec[n++] = IOVEC_MAKE_STRING(syslog_identifier);
                 }
 
-                if (pid) {
-                        syslog_pid = strjoin("SYSLOG_PID=", pid);
-                        if (syslog_pid)
-                                iovec[n++] = IOVEC_MAKE_STRING(syslog_pid);
+                if (pid_is_valid(pid)) {
+                        xsprintf(syslog_pid, "SYSLOG_PID="PID_FMT, pid);
+                        iovec[n++] = IOVEC_MAKE_STRING(syslog_pid);
                 }
         }
 
         if (cunescape_length_with_prefix(p, pl, "MESSAGE=", UNESCAPE_RELAX, &message) >= 0)
                 iovec[n++] = IOVEC_MAKE_STRING(message);
 
-        server_dispatch_message(s, iovec, n, ELEMENTSOF(iovec), c, NULL, priority, 0);
+        manager_dispatch_message(m, iovec, n, ELEMENTSOF(iovec), c, NULL, priority, 0);
 
         if (saved_log_max_level != INT_MAX)
                 log_set_max_level(saved_log_max_level);
 
+        m->dev_kmsg_timestamp = usec;
+        sync_req_revalidate_by_timestamp(m);
+
 finish:
         for (j = 0; j < z; j++)
                 free(iovec[j].iov_base);
 }
 
-static int server_read_dev_kmsg(Server *s) {
+static int manager_read_dev_kmsg(Manager *m) {
         char buffer[8192+1]; /* the kernel-side limit per record is 8K currently */
         ssize_t l;
 
-        assert(s);
-        assert(s->dev_kmsg_fd >= 0);
+        assert(m);
+        assert(m->dev_kmsg_fd >= 0);
+        assert(m->config.read_kmsg);
 
-        l = read(s->dev_kmsg_fd, buffer, sizeof(buffer) - 1);
+        l = read(m->dev_kmsg_fd, buffer, sizeof(buffer) - 1);
         if (l == 0)
                 return 0;
         if (l < 0) {
-                /* Old kernels which don't allow reading from /dev/kmsg return EINVAL when we try. So handle
-                 * this cleanly, but don't try to ever read from it again. */
-                if (errno == EINVAL) {
-                        s->dev_kmsg_event_source = sd_event_source_unref(s->dev_kmsg_event_source);
-                        s->dev_kmsg_readable = false;
-                        return 0;
-                }
-
                 if (ERRNO_IS_TRANSIENT(errno) || errno == EPIPE)
                         return 0;
 
                 return log_ratelimit_error_errno(errno, JOURNAL_LOG_RATELIMIT, "Failed to read from /dev/kmsg: %m");
         }
 
-        dev_kmsg_record(s, buffer, l);
+        dev_kmsg_record(m, buffer, l);
         return 1;
 }
 
-int server_flush_dev_kmsg(Server *s) {
+int manager_flush_dev_kmsg(Manager *m) {
         int r;
 
-        assert(s);
+        assert(m);
 
-        if (s->dev_kmsg_fd < 0)
+        if (m->dev_kmsg_fd < 0)
                 return 0;
 
-        if (!s->dev_kmsg_readable)
+        if (!m->config.read_kmsg)
                 return 0;
 
         log_debug("Flushing /dev/kmsg...");
 
         for (;;) {
-                r = server_read_dev_kmsg(s);
-                if (r < 0)
+                r = manager_read_dev_kmsg(m);
+                if (r <= 0)
                         return r;
-
-                if (r == 0)
-                        break;
         }
 
         return 0;
 }
 
 static int dispatch_dev_kmsg(sd_event_source *es, int fd, uint32_t revents, void *userdata) {
-        Server *s = ASSERT_PTR(userdata);
+        Manager *m = ASSERT_PTR(userdata);
 
         assert(es);
-        assert(fd == s->dev_kmsg_fd);
+        assert(fd == m->dev_kmsg_fd);
 
         if (revents & EPOLLERR)
                 log_ratelimit_warning(JOURNAL_LOG_RATELIMIT,
@@ -382,67 +380,110 @@ static int dispatch_dev_kmsg(sd_event_source *es, int fd, uint32_t revents, void
         if (!(revents & EPOLLIN))
                 log_error("Got invalid event from epoll for /dev/kmsg: %"PRIx32, revents);
 
-        return server_read_dev_kmsg(s);
+        return manager_read_dev_kmsg(m);
 }
 
-int server_open_dev_kmsg(Server *s) {
+int manager_open_dev_kmsg(Manager *m) {
         int r;
 
-        assert(s);
+        assert(m);
+        assert(m->dev_kmsg_fd < 0);
+        assert(!m->dev_kmsg_event_source);
 
-        mode_t mode = O_CLOEXEC|O_NONBLOCK|O_NOCTTY|
-                (s->read_kmsg ? O_RDWR : O_WRONLY);
+        mode_t mode = O_CLOEXEC|O_NONBLOCK|O_NOCTTY|(m->config.read_kmsg ? O_RDWR : O_WRONLY);
 
-        s->dev_kmsg_fd = open("/dev/kmsg", mode);
-        if (s->dev_kmsg_fd < 0) {
+        _cleanup_close_ int fd = open("/dev/kmsg", mode);
+        if (fd < 0) {
                 log_full_errno(errno == ENOENT ? LOG_DEBUG : LOG_WARNING,
                                errno, "Failed to open /dev/kmsg for %s access, ignoring: %m", accmode_to_string(mode));
                 return 0;
         }
 
-        if (!s->read_kmsg)
+        if (!m->config.read_kmsg) {
+                m->dev_kmsg_fd = TAKE_FD(fd);
                 return 0;
-
-        r = sd_event_add_io(s->event, &s->dev_kmsg_event_source, s->dev_kmsg_fd, EPOLLIN, dispatch_dev_kmsg, s);
-        if (r == -EPERM) { /* This will fail with EPERM on older kernels where /dev/kmsg is not readable. */
-                log_debug_errno(r, "Not reading from /dev/kmsg since that's not supported, apparently.");
-                r = 0;
-                goto finish;
-        }
-        if (r < 0) {
-                log_error_errno(r, "Failed to add /dev/kmsg fd to event loop: %m");
-                goto finish;
         }
 
-        r = sd_event_source_set_priority(s->dev_kmsg_event_source, SD_EVENT_PRIORITY_IMPORTANT+10);
-        if (r < 0) {
-                log_error_errno(r, "Failed to adjust priority of kmsg event source: %m");
-                goto finish;
-        }
+        _cleanup_(sd_event_source_unrefp) sd_event_source *es = NULL;
+        r = sd_event_add_io(m->event, &es, fd, EPOLLIN, dispatch_dev_kmsg, m);
+        if (r < 0)
+                return log_error_errno(r, "Failed to add /dev/kmsg fd to event loop: %m");
 
-        s->dev_kmsg_readable = true;
+        r = sd_event_source_set_priority(es, SD_EVENT_PRIORITY_NORMAL+5);
+        if (r < 0)
+                return log_error_errno(r, "Failed to adjust priority of kmsg event source: %m");
+
+        m->dev_kmsg_fd = TAKE_FD(fd);
+        m->dev_kmsg_event_source = TAKE_PTR(es);
         return 0;
-
-finish:
-        s->dev_kmsg_event_source = sd_event_source_unref(s->dev_kmsg_event_source);
-        s->dev_kmsg_fd = safe_close(s->dev_kmsg_fd);
-        return r;
 }
 
-int server_open_kernel_seqnum(Server *s) {
+int manager_open_kernel_seqnum(Manager *m) {
         int r;
 
-        assert(s);
+        assert(m);
+        assert(!m->kernel_seqnum);
 
         /* We store the seqnum we last read in an mmapped file. That way we can just use it like a variable,
          * but it is persistent and automatically flushed at reboot. */
 
-        if (!s->dev_kmsg_readable)
+        if (!m->config.read_kmsg)
                 return 0;
 
-        r = server_map_seqnum_file(s, "kernel-seqnum", sizeof(uint64_t), (void**) &s->kernel_seqnum);
+        r = manager_map_seqnum_file(m, "kernel-seqnum", sizeof(uint64_t), (void**) &m->kernel_seqnum);
         if (r < 0)
                 return log_error_errno(r, "Failed to map kernel seqnum file: %m");
 
         return 0;
 }
+
+void manager_close_kernel_seqnum(Manager *m) {
+        assert(m);
+
+        manager_unmap_seqnum_file(m->kernel_seqnum, sizeof(*m->kernel_seqnum));
+        m->kernel_seqnum = NULL;
+}
+
+static int manager_unlink_kernel_seqnum(Manager *m) {
+        assert(m);
+        assert(!m->kernel_seqnum); /* The file must not be mmap()ed. */
+
+        return manager_unlink_seqnum_file(m, "kernel-seqnum");
+}
+
+int manager_reopen_dev_kmsg(Manager *m, bool old_read_kmsg) {
+        int r;
+
+        assert(m);
+
+        /* If the fd has not yet been initialized, let's shortcut and simply open /dev/kmsg. */
+        if (m->dev_kmsg_fd < 0)
+                return manager_open_dev_kmsg(m);
+
+        if (m->config.read_kmsg == old_read_kmsg)
+                return 0; /* Setting is unchanged. */
+
+        if (!m->config.read_kmsg) {
+                /* If reading kmsg was enabled but now disable, let's flush the buffer before disabling it. */
+                m->config.read_kmsg = true;
+                (void) manager_flush_dev_kmsg(m);
+                m->config.read_kmsg = false;
+
+                /* seqnum file is not necessary anymore. Let's close it. */
+                manager_close_kernel_seqnum(m);
+
+                /* Also, unlink the file name as we will not warn some kmsg are lost when reading kmsg is
+                 * re-enabled later. */
+                manager_unlink_kernel_seqnum(m);
+        }
+
+        /* Close previously configured event source and opened file descriptor. */
+        m->dev_kmsg_event_source = sd_event_source_disable_unref(m->dev_kmsg_event_source);
+        m->dev_kmsg_fd = safe_close(m->dev_kmsg_fd);
+
+        r = manager_open_dev_kmsg(m);
+        if (r < 0)
+                return r;
+
+        return manager_open_kernel_seqnum(m);
+}
