@@ -1,10 +1,7 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
-#include <inttypes.h>
-
-#include "hashmap.h"
-#include "time-util.h"
+#include "journald-forward.h"
 
 int journal_ratelimit_test(
                 OrderedHashmap **groups_by_id,
