@@ -1,11 +1,10 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include "sd-bus.h"
+
 #include "alloc-util.h"
-#include "bus-dump.h"
 #include "bus-message.h"
 #include "env-util.h"
-#include "fd-util.h"
-#include "fileio.h"
 #include "fuzz.h"
 #include "memstream-util.h"
 
@@ -33,7 +32,11 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
         if (getenv_bool("SYSTEMD_FUZZ_OUTPUT") <= 0)
                 assert_se(g = memstream_init(&ms));
 
-        sd_bus_message_dump(m, g ?: stdout, SD_BUS_MESSAGE_DUMP_WITH_HEADER);
+        (void) sd_bus_message_dump(m, g, SD_BUS_MESSAGE_DUMP_WITH_HEADER);
+
+        _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL;
+        if (sd_bus_message_dump_json(m, SD_BUS_MESSAGE_DUMP_WITH_HEADER, &v) >= 0)
+                (void) sd_json_variant_dump(v, SD_JSON_FORMAT_PRETTY | SD_JSON_FORMAT_COLOR_AUTO, g, NULL);
 
         r = sd_bus_message_rewind(m, true);
         assert_se(r >= 0);
