@@ -7,6 +7,7 @@
 #include "sd-journal.h"
 
 #include "alloc-util.h"
+#include "argv-util.h"
 #include "chattr-util.h"
 #include "iovec-util.h"
 #include "journal-file-util.h"
@@ -16,8 +17,10 @@
 #include "parse-util.h"
 #include "random-util.h"
 #include "rm-rf.h"
-#include "tmpfile-util.h"
+#include "strv.h"
 #include "tests.h"
+#include "time-util.h"
+#include "tmpfile-util.h"
 
 /* This program tests skipping around in a multi-file journal. */
 
@@ -28,7 +31,7 @@ static JournalFile* test_open_internal(const char *name, JournalFileFlags flags)
         _cleanup_(mmap_cache_unrefp) MMapCache *m = NULL;
         JournalFile *f;
 
-        ASSERT_NOT_NULL(m = mmap_cache_new());
+        ASSERT_NOT_NULL((m = mmap_cache_new()));
         ASSERT_OK(journal_file_open(-EBADF, name, O_RDWR|O_CREAT, flags, 0644, UINT64_MAX, NULL, m, NULL, &f));
         return f;
 }
@@ -80,11 +83,11 @@ static void append_number(JournalFile *f, unsigned n, const sd_id128_t *boot_id,
         ASSERT_OK(asprintf(&p, "NUMBER=%u", n));
         iovec[n_iov++] = IOVEC_MAKE_STRING(p);
 
-        ASSERT_NOT_NULL(s = strjoin("LESS_THAN_FIVE=", yes_no(n < 5)));
+        ASSERT_NOT_NULL((s = strjoin("LESS_THAN_FIVE=", yes_no(n < 5))));
         iovec[n_iov++] = IOVEC_MAKE_STRING(s);
 
         if (boot_id) {
-                ASSERT_NOT_NULL(q = strjoin("_BOOT_ID=", SD_ID128_TO_STRING(*boot_id)));
+                ASSERT_NOT_NULL((q = strjoin("_BOOT_ID=", SD_ID128_TO_STRING(*boot_id))));
                 iovec[n_iov++] = IOVEC_MAKE_STRING(q);
         }
 
@@ -101,7 +104,7 @@ static void append_unreferenced_data(JournalFile *f, const sd_id128_t *boot_id)
         ts.monotonic = usec_sub_unsigned(previous_ts.monotonic, 10);
         ts.realtime = usec_sub_unsigned(previous_ts.realtime, 10);
 
-        ASSERT_NOT_NULL(q = strjoin("_BOOT_ID=", SD_ID128_TO_STRING(*boot_id)));
+        ASSERT_NOT_NULL((q = strjoin("_BOOT_ID=", SD_ID128_TO_STRING(*boot_id))));
         iovec = IOVEC_MAKE_STRING(q);
 
         ASSERT_ERROR(journal_file_append_entry(f, &ts, boot_id, &iovec, 1, NULL, NULL, NULL, NULL), EREMCHG);
@@ -116,7 +119,7 @@ static void test_check_number(sd_journal *j, unsigned expected) {
         ASSERT_OK(sd_journal_get_data(j, "NUMBER", &d, &l));
 
         _cleanup_free_ char *k = NULL;
-        ASSERT_NOT_NULL(k = strndup(d, l));
+        ASSERT_NOT_NULL((k = strndup(d, l)));
         printf("%s %s (expected=%u)\n", SD_ID128_TO_STRING(boot_id), k, expected);
 
         unsigned x;
@@ -142,7 +145,6 @@ static void test_check_numbers_up(sd_journal *j, unsigned count) {
                 else
                         ASSERT_OK_POSITIVE(sd_journal_previous(j));
         }
-
 }
 
 static void setup_sequential(void) {
@@ -225,7 +227,7 @@ static void mkdtemp_chdir_chattr(const char *template, char **ret) {
 
         /* Speed up things a bit on btrfs, ensuring that CoW is turned off for all files created in our
          * directory during the test run */
-        (void) chattr_path(path, FS_NOCOW_FL, FS_NOCOW_FL, NULL);
+        (void) chattr_path(path, FS_NOCOW_FL, FS_NOCOW_FL);
 
         *ret = TAKE_PTR(path);
 }
@@ -444,7 +446,7 @@ static void test_boot_id_one(void (*setup)(void), size_t n_ids_expected) {
         ASSERT_OK(sd_journal_open_directory(&j, t, SD_JOURNAL_ASSUME_IMMUTABLE));
         ASSERT_OK(journal_get_boots(
                                 j,
-                                /* advance_older = */ false, /* max_ids = */ SIZE_MAX,
+                                /* advance_older= */ false, /* max_ids= */ SIZE_MAX,
                                 &ids, &n_ids));
         ASSERT_NOT_NULL(ids);
         ASSERT_EQ(n_ids, n_ids_expected);
@@ -475,7 +477,7 @@ static void test_boot_id_one(void (*setup)(void), size_t n_ids_expected) {
 
                 ASSERT_OK(journal_get_boots(
                                         j,
-                                        /* advance_older = */ false, /* max_ids = */ i,
+                                        /* advance_older= */ false, /* max_ids= */ i,
                                         &ids_limited, &n_ids_limited));
                 ASSERT_TRUE(ids_limited || i == 0);
                 ASSERT_EQ(n_ids_limited, MIN(i, n_ids_expected));
@@ -488,7 +490,7 @@ static void test_boot_id_one(void (*setup)(void), size_t n_ids_expected) {
 
                 ASSERT_OK(journal_get_boots(
                                         j,
-                                        /* advance_older = */ true, /* max_ids = */ i,
+                                        /* advance_older= */ true, /* max_ids= */ i,
                                         &ids_limited, &n_ids_limited));
                 ASSERT_TRUE(ids_limited || i == 0);
                 ASSERT_EQ(n_ids_limited, MIN(i, n_ids_expected));
@@ -509,7 +511,7 @@ static void test_sequence_numbers_one(void) {
         uint64_t seqnum = 0;
         sd_id128_t seqnum_id;
 
-        ASSERT_NOT_NULL(m = mmap_cache_new());
+        ASSERT_NOT_NULL((m = mmap_cache_new()));
 
         mkdtemp_chdir_chattr("/var/tmp/journal-seq-XXXXXX", &t);
 
@@ -832,15 +834,15 @@ static void test_generic_array_bisect_one(size_t n, size_t num_corrupted) {
 
         log_info("/* %s(%zu, %zu) */", __func__, n, num_corrupted);
 
-        ASSERT_NOT_NULL(m = mmap_cache_new());
+        ASSERT_NOT_NULL((m = mmap_cache_new()));
 
         mkdtemp_chdir_chattr("/var/tmp/journal-seq-XXXXXX", &t);
 
         ASSERT_OK(journal_file_open(-EBADF, "test.journal", O_RDWR|O_CREAT, JOURNAL_COMPRESS, 0644,
                                     UINT64_MAX, NULL, m, NULL, &f));
 
-        ASSERT_NOT_NULL(seqnum = new0(uint64_t, n));
-        ASSERT_NOT_NULL(offset = new0(uint64_t, n));
+        ASSERT_NOT_NULL((seqnum = new0(uint64_t, n)));
+        ASSERT_NOT_NULL((offset = new0(uint64_t, n)));
 
         for (size_t i = 0; i < n; i++) {
                 append_number(f, i, NULL, seqnum + i, offset + i);
@@ -848,7 +850,7 @@ static void test_generic_array_bisect_one(size_t n, size_t num_corrupted) {
                 ASSERT_GT(offset[i], i == 0 ? 0 : offset[i-1]);
         }
 
-        ASSERT_NOT_NULL(offset_candidates = newdup(uint64_t, offset, n));
+        ASSERT_NOT_NULL((offset_candidates = newdup(uint64_t, offset, n)));
 
         verify(f, seqnum, offset_candidates, offset, n);
 
@@ -1004,7 +1006,7 @@ static void verify_entry(sd_journal *j, const TestEntry *entry) {
         ASSERT_EQ(t, entry->ts.realtime);
 
         ASSERT_OK(sd_journal_get_data(j, "NUMBER", &d, &l));
-        ASSERT_NOT_NULL(s = strndup(d, l));
+        ASSERT_NOT_NULL((s = strndup(d, l)));
         ASSERT_OK(asprintf(&e, "NUMBER=%u", entry->number));
         ASSERT_STREQ(s, e);
 
@@ -1012,7 +1014,7 @@ static void verify_entry(sd_journal *j, const TestEntry *entry) {
         e = mfree(e);
 
         ASSERT_OK(sd_journal_get_data(j, "DATA", &d, &l));
-        ASSERT_NOT_NULL(s = strndup(d, l));
+        ASSERT_NOT_NULL((s = strndup(d, l)));
         ASSERT_OK(asprintf(&e, "DATA=%u", entry->data));
         ASSERT_STREQ(s, e);
 }
@@ -1134,15 +1136,15 @@ static void append_test_entry_full(
                                         iovec, n_iovec,
                                         seqnum,
                                         seqnum_id,
-                                        /* ret_object = */ NULL,
-                                        /* ret_offset = */ NULL), EREMCHG);
+                                        /* ret_object= */ NULL,
+                                        /* ret_offset= */ NULL), EREMCHG);
 
                 ASSERT_OK(journal_file_rotate(
                                         f,
                                         m,
-                                        /* file_flags = */ JOURNAL_STRICT_ORDER,
-                                        /* compress_threshold_bytes = */ UINT64_MAX,
-                                        /* deferred_closes = */ NULL));
+                                        /* file_flags= */ JOURNAL_STRICT_ORDER,
+                                        /* compress_threshold_bytes= */ UINT64_MAX,
+                                        /* deferred_closes= */ NULL));
         }
 
         ASSERT_OK(journal_file_append_entry(
@@ -1152,8 +1154,8 @@ static void append_test_entry_full(
                                 iovec, n_iovec,
                                 seqnum,
                                 seqnum_id,
-                                /* ret_object = */ NULL,
-                                /* ret_offset = */ NULL));
+                                /* ret_object= */ NULL,
+                                /* ret_offset= */ NULL));
 
         ASSERT_NOT_NULL(GREEDY_REALLOC(*entries, *n_entries + 1));
         (*entries)[(*n_entries)++] = (TestEntry) {
@@ -1177,7 +1179,7 @@ static void append_test_entry(
                 unsigned *number,
                 unsigned data) {
 
-        append_test_entry_full(&f, NULL, entries, n_entries, seqnum, seqnum_id, boot_id, ts, number, data, /* expect_rotate = */ false);
+        append_test_entry_full(&f, NULL, entries, n_entries, seqnum, seqnum_id, boot_id, ts, number, data, /* expect_rotate= */ false);
 }
 
 TEST(seek_time) {
@@ -1189,7 +1191,7 @@ TEST(seek_time) {
 
         mkdtemp_chdir_chattr("/var/tmp/journal-seek-time-XXXXXX", &t);
 
-        ASSERT_NOT_NULL(m = mmap_cache_new());
+        ASSERT_NOT_NULL((m = mmap_cache_new()));
 
         ASSERT_OK(journal_file_open(
                                   -EBADF,
@@ -1197,10 +1199,10 @@ TEST(seek_time) {
                                   O_RDWR|O_CREAT,
                                   JOURNAL_STRICT_ORDER,
                                   0644,
-                                  /* compress_threshold_bytes = */ UINT64_MAX,
-                                  /* metrics = */ NULL,
+                                  /* compress_threshold_bytes= */ UINT64_MAX,
+                                  /* metrics= */ NULL,
                                   m,
-                                  /* template = */ NULL,
+                                  /* template= */ NULL,
                                   &f));
 
         uint64_t seqnum = 1;
@@ -1227,7 +1229,7 @@ TEST(seek_time) {
         /* realtime goes to backward */
         ts.realtime -= 100;
         ts.monotonic += 10;
-        append_test_entry_full(&f, m, &entries, &n_entries, &seqnum, &seqnum_id, &boot_id, &ts, &n, 200, /* expect_rotate = */ true);
+        append_test_entry_full(&f, m, &entries, &n_entries, &seqnum, &seqnum_id, &boot_id, &ts, &n, 200, /* expect_rotate= */ true);
 
         ts.realtime += 10;
         ts.monotonic += 10;
@@ -1303,19 +1305,19 @@ TEST(seek_time) {
                 }
 
                 FOREACH_ARRAY(e, entries, n_entries) {
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ true,  e->boot_id, e->ts.monotonic - 1, data);
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ true,  e->boot_id, e->ts.monotonic,     data);
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ true,  e->boot_id, e->ts.monotonic + 1, data);
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ false, e->boot_id, e->ts.monotonic - 1, data);
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ false, e->boot_id, e->ts.monotonic,     data);
-                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next = */ false, e->boot_id, e->ts.monotonic + 1, data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ true,  e->boot_id, e->ts.monotonic - 1, data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ true,  e->boot_id, e->ts.monotonic,     data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ true,  e->boot_id, e->ts.monotonic + 1, data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ false, e->boot_id, e->ts.monotonic - 1, data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ false, e->boot_id, e->ts.monotonic,     data);
+                        test_sd_journal_seek_monotonic_usec(j, entries, n_entries, /* next= */ false, e->boot_id, e->ts.monotonic + 1, data);
 
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ true,  e->ts.monotonic - 1, data);
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ true,  e->ts.monotonic,     data);
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ true,  e->ts.monotonic + 1, data);
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ false, e->ts.monotonic - 1, data);
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ false, e->ts.monotonic,     data);
-                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next = */ false, e->ts.monotonic + 1, data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ true,  e->ts.monotonic - 1, data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ true,  e->ts.monotonic,     data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ true,  e->ts.monotonic + 1, data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ false, e->ts.monotonic - 1, data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ false, e->ts.monotonic,     data);
+                        test_sd_journal_seek_realtime_usec(j, entries, n_entries, /* next= */ false, e->ts.monotonic + 1, data);
                 }
         }
 }
