@@ -1,7 +1,9 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
-#include "conf-parser.h"
+#include "logind-forward.h"
+#include "logind-inhibit.h"
+#include "sleep-config.h"
 
 typedef enum HandleAction {
         HANDLE_IGNORE,
@@ -30,8 +32,6 @@ typedef enum HandleAction {
         _HANDLE_ACTION_INVALID = -EINVAL,
 } HandleAction;
 
-typedef struct HandleActionData HandleActionData;
-
 typedef enum HandleActionSleepMask {
         HANDLE_SLEEP_SUSPEND_MASK                = 1U << HANDLE_SUSPEND,
         HANDLE_SLEEP_HIBERNATE_MASK              = 1U << HANDLE_HIBERNATE,
@@ -41,10 +41,6 @@ typedef enum HandleActionSleepMask {
 
 #define HANDLE_ACTION_SLEEP_MASK_DEFAULT (HANDLE_SLEEP_SUSPEND_THEN_HIBERNATE_MASK|HANDLE_SLEEP_SUSPEND_MASK|HANDLE_SLEEP_HIBERNATE_MASK)
 
-#include "logind-inhibit.h"
-#include "logind.h"
-#include "sleep-config.h"
-
 static inline bool handle_action_valid(HandleAction a) {
         return a >= 0 && a < _HANDLE_ACTION_MAX;
 }
@@ -57,7 +53,7 @@ static inline bool HANDLE_ACTION_IS_SLEEP(HandleAction a) {
         return a >= _HANDLE_ACTION_SLEEP_FIRST && a <= _HANDLE_ACTION_SLEEP_LAST;
 }
 
-struct HandleActionData {
+typedef struct HandleActionData {
         HandleAction handle;
         const char *target;
         InhibitWhat inhibit_what;
@@ -68,7 +64,7 @@ struct HandleActionData {
         const char* message_id;
         const char* message;
         const char* log_verb;
-};
+} HandleActionData;
 
 int handle_action_get_enabled_sleep_actions(HandleActionSleepMask mask, char ***ret);
 HandleAction handle_action_sleep_select(Manager *m);
@@ -76,7 +72,7 @@ HandleAction handle_action_sleep_select(Manager *m);
 int manager_handle_action(
                 Manager *m,
                 InhibitWhat inhibit_key,
-                HandleAction handle,
+                HandleAction action,
                 bool ignore_inhibited,
                 bool is_edge,
                 const char *action_seat);
@@ -86,7 +82,7 @@ const char* handle_action_verb_to_string(HandleAction h) _const_;
 const char* handle_action_to_string(HandleAction h) _const_;
 HandleAction handle_action_from_string(const char *s) _pure_;
 
-const HandleActionData* handle_action_lookup(HandleAction handle);
+const HandleActionData* handle_action_lookup(HandleAction action);
 
 CONFIG_PARSER_PROTOTYPE(config_parse_handle_action);
 
