@@ -3,18 +3,14 @@
 #include <endian.h>
 #include <fcntl.h>
 #include <pwd.h>
-#include <security/_pam_macros.h>
-#include <security/pam_ext.h>
 #include <security/pam_misc.h>
-#include <security/pam_modules.h>
-#include <security/pam_modutil.h>
 #include <sys/file.h>
-#include "time-util.h"
 #include <sys/stat.h>
 #include <sys/sysmacros.h>
 #include <unistd.h>
 
 #include "sd-bus.h"
+#include "sd-daemon.h"
 #include "sd-varlink.h"
 
 #include "alloc-util.h"
@@ -52,6 +48,7 @@
 #include "string-util.h"
 #include "strv.h"
 #include "terminal-util.h"
+#include "time-util.h"
 #include "tmpfile-util.h"
 #include "user-util.h"
 #include "userdb.h"
@@ -100,7 +97,7 @@ static int parse_caps(
                 if (!caps)
                         continue;
 
-                if (*caps == UINT64_MAX)
+                if (*caps == CAP_MASK_UNSET)
                         b = subtract ? all_capabilities() : 0;
                 else
                         b = *caps;
@@ -770,14 +767,14 @@ static int apply_user_record_settings(
 
         uint64_t a, b;
         a = user_record_capability_ambient_set(ur);
-        if (a == UINT64_MAX)
+        if (a == CAP_MASK_UNSET)
                 a = default_capability_ambient_set;
 
         b = user_record_capability_bounding_set(ur);
-        if (b == UINT64_MAX)
+        if (b == CAP_MASK_UNSET)
                 b = default_capability_bounding_set;
 
-        if (a != UINT64_MAX && a != 0) {
+        if (a != CAP_MASK_UNSET && a != 0) {
                 a &= b;
 
                 r = capability_ambient_set_apply(a, /* also_inherit= */ true);
@@ -786,7 +783,7 @@ static int apply_user_record_settings(
                                          "Failed to set ambient capabilities, ignoring: %m");
         }
 
-        if (b != UINT64_MAX && !cap_test_all(b)) {
+        if (b != CAP_MASK_UNSET && !cap_test_all(b)) {
                 r = capability_bounding_set_drop(b, /* right_now= */ false);
                 if (r < 0)
                         pam_syslog_errno(handle, LOG_ERR, r,
@@ -808,7 +805,7 @@ static uint64_t pick_default_capability_ambient_set(
 
         return ur &&
                 user_record_disposition(ur) == USER_REGULAR &&
-                (streq_ptr(service, "systemd-user") || !isempty(seat)) ? (UINT64_C(1) << CAP_WAKE_ALARM) : UINT64_MAX;
+                (streq_ptr(service, "systemd-user") || !isempty(seat)) ? (UINT64_C(1) << CAP_WAKE_ALARM) : CAP_MASK_UNSET;
 }
 
 typedef struct SessionContext {
@@ -927,12 +924,14 @@ static void session_context_mangle(
         assert(c);
         assert(ur);
 
+        /* The session class can be overridden via the PAM environment, and we try to honor that selection. */
         if (streq_ptr(c->service, "systemd-user")) {
                 /* If we detect that we are running in the "systemd-user" PAM stack, then let's patch the class to
                  * 'manager' if not set, simply for robustness reasons. */
                 c->type = "unspecified";
-                c->class = IN_SET(user_record_disposition(ur), USER_INTRINSIC, USER_SYSTEM, USER_DYNAMIC) ?
-                        "manager-early" : "manager";
+                if (isempty(c->class))
+                        c->class = IN_SET(user_record_disposition(ur), USER_INTRINSIC, USER_SYSTEM, USER_DYNAMIC) ?
+                                "manager-early" : "manager";
                 c->tty = NULL;
 
         } else if (c->tty && strchr(c->tty, ':')) {
@@ -948,14 +947,16 @@ static void session_context_mangle(
                  * (as they otherwise even try to update it!) — but cron doesn't actually allocate a TTY for its forked
                  * off processes.) */
                 c->type = "unspecified";
-                c->class = "background";
+                if (isempty(c->class))
+                        c->class = "background";
                 c->tty = NULL;
 
         } else if (streq_ptr(c->tty, "ssh")) {
                 /* ssh has been setting PAM_TTY to "ssh" (for the same reason as cron does this, see above. For further
                  * details look for "PAM_TTY_KLUDGE" in the openssh sources). */
                 c->type = "tty";
-                c->class = "user";
+                if (isempty(c->class))
+                        c->class = "user";
                 c->tty = NULL; /* This one is particularly sad, as this means that ssh sessions — even though
                                * usually associated with a pty — won't be tracked by their tty in
                                * logind. This is because ssh does the PAM session registration early for new
@@ -1154,7 +1155,7 @@ static int register_session(
                                         JSON_BUILD_PAIR_STRING_NON_EMPTY("RemoteHost", c->remote_host));
                         if (r < 0)
                                 return pam_syslog_errno(handle, LOG_ERR, r,
-                                                        "Failed to register session: %s", error_id);
+                                                        "Failed to issue io.systemd.Login.CreateSession varlink call: %m");
                         if (streq_ptr(error_id, "io.systemd.Login.AlreadySessionMember")) {
                                 /* We are already in a session, don't do anything */
                                 pam_debug_syslog(handle, debug, "Not creating session: %s", error_id);
@@ -1163,7 +1164,7 @@ static int register_session(
                         }
                         if (error_id)
                                 return pam_syslog_errno(handle, LOG_ERR, sd_varlink_error_to_errno(error_id, vreply),
-                                                        "Failed to issue CreateSession() varlink call: %s", error_id);
+                                                        "Varlink call io.systemd.Login.CreateSession failed: %s", error_id);
 
                         struct {
                                 const char *id;
@@ -1219,7 +1220,7 @@ static int register_session(
                                 handle,
                                 ur,
                                 c,
-                                /* avoid_pidfd = */ false,
+                                /* avoid_pidfd= */ false,
                                 &m);
                 if (r < 0)
                         return pam_bus_log_create_error(handle, r);
@@ -1236,7 +1237,7 @@ static int register_session(
                                                    handle,
                                                    ur,
                                                    c,
-                                                   /* avoid_pidfd = */ true,
+                                                   /* avoid_pidfd= */ true,
                                                    &m);
                         if (r < 0)
                                 return pam_bus_log_create_error(handle, r);
@@ -1257,19 +1258,35 @@ static int register_session(
                         return PAM_SESSION_ERR;
                 }
 
+                int session_fd;
                 r = sd_bus_message_read(
                                 reply,
                                 "soshusub",
                                 &id,
                                 &object_path,
                                 &runtime_path,
-                                /* session_fd = */ NULL,
+                                &session_fd,
                                 &original_uid,
                                 &real_seat,
                                 &real_vtnr,
                                 &existing);
                 if (r < 0)
                         return pam_bus_log_parse_error(handle, r);
+
+                /* Since v258, logind fully relies on pidfd to monitor the lifetime of the session leader
+                 * process and returns a dummy session_fd (no longer a fifo). However because logind cannot
+                 * be restarted (known long-standing issue), we must still be prepared to receive a fifo fd
+                 * from a running logind older than v258. */
+                if (sd_is_fifo(session_fd, NULL) > 0) {
+                        _cleanup_close_ int fd = fcntl(session_fd, F_DUPFD_CLOEXEC, 3);
+                        if (fd < 0)
+                                return pam_syslog_errno(handle, LOG_ERR, errno, "Failed to dup session fd: %m");
+
+                        r = pam_set_data(handle, "systemd.session-fd", FD_TO_PTR(fd), NULL);
+                        if (r != PAM_SUCCESS)
+                                return pam_syslog_pam_error(handle, LOG_ERR, r, "Failed to install session fd: @PAMERR@");
+                        TAKE_FD(fd);
+                }
         }
 
         pam_debug_syslog(handle, debug,
@@ -1731,9 +1748,13 @@ _public_ PAM_EXTERN int pam_sm_open_session(
 
         assert(handle);
 
+        r = dlopen_libpam();
+        if (r < 0)
+                return PAM_SERVICE_ERR;
+
         pam_log_setup();
 
-        uint64_t default_capability_bounding_set = UINT64_MAX, default_capability_ambient_set = UINT64_MAX;
+        uint64_t default_capability_bounding_set = CAP_MASK_UNSET, default_capability_ambient_set = CAP_MASK_UNSET;
         const char *class_pam = NULL, *type_pam = NULL, *desktop_pam = NULL, *area_pam = NULL;
         bool debug = false;
         if (parse_argv(handle,
@@ -1798,7 +1819,7 @@ _public_ PAM_EXTERN int pam_sm_open_session(
         if (r != PAM_SUCCESS)
                 return r;
 
-        if (default_capability_ambient_set == UINT64_MAX)
+        if (default_capability_ambient_set == CAP_MASK_UNSET)
                 default_capability_ambient_set = pick_default_capability_ambient_set(ur, c.service, c.seat);
 
         r = apply_user_record_settings(handle, ur, debug, default_capability_bounding_set, default_capability_ambient_set);
@@ -1862,10 +1883,11 @@ _public_ PAM_EXTERN int pam_sm_close_session(
                                         &error_id,
                                         SD_JSON_BUILD_PAIR_STRING("Id", id));
                         if (r < 0)
-                                return pam_syslog_errno(handle, LOG_ERR, r, "Failed to register session: %s", error_id);
+                                return pam_syslog_errno(handle, LOG_ERR, r,
+                                                        "Failed to issue io.systemd.Login.ReleaseSession varlink call: %m");
                         if (error_id)
                                 return pam_syslog_errno(handle, LOG_ERR, sd_varlink_error_to_errno(error_id, vreply),
-                                                        "Failed to issue ReleaseSession() varlink call: %s", error_id);
+                                                        "Varlink call io.systemd.Login.ReleaseSession failed: %s", error_id);
 
                         done = true;
                 }
