@@ -505,7 +505,7 @@ static int parse_argv(int argc, char *argv[]) {
 
                 _cleanup_free_ char *dev_bound = NULL;
                 r = fstab_filter_options(arg_mount_options, "x-systemd.device-bound\0",
-                                         /* ret_namefound = */ NULL, &dev_bound, /* ret_values = */ NULL, /* ret_filtered = */ NULL);
+                                         /* ret_namefound= */ NULL, &dev_bound, /* ret_values= */ NULL, /* ret_filtered= */ NULL);
                 if (r < 0)
                         return log_error_errno(r, "Failed to parse mount options for x-systemd.device-bound=: %m");
                 if (r > 0 && !isempty(dev_bound)) {
@@ -852,7 +852,7 @@ static int find_mount_points_by_source(const char *what, char ***ret) {
         /* Obtain all mount points with source being "what" from /proc/self/mountinfo, return value shows
          * the total number of them. */
 
-        r = libmount_parse_mountinfo(/* source = */ NULL, &table, &iter);
+        r = libmount_parse_mountinfo(/* source= */ NULL, &table, &iter);
         if (r < 0)
                 return log_error_errno(r, "Failed to parse /proc/self/mountinfo: %m");
 
@@ -860,14 +860,14 @@ static int find_mount_points_by_source(const char *what, char ***ret) {
                 struct libmnt_fs *fs;
                 const char *source, *target;
 
-                r = mnt_table_next_fs(table, iter, &fs);
+                r = sym_mnt_table_next_fs(table, iter, &fs);
                 if (r == 1)
                         break;
                 if (r < 0)
                         return log_error_errno(r, "Failed to get next entry from /proc/self/mountinfo: %m");
 
-                source = mnt_fs_get_source(fs);
-                target = mnt_fs_get_target(fs);
+                source = sym_mnt_fs_get_source(fs);
+                target = sym_mnt_fs_get_target(fs);
                 if (!source || !target)
                         continue;
 
@@ -894,7 +894,7 @@ static int find_loop_device(const char *backing_file, sd_device **ret) {
         if (r < 0)
                 return log_oom();
 
-        r = sd_device_enumerator_add_match_subsystem(e, "block", /* match = */ true);
+        r = sd_device_enumerator_add_match_subsystem(e, "block", /* match= */ true);
         if (r < 0)
                 return log_error_errno(r, "Failed to add subsystem match: %m");
 
@@ -906,7 +906,7 @@ static int find_loop_device(const char *backing_file, sd_device **ret) {
         if (r < 0)
                 return log_error_errno(r, "Failed to add sysname match: %m");
 
-        r = sd_device_enumerator_add_match_sysattr(e, "loop/backing_file", /* value = */ NULL, /* match = */ true);
+        r = sd_device_enumerator_add_match_sysattr(e, "loop/backing_file", /* value= */ NULL, /* match= */ true);
         if (r < 0)
                 return log_error_errno(r, "Failed to add sysattr match: %m");
 
