@@ -1,12 +1,15 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "af-list.h"
+#include "alloc-util.h"
 #include "daemon-util.h"
-#include "data-fd-util.h"
+#include "errno-util.h"
 #include "fd-util.h"
 #include "fileio.h"
+#include "hashmap.h"
 #include "iovec-util.h"
 #include "json-util.h"
+#include "memfd-util.h"
 #include "networkd-address.h"
 #include "networkd-json.h"
 #include "networkd-link.h"
@@ -14,6 +17,7 @@
 #include "networkd-nexthop.h"
 #include "networkd-route.h"
 #include "networkd-serialize.h"
+#include "string-util.h"
 
 int manager_serialize(Manager *manager) {
         _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL, *array = NULL;
@@ -37,7 +41,7 @@ int manager_serialize(Manager *manager) {
                 if (r < 0)
                         return r;
 
-                r = addresses_append_json(link, /* serializing = */ true, &e);
+                r = addresses_append_json(link, /* serializing= */ true, &e);
                 if (r < 0)
                         return r;
 
@@ -50,11 +54,11 @@ int manager_serialize(Manager *manager) {
         if (r < 0)
                 return r;
 
-        r = nexthops_append_json(manager, /* ifindex = */ -1, &v);
+        r = nexthops_append_json(manager, /* ifindex= */ -1, &v);
         if (r < 0)
                 return r;
 
-        r = routes_append_json(manager, /* ifindex = */ -1, &v);
+        r = routes_append_json(manager, /* ifindex= */ -1, &v);
         if (r < 0)
                 return r;
 
@@ -65,12 +69,12 @@ int manager_serialize(Manager *manager) {
         }
 
         _cleanup_free_ char *dump = NULL;
-        r = sd_json_variant_format(v, /* flags = */ 0, &dump);
+        r = sd_json_variant_format(v, /* flags= */ 0, &dump);
         if (r < 0)
                 return r;
 
         _cleanup_close_ int fd = -EBADF;
-        fd = acquire_data_fd(dump);
+        fd = memfd_new_and_seal_string("serialization", dump);
         if (fd < 0)
                 return fd;
 
@@ -443,11 +447,11 @@ int manager_deserialize(Manager *manager) {
                 return log_debug_errno(errno, "Failed to fdopen() serialization file descriptor: %m");
 
         _cleanup_(sd_json_variant_unrefp) sd_json_variant *v = NULL;
-        unsigned err_line, err_column;
+        unsigned err_line = 0, err_column = 0;
         r = sd_json_parse_file(
                         f,
-                        /* path = */ NULL,
-                        /* flags = */ 0,
+                        /* path= */ NULL,
+                        /* flags= */ 0,
                         &v,
                         &err_line,
                         &err_column);
