@@ -7,7 +7,7 @@
  * fine given that LGPL-2.1-or-later downgrades to GPL if needed.
  */
 
-#include "bpf-dlopen.h"
+#include "bpf-dlopen.h"                                 /* IWYU pragma: keep */
 
 /* libbpf is used via dlopen(), so rename symbols */
 #define bpf_object__attach_skeleton sym_bpf_object__attach_skeleton
@@ -15,4 +15,4 @@
 #define bpf_object__load_skeleton sym_bpf_object__load_skeleton
 #define bpf_object__open_skeleton sym_bpf_object__open_skeleton
 
-#include "bpf/userns-restrict/userns-restrict.skel.h"
+#include "bpf/userns-restrict/userns-restrict.skel.h"   /* IWYU pragma: export */
