@@ -6,6 +6,7 @@
 
 #include "btrfs-util.h"
 #include "chattr-util.h"
+#include "env-util.h"
 #include "errno-util.h"
 #include "fd-util.h"
 #include "fs-util.h"
@@ -13,6 +14,7 @@
 #include "log.h"
 #include "rm-rf.h"
 #include "sync-util.h"
+#include "time-util.h"
 
 static int fs_make_very_read_only(int fd) {
         struct stat st;
@@ -95,6 +97,11 @@ static int unlinkat_maybe_dir(int dirfd, const char *pathname) {
         return 0;
 }
 
+static bool need_opath(InstallFileFlags flags) {
+        /* Returns true when we need to pin the source file via an O_PATH fd */
+        return (flags & (INSTALL_FSYNC|INSTALL_FSYNC_FULL|INSTALL_SYNCFS|INSTALL_READ_ONLY)) != 0;
+}
+
 int install_file(int source_atfd, const char *source_name,
                  int target_atfd, const char *target_name,
                  InstallFileFlags flags) {
@@ -118,7 +125,7 @@ int install_file(int source_atfd, const char *source_name,
          * already in place, and only the syncing/read-only marking shall be applied. Note that with
          * target_name=NULL and flags=0 this call is a NOP */
 
-        if ((flags & (INSTALL_FSYNC|INSTALL_FSYNC_FULL|INSTALL_SYNCFS|INSTALL_READ_ONLY)) != 0) {
+        if (need_opath(flags)) {
                 _cleanup_close_ int pfd = -EBADF;
                 struct stat st;
 
@@ -137,24 +144,32 @@ int install_file(int source_atfd, const char *source_name,
                         _cleanup_close_ int regfd = -EBADF;
 
                         regfd = fd_reopen(pfd, O_RDONLY|O_CLOEXEC);
-                        if (regfd < 0)
-                                return regfd;
+                        if (regfd < 0) {
+                                if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                        return log_debug_errno(regfd, "Failed to open referenced inode: %m");
 
-                        if ((flags & (INSTALL_FSYNC_FULL|INSTALL_SYNCFS)) != 0) {
-                                /* If this is just a regular file (as oppose to a fully populated directory)
-                                 * let's downgrade INSTALL_SYNCFS to INSTALL_FSYNC_FULL, after all this is
-                                 * going to be a single inode we install */
-                                r = fsync_full(regfd);
-                                if (r < 0)
-                                        return r;
-                        } else if (flags & INSTALL_FSYNC) {
-                                if (fsync(regfd) < 0)
-                                        return -errno;
+                                log_debug_errno(regfd, "Failed to open referenced inode, ignoring: %m");
+                        } else {
+                                if ((flags & (INSTALL_FSYNC_FULL|INSTALL_SYNCFS)) != 0)
+                                        /* If this is just a regular file (as opposed to a fully populated
+                                         * directory) let's downgrade INSTALL_SYNCFS to INSTALL_FSYNC_FULL,
+                                         * after all this is going to be a single inode we install */
+                                        r = fsync_full(regfd);
+                                else if (flags & INSTALL_FSYNC)
+                                        r = RET_NERRNO(fsync(regfd));
+                                else
+                                        r = 0;
+                                if (r < 0) {
+                                        if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                                return log_debug_errno(r, "Failed to sync source inode: %m");
+
+                                        log_debug_errno(r, "Failed to sync source inode, ignoring: %m");
+                                }
+
+                                if (flags & INSTALL_READ_ONLY)
+                                        rofd = TAKE_FD(regfd);
                         }
 
-                        if (flags & INSTALL_READ_ONLY)
-                                rofd = TAKE_FD(regfd);
-
                         break;
                 }
 
@@ -162,24 +177,31 @@ int install_file(int source_atfd, const char *source_name,
                         _cleanup_close_ int dfd = -EBADF;
 
                         dfd = fd_reopen(pfd, O_RDONLY|O_DIRECTORY|O_CLOEXEC);
-                        if (dfd < 0)
-                                return dfd;
+                        if (dfd < 0) {
+                                if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                        return log_debug_errno(dfd, "Failed to open referenced inode: %m");
 
-                        if (flags & INSTALL_SYNCFS) {
-                                if (syncfs(dfd) < 0)
-                                        return -errno;
-                        } else if (flags & INSTALL_FSYNC_FULL) {
-                                r = fsync_full(dfd);
-                                if (r < 0)
-                                        return r;
-                        } else if (flags & INSTALL_FSYNC) {
-                                if (fsync(dfd) < 0)
-                                        return -errno;
+                                log_debug_errno(dfd, "Failed to open referenced inode, ignoring: %m");
+                        } else {
+                                if (flags & INSTALL_SYNCFS)
+                                        r = RET_NERRNO(syncfs(dfd));
+                                else if (flags & INSTALL_FSYNC_FULL)
+                                        r = fsync_full(dfd);
+                                else if (flags & INSTALL_FSYNC)
+                                        r = RET_NERRNO(fsync(dfd));
+                                else
+                                        r = 0;
+                                if (r < 0) {
+                                        if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                                return log_debug_errno(r, "Failed to sync source inode: %m");
+
+                                        log_debug_errno(r, "Failed to sync source inode, ignoring: %m");
+                                }
+
+                                if (flags & INSTALL_READ_ONLY)
+                                        rofd = TAKE_FD(dfd);
                         }
 
-                        if (flags & INSTALL_READ_ONLY)
-                                rofd = TAKE_FD(dfd);
-
                         break;
                 }
 
@@ -190,8 +212,12 @@ int install_file(int source_atfd, const char *source_name,
 
                         if (target_name && (flags & (INSTALL_FSYNC_FULL|INSTALL_SYNCFS)) != 0) {
                                 r = fsync_directory_of_file(pfd);
-                                if (r < 0)
-                                        return r;
+                                if (r < 0) {
+                                        if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                                return log_debug_errno(r, "Failed to sync source inode: %m");
+
+                                        log_debug_errno(r, "Failed to sync source inode, ignoring: %m");
+                                }
                         }
 
                         break;
@@ -254,8 +280,12 @@ int install_file(int source_atfd, const char *source_name,
 
         if (rofd >= 0) {
                 r = fs_make_very_read_only(rofd);
-                if (r < 0)
-                        return r;
+                if (r < 0) {
+                        if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                return log_debug_errno(r, "Failed to make inode read-only: %m");
+
+                        log_debug_errno(r, "Failed to make inode read-only, ignoring: %m");
+                }
         }
 
         if ((flags & (INSTALL_FSYNC_FULL|INSTALL_SYNCFS)) != 0) {
@@ -263,9 +293,48 @@ int install_file(int source_atfd, const char *source_name,
                         r = fsync_parent_at(target_atfd, target_name);
                 else
                         r = fsync_parent_at(source_atfd, source_name);
-                if (r < 0)
-                        return r;
+                if (r < 0) {
+                        if (!FLAGS_SET(flags, INSTALL_GRACEFUL))
+                                return log_debug_errno(r, "Failed to sync inode: %m");
+
+                        log_debug_errno(r, "Failed to sync inode, ignoring: %m");
+                }
         }
 
         return 0;
 }
+
+usec_t parse_source_date_epoch(void) {
+        static usec_t cache;
+        static bool cached = false;
+        int r;
+
+        if (cached)
+                return cache;
+
+        uint64_t t;
+        r = secure_getenv_uint64("SOURCE_DATE_EPOCH", &t);
+        if (r >= 0) {
+                if (MUL_SAFE(&cache, t, USEC_PER_SEC)) {
+                        cached = true;
+                        return cache;
+                }
+
+                r = -ERANGE;
+        }
+        if (r != -ENXIO)
+                log_debug_errno(r, "Failed to parse $SOURCE_DATE_EPOCH, ignoring: %m");
+
+        cached = true;
+        return (cache = USEC_INFINITY);
+}
+
+usec_t source_date_epoch_or_now(void) {
+        usec_t epoch;
+
+        epoch = parse_source_date_epoch();
+        if (epoch != USEC_INFINITY)
+                return epoch;
+
+        return now(CLOCK_REALTIME);
+}
