@@ -1,13 +1,14 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include <stdlib.h>
 #include <sys/mount.h>
 #include <unistd.h>
 
-#include "dirent-util.h"
 #include "fd-util.h"
 #include "fileio.h"
+#include "format-util.h"
 #include "fs-util.h"
-#include "id128-util.h"
+#include "log.h"
 #include "mkfs-util.h"
 #include "mount-util.h"
 #include "mountpoint-util.h"
@@ -18,6 +19,7 @@
 #include "stat-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
+#include "strv.h"
 #include "tmpfile-util.h"
 #include "utf8.h"
 
@@ -111,7 +113,7 @@ static int do_mcopy(const char *node, const char *root) {
         assert(root);
 
         /* Return early if there's nothing to copy. */
-        if (dir_is_empty(root, /*ignore_hidden_or_backup=*/ false))
+        if (dir_is_empty(root, /* ignore_hidden_or_backup= */ false))
                 return 0;
 
         r = find_executable("mcopy", &mcopy);
@@ -206,7 +208,7 @@ static int protofile_print_item(
          * delimiter. To work around this limitation, mkfs.xfs allows escaping whitespace by using the /
          * character (which isn't allowed in filenames and as such can be used to escape whitespace). See
          * https://lore.kernel.org/linux-xfs/20230222090303.h6tujm7y32gjhgal@andromeda/T/#m8066b3e7d62a080ee7434faac4861d944e64493b
-         * for more information.*/
+         * for more information. */
 
         if (strchr(de->d_name, ' ')) {
                 copy = strdup(de->d_name);
@@ -320,8 +322,7 @@ int make_filesystem(
                 const char *label,
                 const char *root,
                 sd_id128_t uuid,
-                bool discard,
-                bool quiet,
+                MakeFileSystemFlags flags,
                 uint64_t sector_size,
                 char *compression,
                 char *compression_level,
@@ -333,7 +334,7 @@ int make_filesystem(
         _cleanup_(unlink_and_freep) char *protofile = NULL;
         char vol_id[CONST_MAX(SD_ID128_UUID_STRING_MAX, 8U + 1U)] = {};
         int stdio_fds[3] = { -EBADF, STDERR_FILENO, STDERR_FILENO};
-        ForkFlags flags = FORK_RESET_SIGNALS|FORK_RLIMIT_NOFILE_SAFE|FORK_DEATHSIG_SIGTERM|FORK_LOG|FORK_WAIT|
+        ForkFlags fork_flags = FORK_RESET_SIGNALS|FORK_RLIMIT_NOFILE_SAFE|FORK_DEATHSIG_SIGTERM|FORK_LOG|FORK_WAIT|
                         FORK_CLOSE_ALL_FDS|FORK_REARRANGE_STDIO|FORK_REOPEN_LOG;
         int r;
 
@@ -417,12 +418,19 @@ int make_filesystem(
 
         /* When changing this conditional, also adjust the log statement below. */
         if (STR_IN_SET(fstype, "ext2", "ext3", "ext4")) {
+                const char *ext_e_opts;
+
+                /* Set hash_seed to the same value as the filesystem UUID for reproducibility */
+                ext_e_opts = strjoina(FLAGS_SET(flags, MKFS_DISCARD) ? "discard" : "nodiscard",
+                                      ",lazy_itable_init=1,hash_seed=",
+                                      vol_id);
+
                 argv = strv_new(mkfs,
                                 "-L", label,
                                 "-U", vol_id,
                                 "-I", "256",
                                 "-m", "0",
-                                "-E", discard ? "discard,lazy_itable_init=1" : "nodiscard,lazy_itable_init=1",
+                                "-E", ext_e_opts,
                                 "-b", "4096",
                                 "-T", "default");
                 if (!argv)
@@ -431,7 +439,10 @@ int make_filesystem(
                 if (root && strv_extend_many(&argv, "-d", root) < 0)
                         return log_oom();
 
-                if (quiet && strv_extend(&argv, "-q") < 0)
+                if (FLAGS_SET(flags, MKFS_QUIET) && strv_extend(&argv, "-q") < 0)
+                        return log_oom();
+
+                if (FLAGS_SET(flags, MKFS_FS_VERITY) && strv_extend_many(&argv, "-O", "verity") < 0)
                         return log_oom();
 
                 if (strv_extend(&argv, node) < 0)
@@ -439,12 +450,24 @@ int make_filesystem(
 
                 if (sector_size > 0) {
                         if (strv_extend(&env, "MKE2FS_DEVICE_SECTSIZE") < 0)
-                                        return log_oom();
+                                return log_oom();
 
                         if (strv_extendf(&env, "%"PRIu64, sector_size) < 0)
                                 return log_oom();
                 }
 
+                /* e2fsprogs supports $SOURCE_DATE_EPOCH since v1.47.1. For older versions, we need to set
+                 * $E2FSPROGS_FAKE_TIME. See the following:
+                 * https://github.com/tytso/e2fsprogs/commit/b6e2913061577ad981464e435026d71a48fd5caf
+                 * Note, $E2FSPROGS_FAKE_TIME and $SOURCE_DATE_EPOCH are mostly equivalent, except for the
+                 * 0 value handling, where $E2FSPROGS_FAKE_TIME=0 is ignored and the current time is used,
+                 * but $SOURCE_DATE_EPOCH=0 sets 1970-01-01 as the timestamp. */
+                if (!secure_getenv("E2FSPROGS_FAKE_TIME")) { /* honor $E2FSPROGS_FAKE_TIME if already set */
+                        const char *e = secure_getenv("SOURCE_DATE_EPOCH");
+                        if (e && strv_extend_strv(&env, STRV_MAKE("E2FSPROGS_FAKE_TIME", e), /* filter_duplicates= */ false) < 0)
+                                return log_oom();
+                }
+
         } else if (streq(fstype, "btrfs")) {
                 argv = strv_new(mkfs,
                                 "-L", label,
@@ -452,18 +475,37 @@ int make_filesystem(
                 if (!argv)
                         return log_oom();
 
-                if (!discard && strv_extend(&argv, "--nodiscard") < 0)
+                if (!FLAGS_SET(flags, MKFS_DISCARD) && strv_extend(&argv, "--nodiscard") < 0)
                         return log_oom();
 
                 if (root && strv_extend_many(&argv, "-r", root) < 0)
                         return log_oom();
 
-                if (quiet && strv_extend(&argv, "-q") < 0)
+                if (FLAGS_SET(flags, MKFS_QUIET) && strv_extend(&argv, "-q") < 0)
                         return log_oom();
 
+                if (compression) {
+                        if (!root)
+                                log_warning("Btrfs compression setting ignored because no files are being copied. "
+                                            "Compression= can only be applied when CopyFiles= is also specified.");
+                        else {
+                                _cleanup_free_ char *c = NULL;
+
+                                c = strdup(compression);
+                                if (!c)
+                                        return log_oom();
+
+                                if (compression_level && !strextend(&c, ":", compression_level))
+                                        return log_oom();
+
+                                if (strv_extend_many(&argv, "--compress", c) < 0)
+                                        return log_oom();
+                        }
+                }
+
                 /* mkfs.btrfs unconditionally warns about several settings changing from v5.15 onwards which
                  * isn't silenced by "-q", so let's redirect stdout to /dev/null as well. */
-                if (quiet)
+                if (FLAGS_SET(flags, MKFS_QUIET))
                         stdio_fds[1] = -EBADF;
 
                 /* mkfs.btrfs expects a sector size of at least 4k bytes. */
@@ -479,11 +521,14 @@ int make_filesystem(
                                 "-f",  /* force override, without this it doesn't seem to want to write to an empty partition */
                                 "-l", label,
                                 "-U", vol_id,
-                                "-t", one_zero(discard));
+                                "-t", one_zero(FLAGS_SET(flags, MKFS_DISCARD)));
                 if (!argv)
                         return log_oom();
 
-                if (quiet && strv_extend(&argv, "-q") < 0)
+                if (FLAGS_SET(flags, MKFS_QUIET) && strv_extend(&argv, "-q") < 0)
+                        return log_oom();
+
+                if (FLAGS_SET(flags, MKFS_FS_VERITY) && strv_extend_many(&argv, "-O", "verity") < 0)
                         return log_oom();
 
                 if (sector_size > 0) {
@@ -509,7 +554,7 @@ int make_filesystem(
                 if (!argv)
                         return log_oom();
 
-                if (!discard && strv_extend(&argv, "-K") < 0)
+                if (!FLAGS_SET(flags, MKFS_DISCARD) && strv_extend(&argv, "-K") < 0)
                         return log_oom();
 
                 if (root) {
@@ -541,7 +586,7 @@ int make_filesystem(
                                 return log_oom();
                 }
 
-                if (quiet && strv_extend(&argv, "-q") < 0)
+                if (FLAGS_SET(flags, MKFS_QUIET) && strv_extend(&argv, "-q") < 0)
                         return log_oom();
 
                 if (strv_extend(&argv, node) < 0)
@@ -568,7 +613,7 @@ int make_filesystem(
                         return log_oom();
 
                 /* mkfs.vfat does not have a --quiet option so let's redirect stdout to /dev/null instead. */
-                if (quiet)
+                if (FLAGS_SET(flags, MKFS_QUIET))
                         stdio_fds[1] = -EBADF;
 
         } else if (streq(fstype, "swap")) {
@@ -581,7 +626,7 @@ int make_filesystem(
                 if (!argv)
                         return log_oom();
 
-                if (quiet)
+                if (FLAGS_SET(flags, MKFS_QUIET))
                         stdio_fds[1] = -EBADF;
 
         } else if (streq(fstype, "squashfs")) {
@@ -601,7 +646,7 @@ int make_filesystem(
                 }
 
                 /* mksquashfs -quiet option is pretty new so let's redirect stdout to /dev/null instead. */
-                if (quiet)
+                if (FLAGS_SET(flags, MKFS_QUIET))
                         stdio_fds[1] = -EBADF;
 
         } else if (streq(fstype, "erofs")) {
@@ -610,7 +655,7 @@ int make_filesystem(
                 if (!argv)
                         return log_oom();
 
-                if (quiet && strv_extend(&argv, "--quiet") < 0)
+                if (FLAGS_SET(flags, MKFS_QUIET) && strv_extend(&argv, "--quiet") < 0)
                         return log_oom();
 
                 if (compression) {
@@ -647,7 +692,7 @@ int make_filesystem(
                         return log_error_errno(r, "Failed to stat '%s': %m", node);
 
                 if (S_ISBLK(st.st_mode))
-                        flags |= FORK_NEW_MOUNTNS;
+                        fork_flags |= FORK_NEW_MOUNTNS;
         }
 
         if (DEBUG_LOGGING) {
@@ -660,17 +705,17 @@ int make_filesystem(
         r = safe_fork_full(
                         "(mkfs)",
                         stdio_fds,
-                        /*except_fds=*/ NULL,
-                        /*n_except_fds=*/ 0,
-                        flags,
-                        /*ret_pid=*/ NULL);
+                        /* except_fds= */ NULL,
+                        /* n_except_fds= */ 0,
+                        fork_flags,
+                        /* ret_pid= */ NULL);
         if (r < 0)
                 return r;
         if (r == 0) {
                 /* Child */
 
                 STRV_FOREACH_PAIR(k, v, env)
-                        if (setenv(*k, *v, /* replace = */ true) < 0) {
+                        if (setenv(*k, *v, /* replace= */ true) < 0) {
                                 log_error_errno(r, "Failed to set %s=%s environment variable: %m", *k, *v);
                                 _exit(EXIT_FAILURE);
                         }
@@ -679,7 +724,7 @@ int make_filesystem(
                  * on unformatted free space, so let's trick it and other mkfs tools into thinking no
                  * partitions are mounted. See https://github.com/kdave/btrfs-progs/issues/640 for more
                  ° information. */
-                 if (flags & FORK_NEW_MOUNTNS)
+                 if (fork_flags & FORK_NEW_MOUNTNS)
                         (void) mount_nofollow_verbose(LOG_DEBUG, "/dev/null", "/proc/self/mounts", NULL, MS_BIND, NULL);
 
                 execvp(mkfs, argv);
