@@ -1,21 +1,21 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
-#include <endian.h>
-
 #include "alloc-util.h"
 #include "ask-password-api.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "hexdecoct.h"
+#include "log.h"
 #include "memory-util.h"
+#include "memstream-util.h"
 #include "openssl-util.h"
 #include "random-util.h"
 #include "string-util.h"
 #include "strv.h"
 
 #if HAVE_OPENSSL
-#  include <openssl/rsa.h>
 #  include <openssl/ec.h>
+#  include <openssl/rsa.h>
 
 #  if !defined(OPENSSL_NO_ENGINE) && !defined(OPENSSL_NO_DEPRECATED_3_0)
 #    include <openssl/engine.h>
@@ -54,24 +54,41 @@ DEFINE_TRIVIAL_CLEANUP_FUNC_FULL(UI_METHOD*, UI_destroy_method, NULL);
                 UNIQ_T(R, u);                                           \
         })
 
-int openssl_pkey_from_pem(const void *pem, size_t pem_size, EVP_PKEY **ret) {
+int openssl_pubkey_from_pem(const void *pem, size_t pem_size, EVP_PKEY **ret) {
         assert(pem);
         assert(ret);
 
+        if (pem_size == SIZE_MAX)
+                pem_size = strlen(pem);
+
         _cleanup_fclose_ FILE *f = NULL;
         f = fmemopen((void*) pem, pem_size, "r");
         if (!f)
                 return log_oom_debug();
 
-        _cleanup_(EVP_PKEY_freep) EVP_PKEY *pkey = PEM_read_PUBKEY(f, NULL, NULL, NULL);
+        _cleanup_(EVP_PKEY_freep) EVP_PKEY *pkey = PEM_read_PUBKEY(f, /* x= */ NULL, /* pam_password_cb= */ NULL, /* userdata= */ NULL);
         if (!pkey)
                 return log_openssl_errors("Failed to parse PEM");
 
         *ret = TAKE_PTR(pkey);
-
         return 0;
 }
 
+int openssl_pubkey_to_pem(EVP_PKEY *pkey, char **ret) {
+        assert(pkey);
+        assert(ret);
+
+        _cleanup_(memstream_done) MemStream m = {};
+        FILE *f = memstream_init(&m);
+        if (!f)
+                return -ENOMEM;
+
+        if (PEM_write_PUBKEY(f, pkey) <= 0)
+                return -EIO;
+
+        return memstream_finalize(&m, ret, /* ret_size= */ NULL);
+}
+
 /* Returns the number of bytes generated by the specified digest algorithm. This can be used only for
  * fixed-size algorithms, e.g. md5, sha1, sha256, etc. Do not use this for variable-sized digest algorithms,
  * e.g. shake128. Returns 0 on success, -EOPNOTSUPP if the algorithm is not supported, or < 0 for any other
@@ -148,7 +165,7 @@ int openssl_digest_many(
         if (!buf)
                 return log_oom_debug();
 
-        unsigned int size;
+        unsigned size;
         if (!EVP_DigestFinal_ex(ctx, buf, &size))
                 return log_openssl_errors("Failed to finalize Digest");
 
@@ -244,7 +261,7 @@ int openssl_hmac_many(
         size_t size;
         if (!EVP_MAC_final(ctx, buf, &size, digest_size))
 #else
-        unsigned int size;
+        unsigned size;
         if (!HMAC_Final(ctx, buf, &size))
 #endif
                 return log_openssl_errors("Failed to finalize HMAC");
@@ -770,29 +787,6 @@ int rsa_pkey_to_n_e(
         return 0;
 }
 
-/* Generate a new RSA key with the specified number of bits. */
-int rsa_pkey_new(size_t bits, EVP_PKEY **ret) {
-        assert(ret);
-
-        _cleanup_(EVP_PKEY_CTX_freep) EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new_id(EVP_PKEY_RSA, NULL);
-        if (!ctx)
-                return log_openssl_errors("Failed to create new EVP_PKEY_CTX");
-
-        if (EVP_PKEY_keygen_init(ctx) <= 0)
-                return log_openssl_errors("Failed to initialize EVP_PKEY_CTX");
-
-        if (EVP_PKEY_CTX_set_rsa_keygen_bits(ctx, (int) bits) <= 0)
-                return log_openssl_errors("Failed to set RSA bits to %zu", bits);
-
-        _cleanup_(EVP_PKEY_freep) EVP_PKEY *pkey = NULL;
-        if (EVP_PKEY_keygen(ctx, &pkey) <= 0)
-                return log_openssl_errors("Failed to generate ECC key");
-
-        *ret = TAKE_PTR(pkey);
-
-        return 0;
-}
-
 /* Generate ECC public key from provided curve ID and x/y points. */
 int ecc_pkey_from_curve_x_y(
                 int curve_id,
@@ -877,9 +871,8 @@ int ecc_pkey_from_curve_x_y(
         TAKE_PTR(eckey);
 #endif
 
-    *ret = TAKE_PTR(pkey);
-
-    return 0;
+        *ret = TAKE_PTR(pkey);
+        return 0;
 }
 
 int ecc_pkey_to_curve_x_y(
@@ -1090,6 +1083,8 @@ int digest_and_sign(
                 const void *data, size_t size,
                 void **ret, size_t *ret_size) {
 
+        int r;
+
         assert(privkey);
         assert(ret);
         assert(ret_size);
@@ -1107,8 +1102,13 @@ int digest_and_sign(
         if (!mdctx)
                 return log_openssl_errors("Failed to create new EVP_MD_CTX");
 
-        if (EVP_DigestSignInit(mdctx, NULL, md, NULL, privkey) != 1)
-                return log_openssl_errors("Failed to initialize signature context");
+        if (EVP_DigestSignInit(mdctx, NULL, md, NULL, privkey) != 1) {
+                /* Distro security policies often disable support for SHA-1. Let's return a recognizable
+                 * error for that case. */
+                bool invalid_digest = ERR_GET_REASON(ERR_peek_last_error()) == EVP_R_INVALID_DIGEST;
+                r = log_openssl_errors("Failed to initialize signature context");
+                return invalid_digest ? -EADDRNOTAVAIL : r;
+        }
 
         /* Determine signature size */
         size_t ss;
@@ -1127,7 +1127,88 @@ int digest_and_sign(
         return 0;
 }
 
-#  if PREFER_OPENSSL
+int pkcs7_new(X509 *certificate, EVP_PKEY *private_key, const char *hash_algorithm, PKCS7 **ret_p7, PKCS7_SIGNER_INFO **ret_si) {
+        assert(certificate);
+        assert(ret_p7);
+
+        /* This function sets up a new PKCS7 signing context. If a private key is provided, the context is
+         * set up for "in-band" signing with PKCS7_dataFinal(). If a private key is not provided, the context
+         * is set up for "out-of-band" signing, meaning the signature has to be provided by the user and
+         * copied into the signer info's "enc_digest" field. If the signing hash algorithm is not provided,
+         * SHA-256 is used. */
+
+        _cleanup_(PKCS7_freep) PKCS7 *p7 = PKCS7_new();
+        if (!p7)
+                return log_oom();
+
+        if (PKCS7_set_type(p7, NID_pkcs7_signed) == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set PKCS7 type: %s",
+                                       ERR_error_string(ERR_get_error(), NULL));
+
+        if (PKCS7_content_new(p7, NID_pkcs7_data) == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set PKCS7 content: %s",
+                                       ERR_error_string(ERR_get_error(), NULL));
+
+        if (PKCS7_add_certificate(p7, certificate) == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set PKCS7 certificate: %s",
+                                       ERR_error_string(ERR_get_error(), NULL));
+
+        int x509_pknid = 0;
+        if (X509_get_signature_info(certificate, NULL, &x509_pknid, NULL, NULL) == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to get X509 digest NID: %s",
+                                       ERR_error_string(ERR_get_error(), NULL));
+
+        const EVP_MD *md = EVP_get_digestbyname(hash_algorithm ?: "SHA256");
+        if (!md)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to get digest algorithm '%s'",
+                                       hash_algorithm ?: "SHA256");
+
+        _cleanup_(PKCS7_SIGNER_INFO_freep) PKCS7_SIGNER_INFO *si = PKCS7_SIGNER_INFO_new();
+        if (!si)
+                return log_oom();
+
+        if (private_key) {
+                if (PKCS7_SIGNER_INFO_set(si, certificate, private_key, md) <= 0)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to configure signer info: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+        } else {
+                if (ASN1_INTEGER_set(si->version, 1) == 0)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set signer info version: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+
+                if (X509_NAME_set(&si->issuer_and_serial->issuer, X509_get_issuer_name(certificate)) == 0)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set signer info issuer: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+
+                ASN1_INTEGER_free(si->issuer_and_serial->serial);
+                si->issuer_and_serial->serial = ASN1_INTEGER_dup(X509_get0_serialNumber(certificate));
+                if (!si->issuer_and_serial->serial)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set signer info serial: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+
+                if (X509_ALGOR_set0(si->digest_alg, OBJ_nid2obj(EVP_MD_type(md)), V_ASN1_NULL, NULL) == 0)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set signer info digest algorithm: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+
+                if (X509_ALGOR_set0(si->digest_enc_alg, OBJ_nid2obj(x509_pknid), V_ASN1_NULL, NULL) == 0)
+                        return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set signer info signing algorithm: %s",
+                                               ERR_error_string(ERR_get_error(), NULL));
+        }
+
+        if (PKCS7_add_signer(p7, si) == 0)
+                return log_debug_errno(SYNTHETIC_ERRNO(EIO), "Failed to set PKCS7 signer info: %s",
+                                       ERR_error_string(ERR_get_error(), NULL));
+
+        *ret_p7 = TAKE_PTR(p7);
+        if (ret_si)
+                /* We do not pass ownership here, 'si' object remains owned by 'p7' object. */
+                *ret_si = si;
+
+        TAKE_PTR(si);
+
+        return 0;
+}
+
 int string_hashsum(
                 const char *s,
                 size_t len,
@@ -1154,7 +1235,6 @@ int string_hashsum(
         *ret = TAKE_PTR(enc);
         return 0;
 }
-#  endif
 
 static int ecc_pkey_generate_volume_keys(
                 EVP_PKEY *pkey,
@@ -1335,8 +1415,8 @@ static int load_key_from_provider(
 
         _cleanup_(OSSL_STORE_closep) OSSL_STORE_CTX *store = OSSL_STORE_open(
                         private_key_uri,
-                        /*ui_method=*/ NULL,
-                        /*ui_method=*/ NULL,
+                        /* ui_method= */ NULL,
+                        /* ui_method= */ NULL,
                         /* post_process= */ NULL,
                         /* post_process_data= */ NULL);
         if (!store)
@@ -1375,7 +1455,7 @@ static int load_key_from_engine(const char *engine, const char *private_key_uri,
         if (ENGINE_init(e) == 0)
                 return log_openssl_errors("Failed to initialize signing engine '%s'", engine);
 
-        _cleanup_(EVP_PKEY_freep) EVP_PKEY *private_key = ENGINE_load_private_key(e, private_key_uri, /*ui_method=*/ NULL, /*callback_data=*/ NULL);
+        _cleanup_(EVP_PKEY_freep) EVP_PKEY *private_key = ENGINE_load_private_key(e, private_key_uri, /* ui_method= */ NULL, /* callback_data= */ NULL);
         if (!private_key)
                 return log_openssl_errors("Failed to load private key from '%s'", private_key_uri);
         REENABLE_WARNING;
@@ -1401,8 +1481,8 @@ static int openssl_ask_password_ui_read(UI *ui, UI_STRING *uis) {
 
                 req->message = UI_get0_output_string(uis);
 
-                _cleanup_(strv_freep) char **l = NULL;
-                r = ask_password_auto(req, /*until=*/ 0, ASK_PASSWORD_ACCEPT_CACHED|ASK_PASSWORD_PUSH_CACHE, &l);
+                _cleanup_strv_free_ char **l = NULL;
+                r = ask_password_auto(req, ASK_PASSWORD_ACCEPT_CACHED|ASK_PASSWORD_PUSH_CACHE, &l);
                 if (r < 0) {
                         log_error_errno(r, "Failed to query for PIN: %m");
                         return 0;
@@ -1539,8 +1619,8 @@ static int load_x509_certificate_from_provider(const char *provider, const char
 
         _cleanup_(OSSL_STORE_closep) OSSL_STORE_CTX *store = OSSL_STORE_open(
                         certificate_uri,
-                        /*ui_method=*/ NULL,
-                        /*ui_method=*/ NULL,
+                        /* ui_method= */ NULL,
+                        /* ui_method= */ NULL,
                         /* post_process= */ NULL,
                         /* post_process_data= */ NULL);
         if (!store)
@@ -1564,25 +1644,20 @@ static int load_x509_certificate_from_provider(const char *provider, const char
         return -EOPNOTSUPP;
 #endif
 }
-#endif
 
 OpenSSLAskPasswordUI* openssl_ask_password_ui_free(OpenSSLAskPasswordUI *ui) {
-#if HAVE_OPENSSL && !defined(OPENSSL_NO_UI_CONSOLE)
         if (!ui)
                 return NULL;
 
+#ifndef OPENSSL_NO_UI_CONSOLE
         assert(UI_get_default_method() == ui->method);
         UI_set_default_method(UI_OpenSSL());
         UI_destroy_method(ui->method);
-        return mfree(ui);
-#else
-        assert(ui == NULL);
-        return NULL;
 #endif
+        return mfree(ui);
 }
 
 int x509_fingerprint(X509 *cert, uint8_t buffer[static SHA256_DIGEST_SIZE]) {
-#if HAVE_OPENSSL
         _cleanup_free_ uint8_t *der = NULL;
         int dersz;
 
@@ -1594,9 +1669,6 @@ int x509_fingerprint(X509 *cert, uint8_t buffer[static SHA256_DIGEST_SIZE]) {
 
         sha256_direct(der, dersz, buffer);
         return 0;
-#else
-        return log_debug_errno(SYNTHETIC_ERRNO(EOPNOTSUPP), "OpenSSL is not supported, cannot calculate X509 fingerprint.");
-#endif
 }
 
 int openssl_load_x509_certificate(
@@ -1604,7 +1676,7 @@ int openssl_load_x509_certificate(
                 const char *certificate_source,
                 const char *certificate,
                 X509 **ret) {
-#if HAVE_OPENSSL
+
         int r;
 
         assert(certificate);
@@ -1628,9 +1700,6 @@ int openssl_load_x509_certificate(
                                 certificate_source);
 
         return 0;
-#else
-        return log_debug_errno(SYNTHETIC_ERRNO(EOPNOTSUPP), "OpenSSL is not supported, cannot load X509 certificate.");
-#endif
 }
 
 int openssl_load_private_key(
@@ -1640,18 +1709,20 @@ int openssl_load_private_key(
                 const AskPasswordRequest *request,
                 EVP_PKEY **ret_private_key,
                 OpenSSLAskPasswordUI **ret_user_interface) {
-#if HAVE_OPENSSL
+
         int r;
 
         assert(private_key);
         assert(request);
+        assert(ret_private_key);
 
         if (private_key_source_type == OPENSSL_KEY_SOURCE_FILE) {
                 r = openssl_load_private_key_from_file(private_key, ret_private_key);
                 if (r < 0)
                         return r;
 
-                *ret_user_interface = NULL;
+                if (ret_user_interface)
+                        *ret_user_interface = NULL;
         } else {
                 _cleanup_(openssl_ask_password_ui_freep) OpenSSLAskPasswordUI *ui = NULL;
                 r = openssl_ask_password_ui_new(request, &ui);
@@ -1676,15 +1747,41 @@ int openssl_load_private_key(
                                         private_key,
                                         private_key_source);
 
-                *ret_user_interface = TAKE_PTR(ui);
+                if (ret_user_interface)
+                        *ret_user_interface = TAKE_PTR(ui);
         }
 
         return 0;
-#else
-        return log_debug_errno(SYNTHETIC_ERRNO(EOPNOTSUPP), "OpenSSL is not supported, cannot load private key.");
-#endif
 }
 
+int openssl_extract_public_key(EVP_PKEY *private_key, EVP_PKEY **ret) {
+        int r;
+
+        assert(private_key);
+        assert(ret);
+
+        _cleanup_(memstream_done) MemStream m = {};
+        FILE *tf = memstream_init(&m);
+        if (!tf)
+                return -ENOMEM;
+
+        if (i2d_PUBKEY_fp(tf, private_key) != 1)
+                return -EIO;
+
+        _cleanup_(erase_and_freep) char *buf = NULL;
+        size_t len;
+        r = memstream_finalize(&m, &buf, &len);
+        if (r < 0)
+                return r;
+
+        const unsigned char *t = (unsigned char*) buf;
+        if (!d2i_PUBKEY(ret, &t, len))
+                return -EIO;
+
+        return 0;
+}
+#endif
+
 int parse_openssl_certificate_source_argument(
                 const char *argument,
                 char **certificate_source,
