@@ -1,6 +1,8 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <getopt.h>
+#include <stdlib.h>
+#include <sys/stat.h>
 
 #include "alloc-util.h"
 #include "build.h"
@@ -10,33 +12,38 @@
 #include "copy.h"
 #include "creds-util.h"
 #include "dissect-image.h"
-#include "env-util.h"
+#include "errno-util.h"
+#include "extract-word.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "format-util.h"
 #include "fs-util.h"
 #include "hashmap.h"
+#include "image-policy.h"
+#include "install-file.h"
+#include "label-util.h"
+#include "libaudit-util.h"
 #include "libcrypt-util.h"
+#include "log.h"
+#include "loop-util.h"
 #include "main-func.h"
-#include "memory-util.h"
 #include "mount-util.h"
 #include "pager.h"
 #include "parse-argument.h"
 #include "path-util.h"
 #include "pretty-print.h"
-#include "selinux-util.h"
 #include "set.h"
 #include "smack-util.h"
 #include "specifier.h"
-#include "stat-util.h"
 #include "string-util.h"
 #include "strv.h"
 #include "sync-util.h"
+#include "time-util.h"
 #include "tmpfile-util-label.h"
 #include "uid-classification.h"
 #include "uid-range.h"
 #include "user-util.h"
-#include "utf8.h"
+#include "verbs.h"
 
 typedef enum ItemType {
         ADD_USER =   'u',
@@ -106,6 +113,8 @@ STATIC_DESTRUCTOR_REGISTER(arg_image, freep);
 STATIC_DESTRUCTOR_REGISTER(arg_image_policy, image_policy_freep);
 
 typedef struct Context {
+        int audit_fd;
+
         OrderedHashmap *users, *groups;
         OrderedHashmap *todo_uids, *todo_gids;
         OrderedHashmap *members;
@@ -126,6 +135,8 @@ typedef struct Context {
 static void context_done(Context *c) {
         assert(c);
 
+        c->audit_fd = close_audit_fd(c->audit_fd);
+
         ordered_hashmap_free(c->groups);
         ordered_hashmap_free(c->users);
         ordered_hashmap_free(c->members);
@@ -137,7 +148,7 @@ static void context_done(Context *c) {
         hashmap_free(c->database_by_gid);
         hashmap_free(c->database_by_groupname);
 
-        set_free_free(c->names);
+        set_free(c->names);
         uid_range_free(c->uid_range);
 }
 
@@ -163,29 +174,59 @@ static void maybe_emit_login_defs_warning(Context *c) {
         c->login_defs_need_warning = false;
 }
 
+static void log_audit_accounts(Context *c, ItemType what) {
+#if HAVE_AUDIT
+        assert(c);
+        assert(IN_SET(what, ADD_USER, ADD_GROUP));
+
+        if (arg_dry_run || c->audit_fd < 0)
+                return;
+
+        Item *i;
+        int type = what == ADD_USER ? AUDIT_ADD_USER : AUDIT_ADD_GROUP;
+        const char *op = what == ADD_USER ? "adding-user" : "adding-group";
+
+        /* Notes:
+         *
+         * The op must not contain whitespace. The format with a dash matches what Fedora shadow-utils uses.
+         *
+         * We send id == -1, even though we know the number, in particular on success. This is because if we
+         * send the id, the generated audit message will not contain the name. The name seems more useful
+         * than the number, hence send just the name:
+         *
+         * type=ADD_USER msg=audit(01/10/2025 16:02:00.639:3854) :
+         *   pid=3846380 uid=root auid=zbyszek ses=2 msg='op=adding-user id=unknown(952) exe=systemd-sysusers ... res=success'
+         * vs.
+         * type=ADD_USER msg=audit(01/10/2025 16:03:15.457:3908) :
+         *   pid=3846607 uid=root auid=zbyszek ses=2 msg='op=adding-user acct=foo5 exe=systemd-sysusers ... res=success'
+         */
+
+        ORDERED_HASHMAP_FOREACH(i, what == ADD_USER ? c->todo_uids : c->todo_gids)
+                sym_audit_log_acct_message(
+                                c->audit_fd,
+                                type,
+                                program_invocation_short_name,
+                                op,
+                                i->name,
+                                /* id= */ (unsigned) -1,
+                                /* host= */ NULL,
+                                /* addr= */ NULL,
+                                /* tty= */ NULL,
+                                /* success= */ 1);
+#endif
+}
+
 static int load_user_database(Context *c) {
+        _cleanup_free_ char *passwd_path = NULL;
         _cleanup_fclose_ FILE *f = NULL;
-        const char *passwd_path;
         struct passwd *pw;
         int r;
 
         assert(c);
 
-        passwd_path = prefix_roota(arg_root, "/etc/passwd");
-        f = fopen(passwd_path, "re");
-        if (!f)
-                return errno == ENOENT ? 0 : -errno;
-
-        r = hashmap_ensure_allocated(&c->database_by_username, &string_hash_ops);
-        if (r < 0)
-                return r;
-
-        r = hashmap_ensure_allocated(&c->database_by_uid, NULL);
-        if (r < 0)
-                return r;
-
-        /* Note that we use NULL, i.e. trivial_hash_ops here, so identical strings can exist in the set. */
-        r = set_ensure_allocated(&c->names, NULL);
+        r = chase_and_fopen_unlocked("/etc/passwd", arg_root, CHASE_PREFIX_ROOT, "re", &passwd_path, &f);
+        if (r == -ENOENT)
+                return 0;
         if (r < 0)
                 return r;
 
@@ -195,19 +236,20 @@ static int load_user_database(Context *c) {
                 if (!n)
                         return -ENOMEM;
 
-                r = set_consume(c->names, n);
+                /* Note that we use trivial_hash_ops_free here, so identical strings can exist in the set. */
+                r = set_ensure_consume(&c->names, &trivial_hash_ops_free, n);
                 if (r < 0)
                         return r;
                 assert(r > 0);  /* The set uses pointer comparisons, so n must not be in the set. */
 
-                r = hashmap_put(c->database_by_username, n, UID_TO_PTR(pw->pw_uid));
+                r = hashmap_ensure_put(&c->database_by_username, &string_hash_ops, n, UID_TO_PTR(pw->pw_uid));
                 if (r == -EEXIST)
                         log_debug_errno(r, "%s: user '%s' is listed twice, ignoring duplicate uid.",
                                         passwd_path, n);
                 else if (r < 0)
                         return r;
 
-                r = hashmap_put(c->database_by_uid, UID_TO_PTR(pw->pw_uid), n);
+                r = hashmap_ensure_put(&c->database_by_uid, /* hash_ops= */ NULL, UID_TO_PTR(pw->pw_uid), n);
                 if (r == -EEXIST)
                         log_debug_errno(r, "%s: uid "UID_FMT" is listed twice, ignoring duplicate name.",
                                         passwd_path, pw->pw_uid);
@@ -218,50 +260,38 @@ static int load_user_database(Context *c) {
 }
 
 static int load_group_database(Context *c) {
+        _cleanup_free_ char *group_path = NULL;
         _cleanup_fclose_ FILE *f = NULL;
-        const char *group_path;
         struct group *gr;
         int r;
 
         assert(c);
 
-        group_path = prefix_roota(arg_root, "/etc/group");
-        f = fopen(group_path, "re");
-        if (!f)
-                return errno == ENOENT ? 0 : -errno;
-
-        r = hashmap_ensure_allocated(&c->database_by_groupname, &string_hash_ops);
-        if (r < 0)
-                return r;
-
-        r = hashmap_ensure_allocated(&c->database_by_gid, NULL);
-        if (r < 0)
-                return r;
-
-        /* Note that we use NULL, i.e. trivial_hash_ops here, so identical strings can exist in the set. */
-        r = set_ensure_allocated(&c->names, NULL);
+        r = chase_and_fopen_unlocked("/etc/group", arg_root, CHASE_PREFIX_ROOT, "re", &group_path, &f);
+        if (r == -ENOENT)
+                return 0;
         if (r < 0)
                 return r;
 
         while ((r = fgetgrent_sane(f, &gr)) > 0) {
-
                 char *n = strdup(gr->gr_name);
                 if (!n)
                         return -ENOMEM;
 
-                r = set_consume(c->names, n);
+                /* Note that we use trivial_hash_ops_free here, so identical strings can exist in the set. */
+                r = set_ensure_consume(&c->names, &trivial_hash_ops_free, n);
                 if (r < 0)
                         return r;
                 assert(r > 0);  /* The set uses pointer comparisons, so n must not be in the set. */
 
-                r = hashmap_put(c->database_by_groupname, n, GID_TO_PTR(gr->gr_gid));
+                r = hashmap_ensure_put(&c->database_by_groupname, &string_hash_ops, n, GID_TO_PTR(gr->gr_gid));
                 if (r == -EEXIST)
                         log_debug_errno(r, "%s: group '%s' is listed twice, ignoring duplicate gid.",
                                         group_path, n);
                 else if (r < 0)
                         return r;
 
-                r = hashmap_put(c->database_by_gid, GID_TO_PTR(gr->gr_gid), n);
+                r = hashmap_ensure_put(&c->database_by_gid, /* hash_ops= */ NULL, GID_TO_PTR(gr->gr_gid), n);
                 if (r == -EEXIST)
                         log_debug_errno(r, "%s: gid "GID_FMT" is listed twice, ignoring duplicate name.",
                                         group_path, gr->gr_gid);
@@ -425,6 +455,8 @@ static int putsgent_with_members(
 #endif
 
 static const char* pick_shell(const Item *i) {
+        assert(i);
+
         if (i->type != ADD_USER)
                 return NULL;
         if (i->shell)
@@ -449,11 +481,11 @@ static int write_temporary_passwd(
         assert(c);
 
         if (ordered_hashmap_isempty(c->todo_uids))
-                return 0;
+                goto done;
 
         if (arg_dry_run) {
-                log_info("Would write /etc/passwd%s", special_glyph(SPECIAL_GLYPH_ELLIPSIS));
-                return 0;
+                log_info("Would write /etc/passwd%s", glyph(GLYPH_ELLIPSIS));
+                goto done;
         }
 
         r = fopen_temporary_label("/etc/passwd", passwd_path, &passwd, &passwd_tmp);
@@ -558,24 +590,12 @@ static int write_temporary_passwd(
         if (r < 0)
                 return log_debug_errno(r, "Failed to flush %s: %m", passwd_tmp);
 
+done:
         *ret_tmpfile = TAKE_PTR(passwd);
         *ret_tmpfile_path = TAKE_PTR(passwd_tmp);
-
         return 0;
 }
 
-static usec_t epoch_or_now(void) {
-        uint64_t epoch;
-
-        if (secure_getenv_uint64("SOURCE_DATE_EPOCH", &epoch) >= 0) {
-                if (epoch > UINT64_MAX/USEC_PER_SEC) /* Overflow check */
-                        return USEC_INFINITY;
-                return (usec_t) epoch * USEC_PER_SEC;
-        }
-
-        return now(CLOCK_REALTIME);
-}
-
 static int write_temporary_shadow(
                 Context *c,
                 const char *shadow_path,
@@ -592,18 +612,18 @@ static int write_temporary_shadow(
         assert(c);
 
         if (ordered_hashmap_isempty(c->todo_uids))
-                return 0;
+                goto done;
 
         if (arg_dry_run) {
-                log_info("Would write /etc/shadow%s", special_glyph(SPECIAL_GLYPH_ELLIPSIS));
-                return 0;
+                log_info("Would write /etc/shadow%s", glyph(GLYPH_ELLIPSIS));
+                goto done;
         }
 
         r = fopen_temporary_label("/etc/shadow", shadow_path, &shadow, &shadow_tmp);
         if (r < 0)
                 return log_debug_errno(r, "Failed to open temporary copy of %s: %m", shadow_path);
 
-        lstchg = (long) (epoch_or_now() / USEC_PER_DAY);
+        lstchg = (long) (source_date_epoch_or_now() / USEC_PER_DAY);
 
         original = fopen(shadow_path, "re");
         if (original) {
@@ -705,9 +725,9 @@ static int write_temporary_shadow(
         if (r < 0)
                 return log_debug_errno(r, "Failed to flush %s: %m", shadow_tmp);
 
+done:
         *ret_tmpfile = TAKE_PTR(shadow);
         *ret_tmpfile_path = TAKE_PTR(shadow_tmp);
-
         return 0;
 }
 
@@ -727,11 +747,11 @@ static int write_temporary_group(
         assert(c);
 
         if (ordered_hashmap_isempty(c->todo_gids) && ordered_hashmap_isempty(c->members))
-                return 0;
+                goto done;
 
         if (arg_dry_run) {
-                log_info("Would write /etc/group%s", special_glyph(SPECIAL_GLYPH_ELLIPSIS));
-                return 0;
+                log_info("Would write /etc/group%s", glyph(GLYPH_ELLIPSIS));
+                goto done;
         }
 
         r = fopen_temporary_label("/etc/group", group_path, &group, &group_tmp);
@@ -817,9 +837,13 @@ static int write_temporary_group(
         if (r < 0)
                 return log_error_errno(r, "Failed to flush %s: %m", group_tmp);
 
+done:
         if (group_changed) {
                 *ret_tmpfile = TAKE_PTR(group);
                 *ret_tmpfile_path = TAKE_PTR(group_tmp);
+        } else {
+                *ret_tmpfile = NULL;
+                *ret_tmpfile_path = NULL;
         }
         return 0;
 }
@@ -840,11 +864,11 @@ static int write_temporary_gshadow(
         assert(c);
 
         if (ordered_hashmap_isempty(c->todo_gids) && ordered_hashmap_isempty(c->members))
-                return 0;
+                goto done;
 
         if (arg_dry_run) {
-                log_info("Would write /etc/gshadow%s", special_glyph(SPECIAL_GLYPH_ELLIPSIS));
-                return 0;
+                log_info("Would write /etc/gshadow%s", glyph(GLYPH_ELLIPSIS));
+                goto done;
         }
 
         r = fopen_temporary_label("/etc/gshadow", gshadow_path, &gshadow, &gshadow_tmp);
@@ -903,11 +927,16 @@ static int write_temporary_gshadow(
         if (r < 0)
                 return log_error_errno(r, "Failed to flush %s: %m", gshadow_tmp);
 
+done:
         if (group_changed) {
                 *ret_tmpfile = TAKE_PTR(gshadow);
                 *ret_tmpfile_path = TAKE_PTR(gshadow_tmp);
-        }
+        } else
 #endif
+        {
+                *ret_tmpfile = NULL;
+                *ret_tmpfile_path = NULL;
+        }
         return 0;
 }
 
@@ -916,11 +945,21 @@ static int write_files(Context *c) {
         _cleanup_(unlink_and_freep) char *passwd_tmp = NULL, *group_tmp = NULL, *shadow_tmp = NULL, *gshadow_tmp = NULL;
         int r;
 
-        const char
-                *passwd_path = prefix_roota(arg_root, "/etc/passwd"),
-                *shadow_path = prefix_roota(arg_root, "/etc/shadow"),
-                *group_path = prefix_roota(arg_root, "/etc/group"),
-                *gshadow_path = prefix_roota(arg_root, "/etc/gshadow");
+        _cleanup_free_ char *passwd_path = path_join(arg_root, "/etc/passwd");
+        if (!passwd_path)
+                return log_oom();
+
+        _cleanup_free_ char *shadow_path = path_join(arg_root, "/etc/shadow");
+        if (!shadow_path)
+                return log_oom();
+
+        _cleanup_free_ char *group_path = path_join(arg_root, "/etc/group");
+        if (!group_path)
+                return log_oom();
+
+        _cleanup_free_ char *gshadow_path = path_join(arg_root, "/etc/gshadow");
+        if (!gshadow_path)
+                return log_oom();
 
         assert(c);
 
@@ -971,6 +1010,8 @@ static int write_files(Context *c) {
                                                group_tmp, group_path);
                 group_tmp = mfree(group_tmp);
         }
+        /* OK, we have written the group entries successfully */
+        log_audit_accounts(c, ADD_GROUP);
         if (gshadow) {
                 r = rename_and_apply_smack_floor_label(gshadow_tmp, gshadow_path);
                 if (r < 0)
@@ -988,6 +1029,8 @@ static int write_files(Context *c) {
 
                 passwd_tmp = mfree(passwd_tmp);
         }
+        /* OK, we have written the user entries successfully */
+        log_audit_accounts(c, ADD_USER);
         if (shadow) {
                 r = rename_and_apply_smack_floor_label(shadow_tmp, shadow_path);
                 if (r < 0)
@@ -1058,11 +1101,8 @@ static int uid_is_ok(
         return 1;
 }
 
-static int root_stat(const char *p, struct stat *st) {
-        const char *fix;
-
-        fix = prefix_roota(arg_root, p);
-        return RET_NERRNO(stat(fix, st));
+static int root_stat(const char *p, struct stat *ret_st) {
+        return chase_and_stat(p, arg_root, CHASE_PREFIX_ROOT, /* ret_path= */ NULL, ret_st);
 }
 
 static int read_id_from_file(Item *i, uid_t *ret_uid, gid_t *ret_gid) {
@@ -2014,12 +2054,14 @@ static int help(void) {
         if (r < 0)
                 return log_oom();
 
-        printf("%s [OPTIONS...] [CONFIGURATION FILE...]\n\n"
-               "Creates system user accounts.\n\n"
-               "  -h --help                 Show this help\n"
-               "     --version              Show package version\n"
+        printf("%1$s [OPTIONS...] [CONFIGURATION FILE...]\n"
+               "\n%2$sCreates system user and group accounts.%4$s\n"
+               "\n%3$sCommands:%4$s\n"
                "     --cat-config           Show configuration files\n"
                "     --tldr                 Show non-comment parts of configuration\n"
+               "  -h --help                 Show this help\n"
+               "     --version              Show package version\n"
+               "\n%3$sOptions:%4$s\n"
                "     --root=PATH            Operate on an alternate filesystem root\n"
                "     --image=PATH           Operate on disk image as filesystem root\n"
                "     --image-policy=POLICY  Specify disk image dissection policy\n"
@@ -2027,8 +2069,11 @@ static int help(void) {
                "     --dry-run              Just print what would be done\n"
                "     --inline               Treat arguments as configuration lines\n"
                "     --no-pager             Do not pipe output into a pager\n"
-               "\nSee the %s for details.\n",
+               "\nSee the %5$s for details.\n",
                program_invocation_short_name,
+               ansi_highlight(),
+               ansi_underline(),
+               ansi_normal(),
                link);
 
         return 0;
@@ -2189,13 +2234,13 @@ static int read_config_files(Context *c, char **args) {
 
         STRV_FOREACH(f, files)
                 if (p && path_equal(*f, p)) {
-                        log_debug("Parsing arguments at position \"%s\"%s", *f, special_glyph(SPECIAL_GLYPH_ELLIPSIS));
+                        log_debug("Parsing arguments at position \"%s\"%s", *f, glyph(GLYPH_ELLIPSIS));
 
                         r = parse_arguments(c, args);
                         if (r < 0)
                                 return r;
                 } else {
-                        log_debug("Reading config file \"%s\"%s", *f, special_glyph(SPECIAL_GLYPH_ELLIPSIS));
+                        log_debug("Reading config file \"%s\"%s", *f, glyph(GLYPH_ELLIPSIS));
 
                         /* Just warn, ignore result otherwise */
                         (void) read_config_file(c, *f, /* ignore_enoent= */ true);
@@ -2232,6 +2277,7 @@ static int run(int argc, char *argv[]) {
 #endif
         _cleanup_close_ int lock = -EBADF;
         _cleanup_(context_done) Context c = {
+                .audit_fd = -EBADF,
                 .search_uid = UID_INVALID,
         };
 
@@ -2247,6 +2293,9 @@ static int run(int argc, char *argv[]) {
         if (arg_cat_flags != CAT_CONFIG_OFF)
                 return cat_config();
 
+        if (should_bypass("SYSTEMD_SYSUSERS"))
+                return 0;
+
         umask(0022);
 
         r = mac_init();
@@ -2281,6 +2330,10 @@ static int run(int argc, char *argv[]) {
         assert(!arg_image);
 #endif
 
+        /* Prepare to emit audit events, but only if we're operating on the host system. */
+        if (!arg_root)
+                c.audit_fd = open_audit_fd_or_warn();
+
         /* If command line arguments are specified along with --replace, read all configuration files and
          * insert the positional arguments at the specified place. Otherwise, if command line arguments are
          * specified, execute just them, and finally, without --replace= or any positional arguments, just
