@@ -1,13 +1,11 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
+#include <stdlib.h>
+#include <unistd.h>
+
 #include "env-util.h"
-#include "fd-util.h"
-#include "fileio.h"
-#include "fs-util.h"
 #include "parse-util.h"
 #include "process-util.h"
-#include "serialize.h"
-#include "string-util.h"
 #include "strv.h"
 #include "tests.h"
 
