@@ -141,6 +141,25 @@ def try_import(modname: str, name: Optional[str] = None) -> ModuleType:
         raise ValueError(f'Kernel is compressed with {name or modname}, but module unavailable') from e
 
 
+def read_env_file(text: str) -> dict[str, str]:
+    result = {}
+
+    for line in text.splitlines():
+        line = line.rstrip()
+        if not line or line.startswith('#'):
+            continue
+        if m := re.match(r'([A-Z][A-Z_0-9]+)=(.*)', line):
+            name, val = m.groups()
+            if val and val[0] in '"\'':
+                val = next(shlex.shlex(val, posix=True))
+
+            result[name] = val
+        else:
+            print(f'bad line {line!r}', file=sys.stderr)
+
+    return result
+
+
 def get_zboot_kernel(f: IO[bytes]) -> bytes:
     """Decompress zboot efistub kernel if compressed. Return contents."""
     # See linux/drivers/firmware/efi/libstub/Makefile.zboot
@@ -179,9 +198,14 @@ def get_zboot_kernel(f: IO[bytes]) -> bytes:
     elif comp_type.startswith(b'xzkern'):
         raise NotImplementedError('xzkern decompression not implemented')
     elif comp_type.startswith(b'zstd'):
-        zstd = try_import('zstandard')
-        data = f.read(size)
-        return cast(bytes, zstd.ZstdDecompressor().stream_reader(data).read())
+        try:
+            zstd = try_import('compression.zstd')
+            data = f.read(size)
+            return cast(bytes, zstd.zstd.ZstdDecompressor().decompress(data))
+        except ValueError:
+            zstd = try_import('zstandard')
+            data = f.read(size)
+            return cast(bytes, zstd.ZstdDecompressor().stream_reader(data).read())
 
     raise NotImplementedError(f'unknown compressed type: {comp_type!r}')
 
@@ -211,8 +235,12 @@ def maybe_decompress(filename: Union[str, Path]) -> bytes:
         return cast(bytes, gzip.open(f).read())
 
     if start.startswith(b'\x28\xb5\x2f\xfd'):
-        zstd = try_import('zstandard')
-        return cast(bytes, zstd.ZstdDecompressor().stream_reader(f.read()).read())
+        try:
+            zstd = try_import('compression.zstd')
+            return cast(bytes, zstd.zstd.ZstdDecompressor().decompress(f.read()))
+        except ValueError:
+            zstd = try_import('zstandard')
+            return cast(bytes, zstd.ZstdDecompressor().stream_reader(f.read()).read())
 
     if start.startswith(b'\x02\x21\x4c\x18'):
         lz4 = try_import('lz4.frame', 'lz4')
@@ -247,7 +275,9 @@ class UkifyConfig:
     efi_arch: str
     hwids: Path
     initrd: list[Path]
+    efifw: list[Path]
     join_profiles: list[Path]
+    sign_profiles: list[str]
     json: Union[Literal['pretty'], Literal['short'], Literal['off']]
     linux: Optional[Path]
     measure: bool
@@ -257,9 +287,13 @@ class UkifyConfig:
     pcr_banks: list[str]
     pcr_private_keys: list[str]
     pcr_public_keys: list[str]
+    pcr_certificates: list[str]
     pcrpkey: Optional[Path]
+    pcrsig: Union[str, Path, None]
+    join_pcrsig: Optional[Path]
     phase_path_groups: Optional[list[str]]
-    profile: Union[str, Path, None]
+    policy_digest: bool
+    profile: Optional[str]
     sb_cert: Union[str, Path, None]
     sb_cert_name: Optional[str]
     sb_cert_validity: int
@@ -374,6 +408,7 @@ DEFAULT_SECTIONS_TO_SHOW = {
     '.dtb':     'binary',
     '.dtbauto': 'binary',
     '.hwids':   'binary',
+    '.efifw':   'binary',
     '.cmdline': 'text',
     '.osrel':   'text',
     '.uname':   'text',
@@ -455,7 +490,10 @@ class UKI:
             if s.name == '.profile':
                 start = i + 1
 
-        if any(section.name == s.name for s in self.sections[start:] if s.name != '.dtbauto'):
+        multiple_allowed_sections = ['.dtbauto', '.efifw']
+        if any(
+            section.name == s.name for s in self.sections[start:] if s.name not in multiple_allowed_sections
+        ):
             raise ValueError(f'Duplicate section {section.name}')
 
         self.sections += [section]
@@ -464,11 +502,11 @@ class UKI:
 class SignTool:
     @staticmethod
     def sign(input_f: str, output_f: str, opts: UkifyConfig) -> None:
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @staticmethod
     def verify(input_f: Path, opts: UkifyConfig) -> bool:
-        raise NotImplementedError()
+        raise NotImplementedError
 
     @staticmethod
     def from_string(name: str) -> type['SignTool']:
@@ -647,7 +685,10 @@ def check_inputs(opts: UkifyConfig) -> None:
         elif isinstance(value, list):
             for item in value:
                 if isinstance(item, Path):
-                    item.open().close()
+                    if item.is_dir():
+                        item.iterdir()
+                    else:
+                        item.open().close()
 
     check_splash(opts.splash)
 
@@ -656,7 +697,7 @@ def check_cert_and_keys_nonexistent(opts: UkifyConfig) -> None:
     # Raise if any of the keys and certs are found on disk
     paths: Iterator[Union[str, Path, None]] = itertools.chain(
         (opts.sb_key, opts.sb_cert),
-        *((priv_key, pub_key) for priv_key, pub_key, _ in key_path_groups(opts)),
+        *((priv_key, pub_key, cert) for priv_key, pub_key, cert, _ in key_path_groups(opts)),
     )
     for path in paths:
         if path and Path(path).exists():
@@ -694,17 +735,19 @@ def combine_signatures(pcrsigs: list[dict[str, str]]) -> str:
     return json.dumps(combined)
 
 
-def key_path_groups(opts: UkifyConfig) -> Iterator[tuple[str, Optional[str], Optional[str]]]:
+def key_path_groups(opts: UkifyConfig) -> Iterator[tuple[str, Optional[str], Optional[str], Optional[str]]]:
     if not opts.pcr_private_keys:
         return
 
     n_priv = len(opts.pcr_private_keys)
     pub_keys = opts.pcr_public_keys or []
+    certs = opts.pcr_certificates or []
     pp_groups = opts.phase_path_groups or []
 
     yield from itertools.zip_longest(
         opts.pcr_private_keys,
         pub_keys[:n_priv],
+        certs[:n_priv],
         pp_groups[:n_priv],
         fillvalue=None,
     )
@@ -718,12 +761,13 @@ def pe_section_size(section: pefile.SectionStructure) -> int:
     return cast(int, min(section.Misc_VirtualSize, section.SizeOfRawData))
 
 
-def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) -> None:
+def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) -> str:
     measure_tool = find_tool(
         'systemd-measure',
         '/usr/lib/systemd/systemd-measure',
         opts=opts,
     )
+    combined = ''
 
     banks = opts.pcr_banks or ()
 
@@ -758,7 +802,8 @@ def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) ->
 
             unique_to_measure[section.name] = section
 
-    if opts.measure:
+    if opts.measure or opts.policy_digest:
+        pcrsigs = []
         to_measure = unique_to_measure.copy()
 
         for dtbauto in dtbauto_to_measure:
@@ -769,7 +814,9 @@ def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) ->
 
             cmd = [
                 measure_tool,
-                'calculate',
+                'calculate' if opts.measure else 'policy-digest',
+                '--json',
+                opts.json,
                 *(f'--{s.name.removeprefix(".")}={s.content}' for s in to_measure.values()),
                 *(f'--bank={bank}' for bank in banks),
                 # For measurement, the keys are not relevant, so we can lump all the phase paths
@@ -777,8 +824,34 @@ def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) ->
                 *(f'--phase={phase_path}' for phase_path in itertools.chain.from_iterable(pp_groups)),
             ]
 
+            # The JSON object will be used for offline signing, include the public key
+            # so that the fingerprint is included too. In case a certificate is passed, use the
+            # right parameter so that systemd-measure can extract the public key from it.
+            if opts.policy_digest:
+                if opts.pcr_public_keys:
+                    cmd += ['--public-key', opts.pcr_public_keys[0]]
+                elif opts.pcr_certificates:
+                    cmd += ['--certificate', opts.pcr_certificates[0]]
+                    if opts.certificate_provider:
+                        cmd += ['--certificate-source', f'provider:{opts.certificate_provider}']
+
             print('+', shell_join(cmd), file=sys.stderr)
-            subprocess.check_call(cmd)
+            output = subprocess.check_output(cmd, text=True)  # type: ignore
+
+            if opts.policy_digest:
+                pcrsig = json.loads(output)
+                pcrsigs += [pcrsig]
+            else:
+                print(output)
+
+        if opts.policy_digest:
+            combined = combine_signatures(pcrsigs)
+            # We need to ensure the section has space for signatures, that will be added separately later,
+            # so add some whitespace to pad the section. At most we'll need 4kb per digest (rsa4096).
+            # We might even check the key type given we have it to know the precise length, but don't
+            # bother for now.
+            combined += ' ' * 1024 * combined.count('"pol":')
+            uki.add_section(Section.create('.pcrsig', combined))
 
     # PCR signing
 
@@ -797,16 +870,24 @@ def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) ->
                 *(f'--bank={bank}' for bank in banks),
             ]
 
-            for priv_key, pub_key, group in key_path_groups(opts):
+            for priv_key, pub_key, cert, group in key_path_groups(opts):
                 extra = [f'--private-key={priv_key}']
                 if opts.signing_engine is not None:
-                    assert pub_key
-                    extra += [f'--private-key-source=engine:{opts.signing_engine}']
-                    extra += [f'--certificate={pub_key}']
+                    assert pub_key or cert
+                    # Backward compatibility, we used to pass the public key as the certificate
+                    # as there was no --pcr-certificate= parameter
+                    extra += [
+                        f'--private-key-source=engine:{opts.signing_engine}',
+                        f'--certificate={pub_key or cert}',
+                    ]
                 elif opts.signing_provider is not None:
-                    assert pub_key
-                    extra += [f'--private-key-source=provider:{opts.signing_provider}']
-                    extra += [f'--certificate={pub_key}']
+                    assert pub_key or cert
+                    extra += [
+                        f'--private-key-source=provider:{opts.signing_provider}',
+                        f'--certificate={pub_key or cert}',
+                    ]
+                elif cert:
+                    extra += [f'--certificate={cert}']
                 elif pub_key:
                     extra += [f'--public-key={pub_key}']
 
@@ -816,13 +897,15 @@ def call_systemd_measure(uki: UKI, opts: UkifyConfig, profile_start: int = 0) ->
                 extra += [f'--phase={phase_path}' for phase_path in group or ()]
 
                 print('+', shell_join(cmd + extra), file=sys.stderr)  # type: ignore
-                pcrsig = subprocess.check_output(cmd + extra, text=True)  # type: ignore
-                pcrsig = json.loads(pcrsig)
+                output = subprocess.check_output(cmd + extra, text=True)  # type: ignore
+                pcrsig = json.loads(output)
                 pcrsigs += [pcrsig]
 
         combined = combine_signatures(pcrsigs)
         uki.add_section(Section.create('.pcrsig', combined))
 
+    return combined
+
 
 def join_initrds(initrds: list[Path]) -> Union[Path, bytes, None]:
     if not initrds:
@@ -853,7 +936,7 @@ class PEError(Exception):
     pass
 
 
-def pe_add_sections(uki: UKI, output: str) -> None:
+def pe_add_sections(opts: UkifyConfig, uki: UKI, output: str) -> None:
     pe = pefile.PE(uki.executable, fast_load=True)
 
     # Old stubs do not have the symbol/string table stripped, even though image files should not have one.
@@ -869,10 +952,10 @@ def pe_add_sections(uki: UKI, output: str) -> None:
             pe.FILE_HEADER.NumberOfSymbols = 0
             pe.FILE_HEADER.IMAGE_FILE_LOCAL_SYMS_STRIPPED = True
 
-    # Old stubs might have been stripped, leading to unaligned raw data values, so let's fix them up here.
     # pylint thinks that Structure doesn't have various members that it has…
     # pylint: disable=no-member
 
+    # Old stubs might have been stripped, leading to unaligned raw data values, so let's fix them up here.
     for i, section in enumerate(pe.sections):
         oldp = section.PointerToRawData
         oldsz = section.SizeOfRawData
@@ -910,10 +993,14 @@ def pe_add_sections(uki: UKI, output: str) -> None:
             continue
         raise PEError(f'pefile warnings treated as errors: {warnings}')
 
-    security = pe.OPTIONAL_HEADER.DATA_DIRECTORY[pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_SECURITY']]
-    if security.VirtualAddress != 0:
-        # We could strip the signatures, but why would anyone sign the stub?
-        raise PEError('Stub image is signed, refusing.')
+    # When attaching signatures we are operating on an existing UKI which might be signed
+    if not opts.pcrsig:
+        security = pe.OPTIONAL_HEADER.DATA_DIRECTORY[
+            pefile.DIRECTORY_ENTRY['IMAGE_DIRECTORY_ENTRY_SECURITY']
+        ]
+        if security.VirtualAddress != 0:
+            # We could strip the signatures, but why would anyone sign the stub?
+            raise PEError('Stub image is signed, refusing')
 
     # Remember how many sections originate from systemd-stub
     n_original_sections = len(pe.sections)
@@ -967,7 +1054,10 @@ def pe_add_sections(uki: UKI, output: str) -> None:
         for i, s in enumerate(pe.sections[:n_original_sections]):
             if pe_strip_section_name(s.Name) == section.name and section.name != '.dtbauto':
                 if new_section.Misc_VirtualSize > s.SizeOfRawData:
-                    raise PEError(f'Not enough space in existing section {section.name} to append new data.')
+                    raise PEError(
+                        f'Not enough space in existing section {section.name} to append new data'
+                        f' (need {new_section.Misc_VirtualSize}, have {s.SizeOfRawData})'
+                    )
 
                 padding = bytes(s.SizeOfRawData - new_section.Misc_VirtualSize)
                 pe.__data__ = (
@@ -991,6 +1081,46 @@ def pe_add_sections(uki: UKI, output: str) -> None:
             pe.__structures__.append(new_section)
             pe.sections.append(new_section)
 
+    # If there is a pre-signed JSON blob, we need to update the existing JSON, by appending the signature to
+    # each corresponding digest object. We have built the unsigned UKI with enough space to fit the .sig
+    # objects, so we can just replace the new signed JSON in the existing sections.
+    if opts.pcrsig:
+        signatures = json.loads(str(opts.pcrsig))
+        for i, section in enumerate(pe.sections):
+            if pe_strip_section_name(section.Name) == '.pcrsig':
+                j = json.loads(
+                    bytes(
+                        pe.__data__[
+                            section.PointerToRawData : section.PointerToRawData + section.SizeOfRawData
+                        ]
+                    )
+                    .rstrip(b'\x00')
+                    .decode()
+                )
+                for (bank, sigs), (input_bank, input_sigs) in itertools.product(
+                    j.items(), signatures.items()
+                ):
+                    if input_bank != bank:
+                        continue
+                    for sig, input_sig in itertools.product(sigs, input_sigs):
+                        if sig['pol'] == input_sig['pol']:
+                            sig['sig'] = input_sig['sig']
+
+                encoded = json.dumps(j).encode()
+                if len(encoded) > section.SizeOfRawData:
+                    raise PEError(
+                        f'Not enough space in existing section .pcrsig of size {section.SizeOfRawData} to append new data of size {len(encoded)}'  # noqa: E501
+                    )
+                section.Misc_VirtualSize = len(encoded)
+                # bytes(n) results in an array of n zeroes
+                padding = bytes(section.SizeOfRawData - len(encoded))
+                pe.__data__ = (
+                    pe.__data__[: section.PointerToRawData]
+                    + encoded
+                    + padding
+                    + pe.__data__[section.PointerToRawData + section.SizeOfRawData :]
+                )
+
     pe.OPTIONAL_HEADER.CheckSum = 0
     pe.OPTIONAL_HEADER.SizeOfImage = round_up(
         pe.sections[-1].VirtualAddress + pe.sections[-1].Misc_VirtualSize,
@@ -1029,34 +1159,40 @@ def merge_sbat(input_pe: list[Path], input_text: list[str]) -> str:
             continue
         sbat += split[1:]
 
-    return (
-        'sbat,1,SBAT Version,sbat,1,https://github.com/rhboot/shim/blob/main/SBAT.md\n'
-        + '\n'.join(sbat)
-        + '\n\x00'
+    return '\n'.join(
+        (
+            'sbat,1,SBAT Version,sbat,1,https://github.com/rhboot/shim/blob/main/SBAT.md',
+            *sbat,
+            '',  # an empty line so that we end up with a newline at the end
+        )
     )
 
 
-# Keep in sync with Device (DEVICE_TYPE_DEVICETREE) from src/boot/chid.h
+# Keep in sync with Device from src/boot/chid.h
 # uint32_t descriptor, EFI_GUID chid, uint32_t name_offset, uint32_t compatible_offset
 DEVICE_STRUCT_SIZE = 4 + 16 + 4 + 4
 NULL_DEVICE = b'\0' * DEVICE_STRUCT_SIZE
 DEVICE_TYPE_DEVICETREE = 1
+DEVICE_TYPE_UEFI_FW = 2
+
+# Keep in sync with efifirmware.h
+FWHEADERMAGIC = 'feeddead'
+EFIFW_HEADER_SIZE = 4 + 4 + 4 + 4
 
 
 def device_make_descriptor(device_type: int, size: int) -> int:
     return (size) | (device_type << 28)
 
 
-DEVICETREE_DESCRIPTOR = device_make_descriptor(DEVICE_TYPE_DEVICETREE, DEVICE_STRUCT_SIZE)
-
-
-def pack_device(offsets: dict[str, int], name: str, compatible: str, chids: set[uuid.UUID]) -> bytes:
+def pack_device(
+    offsets: dict[str, int], devtype: int, name: str, compatible_or_fwid: str, chids: set[uuid.UUID]
+) -> bytes:
     data = b''
-
+    descriptor = device_make_descriptor(devtype, DEVICE_STRUCT_SIZE)
     for chid in sorted(chids):
-        data += struct.pack('<I', DEVICETREE_DESCRIPTOR)
+        data += struct.pack('<I', descriptor)
         data += chid.bytes_le
-        data += struct.pack('<II', offsets[name], offsets[compatible])
+        data += struct.pack('<II', offsets[name], offsets[compatible_or_fwid])
 
     assert len(data) == DEVICE_STRUCT_SIZE * len(chids)
     return data
@@ -1075,21 +1211,48 @@ def pack_strings(strings: set[str], base: int) -> tuple[bytes, dict[str, int]]:
 
 def parse_hwid_dir(path: Path) -> bytes:
     hwid_files = sorted(path.rglob('*.json'))
+    devstr_to_type: dict[str, int] = {
+        'devicetree': DEVICE_TYPE_DEVICETREE,
+        'uefi-fw': DEVICE_TYPE_UEFI_FW,
+    }
+
+    # all attributes in the mandatory attributes list must be present
+    mandatory_attribute = ['type', 'name', 'hwids']
+
+    # at least one of the following attributes must be present
+    one_of = ['compatible', 'fwid']
+
+    one_of_key_to_devtype: dict[str, int] = {
+        'compatible': DEVICE_TYPE_DEVICETREE,
+        'fwid': DEVICE_TYPE_UEFI_FW,
+    }
 
     strings: set[str] = set()
-    devices: collections.defaultdict[tuple[str, str], set[uuid.UUID]] = collections.defaultdict(set)
+    devices: collections.defaultdict[tuple[int, str, str], set[uuid.UUID]] = collections.defaultdict(set)
 
     for hwid_file in hwid_files:
         data = json.loads(hwid_file.read_text(encoding='UTF-8'))
 
-        for k in ['name', 'compatible', 'hwids']:
+        for k in mandatory_attribute:
             if k not in data:
                 raise ValueError(f'hwid description file "{hwid_file}" does not contain "{k}"')
 
-        strings |= {data['name'], data['compatible']}
+        if not any(key in data for key in one_of):
+            required_keys = ','.join(one_of)
+            raise ValueError(f'hwid description file "{hwid_file}" must contain one of {required_keys}')
 
-        # (name, compatible) pair uniquely identifies the device
-        devices[(data['name'], data['compatible'])] |= {uuid.UUID(u) for u in data['hwids']}
+        # (devtype, name, compatible/fwid) pair uniquely identifies the device
+        devtype = devstr_to_type[data['type']]
+
+        for k in one_of:
+            if k in data:
+                if one_of_key_to_devtype[k] != devtype:
+                    raise ValueError(
+                        f'wrong attribute "{k}" for hwid description file "{hwid_file}", '
+                        'device type: "%s"' % devtype
+                    )
+                strings |= {data['name'], data[k]}
+                devices[(devtype, data['name'], data[k])] |= {uuid.UUID(u) for u in data['hwids']}
 
     total_device_structs = 1
     for dev, uuids in devices.items():
@@ -1098,14 +1261,50 @@ def parse_hwid_dir(path: Path) -> bytes:
     strings_blob, offsets = pack_strings(strings, total_device_structs * DEVICE_STRUCT_SIZE)
 
     devices_blob = b''
-    for (name, compatible), uuids in devices.items():
-        devices_blob += pack_device(offsets, name, compatible, uuids)
+    for (devtype, name, compatible_or_fwid), uuids in devices.items():
+        devices_blob += pack_device(offsets, devtype, name, compatible_or_fwid, uuids)
 
     devices_blob += NULL_DEVICE
 
     return devices_blob + strings_blob
 
 
+def parse_efifw_dir(path: Path) -> bytes:
+    if not path.is_dir():
+        raise ValueError(f'{path} is not a directory or it does not exist')
+
+    # only one firmware image must be present in the directory
+    # to uniquely identify that firmware with its ID.
+    if len(list(path.glob('*'))) != 1:
+        raise ValueError(f'{path} must contain exactly one firmware image file')
+
+    payload_blob = b''
+    for fw in path.iterdir():
+        payload_blob += fw.read_bytes()
+
+    payload_len = len(payload_blob)
+    if payload_len == 0:
+        raise ValueError(f'{fw} is a zero byte file!')
+
+    dirname = path.parts[-1]
+    # firmware id is the name of the directory the firmware bundle is in,
+    # terminated by NULL.
+    fwid = b'' + dirname.encode() + b'\0'
+    fwid_len = len(fwid)
+    magic = bytes.fromhex(FWHEADERMAGIC)
+
+    efifw_header_blob = b''
+    efifw_header_blob += struct.pack('<p', magic)
+    efifw_header_blob += struct.pack('<I', EFIFW_HEADER_SIZE)
+    efifw_header_blob += struct.pack('<I', fwid_len)
+    efifw_header_blob += struct.pack('<I', payload_len)
+
+    efifw_blob = b''
+    efifw_blob += efifw_header_blob + fwid + payload_blob
+
+    return efifw_blob
+
+
 STUB_SBAT = """\
 sbat,1,SBAT Version,sbat,1,https://github.com/rhboot/shim/blob/main/SBAT.md
 uki,1,UKI,uki,1,https://uapi-group.org/specifications/specs/unified_kernel_image/
@@ -1125,6 +1324,7 @@ def make_uki(opts: UkifyConfig) -> None:
     sign_args_present = opts.sb_key or opts.sb_cert_name
     sign_kernel = opts.sign_kernel
     linux = opts.linux
+    combined_sigs = '{}'
 
     # On some distros, on some architectures, the vmlinuz is a gzip file, so we need to decompress it
     # if it's not a valid PE file, as it will fail to be booted by the firmware.
@@ -1158,13 +1358,13 @@ def make_uki(opts: UkifyConfig) -> None:
         print('Kernel version not specified, starting autodetection 😖.', file=sys.stderr)
         opts.uname = Uname.scrape(linux, opts=opts)
 
-    uki = UKI(opts.stub)
+    uki = UKI(opts.join_pcrsig if opts.join_pcrsig else opts.stub)
     initrd = join_initrds(opts.initrd)
 
     pcrpkey: Union[bytes, Path, None] = opts.pcrpkey
     if pcrpkey is None:
-        measure_tool = find_tool('systemd-keyutil', '/usr/lib/systemd/systemd-keyutil')
-        cmd = [measure_tool, 'public']
+        keyutil_tool = find_tool('systemd-keyutil', '/usr/lib/systemd/systemd-keyutil')
+        cmd = [keyutil_tool, 'public']
 
         if opts.pcr_public_keys and len(opts.pcr_public_keys) == 1:
             # If we're using an engine or provider, the public key will be an X.509 certificate.
@@ -1177,6 +1377,13 @@ def make_uki(opts: UkifyConfig) -> None:
                 pcrpkey = subprocess.check_output(cmd)
             else:
                 pcrpkey = Path(opts.pcr_public_keys[0])
+        elif opts.pcr_certificates and len(opts.pcr_certificates) == 1:
+            cmd += ['--certificate', opts.pcr_certificates[0]]
+            if opts.certificate_provider:
+                cmd += ['--certificate-source', f'provider:{opts.certificate_provider}']
+
+            print('+', shell_join(cmd), file=sys.stderr)
+            pcrpkey = subprocess.check_output(cmd)
         elif opts.pcr_private_keys and len(opts.pcr_private_keys) == 1:
             cmd += ['--private-key', Path(opts.pcr_private_keys[0])]
 
@@ -1205,6 +1412,7 @@ def make_uki(opts: UkifyConfig) -> None:
         ('.pcrpkey', pcrpkey,         True),
         ('.linux',   linux,           True),
         ('.initrd',  initrd,          True),
+        *(('.efifw', parse_efifw_dir(fw), False) for fw in opts.efifw),
         ('.ucode',   opts.microcode,  True),
     ]  # fmt: skip
 
@@ -1220,11 +1428,11 @@ def make_uki(opts: UkifyConfig) -> None:
     for section in opts.sections:
         uki.add_section(section)
 
-    # Don't add a sbat section to profile PE binaries.
-    if opts.join_profiles or not opts.profile:
+    # Don't add an .sbat section to profile PE binaries.
+    if (opts.join_profiles or not opts.profile) and not opts.pcrsig:
         if linux is not None:
-            # Merge the .sbat sections from stub, kernel and parameter, so that revocation can be done on
-            # either.
+            # Merge the .sbat sections from stub, kernel, and parameter, so
+            # that revocation can be done on either.
             input_pes = [opts.stub, linux]
             if not opts.sbat:
                 opts.sbat = [STUB_SBAT]
@@ -1243,7 +1451,15 @@ def make_uki(opts: UkifyConfig) -> None:
 
     # PCR measurement and signing
 
-    call_systemd_measure(uki, opts=opts)
+    if (
+        not opts.pcrsig
+        and (opts.join_profiles or not opts.profile)
+        and (
+            not opts.sign_profiles
+            or (opts.profile and read_env_file(opts.profile).get('ID') in opts.sign_profiles)
+        )
+    ):
+        combined_sigs = call_systemd_measure(uki, opts=opts)
 
     # UKI profiles
 
@@ -1252,6 +1468,7 @@ def make_uki(opts: UkifyConfig) -> None:
         '.osrel',
         '.cmdline',
         '.initrd',
+        '.efifw',
         '.ucode',
         '.splash',
         '.dtb',
@@ -1260,6 +1477,9 @@ def make_uki(opts: UkifyConfig) -> None:
         '.profile',
     }
 
+    if not opts.os_release:
+        to_import.remove('.osrel')
+
     for profile in opts.join_profiles:
         pe = pefile.PE(profile, fast_load=True)
         prev_len = len(uki.sections)
@@ -1292,7 +1512,16 @@ def make_uki(opts: UkifyConfig) -> None:
                 Section.create(n, pesection.get_data(length=pe_section_size(pesection)), measure=True)
             )
 
-        call_systemd_measure(uki, opts=opts, profile_start=prev_len)
+        if opts.sign_profiles:
+            pesection = next(s for s in pe.sections if pe_strip_section_name(s.Name) == '.profile')
+            id = read_env_file(pesection.get_data(length=pe_section_size(pesection)).decode()).get('ID')
+            if not id or id not in opts.sign_profiles:
+                print(f'Not signing expected PCR measurements for "{id}" profile', file=sys.stderr)
+                continue
+
+        s = call_systemd_measure(uki, opts=opts, profile_start=prev_len)
+        if s:
+            combined_sigs = combine_signatures([json.loads(combined_sigs), json.loads(s)])
 
     # UKI creation
 
@@ -1302,7 +1531,7 @@ def make_uki(opts: UkifyConfig) -> None:
     else:
         unsigned_output = opts.output
 
-    pe_add_sections(uki, unsigned_output)
+    pe_add_sections(opts, uki, unsigned_output)
 
     # UKI signing
 
@@ -1317,6 +1546,8 @@ def make_uki(opts: UkifyConfig) -> None:
         os.chmod(opts.output, 0o777 & ~umask)
 
     print(f'Wrote {"signed" if sign_args_present else "unsigned"} {opts.output}', file=sys.stderr)
+    if opts.policy_digest:
+        print(combined_sigs)
 
 
 @contextlib.contextmanager
@@ -1437,7 +1668,7 @@ def generate_keys(opts: UkifyConfig) -> None:
 
         work = True
 
-    for priv_key, pub_key, _ in key_path_groups(opts):
+    for priv_key, pub_key, _, _ in key_path_groups(opts):
         priv_key_pem, pub_key_pem = generate_priv_pub_key_pair()
 
         print(f'Writing private key for PCR signing to {priv_key}', file=sys.stderr)
@@ -1719,6 +1950,16 @@ CONFIG_ITEMS = [
         config_key='UKI/Initrd',
         config_push=ConfigItem.config_list_prepend,
     ),
+    ConfigItem(
+        '--efifw',
+        metavar='DIR',
+        type=Path,
+        action='append',
+        default=[],
+        help='Directory with efi firmware binary file [.efifw section]',
+        config_key='UKI/Firmware',
+        config_push=ConfigItem.config_list_prepend,
+    ),
     ConfigItem(
         '--microcode',
         metavar='UCODE',
@@ -1800,6 +2041,25 @@ CONFIG_ITEMS = [
         default=[],
         help='A PE binary containing an additional profile to add to the UKI',
     ),
+    ConfigItem(
+        '--sign-profile',
+        dest='sign_profiles',
+        metavar='ID',
+        action='append',
+        default=[],
+        help='Which profiles to sign expected PCR measurements for',
+    ),
+    ConfigItem(
+        '--pcrsig',
+        metavar='TEST|@PATH',
+        help='Signed PCR policy JSON [.pcrsig section] to append to an existing UKI',
+        config_key='UKI/PCRSig',
+    ),
+    ConfigItem(
+        '--join-pcrsig',
+        metavar='PATH',
+        help='A PE binary containing a UKI without a .pcrsig to join with --pcrsig',
+    ),
     ConfigItem(
         '--efi-arch',
         metavar='ARCH',
@@ -1913,6 +2173,15 @@ CONFIG_ITEMS = [
         config_key='PCRSignature:/PCRPublicKey',
         config_push=ConfigItem.config_set_group,
     ),
+    ConfigItem(
+        '--pcr-certificate',
+        dest='pcr_certificates',
+        metavar='PATH',
+        action='append',
+        help='certificate part of the keypair or engine/provider designation for signing PCR signatures',
+        config_key='PCRSignature:/PCRCertificate',
+        config_push=ConfigItem.config_set_group,
+    ),
     ConfigItem(
         '--phases',
         dest='phase_path_groups',
@@ -1939,6 +2208,11 @@ CONFIG_ITEMS = [
         action=argparse.BooleanOptionalAction,
         help='print systemd-measure output for the UKI',
     ),
+    ConfigItem(
+        '--policy-digest',
+        action=argparse.BooleanOptionalAction,
+        help='print systemd-measure policy digests for the UKI',
+    ),
     ConfigItem(
         '--json',
         choices=('pretty', 'short', 'off'),
@@ -2107,11 +2381,27 @@ def finalize_options(opts: argparse.Namespace) -> None:
 
     # Check that --pcr-public-key=, --pcr-private-key=, and --phases=
     # have either the same number of arguments or are not specified at all.
+    # Also check that --pcr-public-key= and --pcr-certificate= are not set at the same time.
+    # But allow a single public key, for offline PCR signing, to pre-populate the JSON object
+    # with the certificate's fingerprint.
+    n_pcr_cert = None if opts.pcr_certificates is None else len(opts.pcr_certificates)
     n_pcr_pub = None if opts.pcr_public_keys is None else len(opts.pcr_public_keys)
     n_pcr_priv = None if opts.pcr_private_keys is None else len(opts.pcr_private_keys)
     n_phase_path_groups = None if opts.phase_path_groups is None else len(opts.phase_path_groups)
-    if n_pcr_pub is not None and n_pcr_pub != n_pcr_priv:
+    if opts.policy_digest and n_pcr_priv is not None:
+        raise ValueError('--pcr-private-key= cannot be specified with --policy-digest')
+    if (
+        opts.policy_digest
+        and (n_pcr_pub is None or n_pcr_pub != 1)
+        and (n_pcr_cert is None or n_pcr_cert != 1)
+    ):
+        raise ValueError('--policy-digest requires exactly one --pcr-public-key= or --pcr-certificate=')
+    if n_pcr_pub is not None and n_pcr_priv is not None and n_pcr_pub != n_pcr_priv:
         raise ValueError('--pcr-public-key= specifications must match --pcr-private-key=')
+    if n_pcr_cert is not None and n_pcr_priv is not None and n_pcr_cert != n_pcr_priv:
+        raise ValueError('--pcr-certificate= specifications must match --pcr-private-key=')
+    if n_pcr_pub is not None and n_pcr_cert is not None:
+        raise ValueError('--pcr-public-key= and --pcr-certificate= cannot be used at the same time')
     if n_phase_path_groups is not None and n_phase_path_groups != n_pcr_priv:
         raise ValueError('--phases= specifications must match --pcr-private-key=')
 
@@ -2125,7 +2415,12 @@ def finalize_options(opts: argparse.Namespace) -> None:
 
     opts.os_release = resolve_at_path(opts.os_release)
 
-    if not opts.os_release and opts.linux:
+    if opts.os_release == '':
+        # If --os-release= with an empty string was passed, treat that as
+        # explicitly disabling the .osrel section, and do not fallback to the
+        # system's os-release files.
+        pass
+    elif opts.os_release is None and opts.linux:
         p = Path('/etc/os-release')
         if not p.exists():
             p = Path('/usr/lib/os-release')
@@ -2134,7 +2429,7 @@ def finalize_options(opts: argparse.Namespace) -> None:
     if opts.efi_arch is None:
         opts.efi_arch = guess_efi_arch()
 
-    if opts.stub is None:
+    if opts.stub is None and not opts.join_pcrsig:
         if opts.linux is not None:
             opts.stub = Path(f'/usr/lib/systemd/boot/efi/linux{opts.efi_arch}.efi.stub')
         else:
@@ -2171,10 +2466,10 @@ def finalize_options(opts: argparse.Namespace) -> None:
         opts.signtool = 'pesign'
 
     if opts.signing_provider and opts.signtool != 'systemd-sbsign':
-        raise ValueError('--signing-provider= can only be used with--signtool=systemd-sbsign')
+        raise ValueError('--signing-provider= can only be used with --signtool=systemd-sbsign')
 
     if opts.certificate_provider and opts.signtool != 'systemd-sbsign':
-        raise ValueError('--certificate-provider= can only be used with--signtool=systemd-sbsign')
+        raise ValueError('--certificate-provider= can only be used with --signtool=systemd-sbsign')
 
     if opts.sign_kernel and not opts.sb_key and not opts.sb_cert_name:
         raise ValueError(
@@ -2182,12 +2477,42 @@ def finalize_options(opts: argparse.Namespace) -> None:
         )
 
     opts.profile = resolve_at_path(opts.profile)
+    if opts.profile and isinstance(opts.profile, Path):
+        opts.profile = opts.profile.read_text()
 
     if opts.join_profiles and not opts.profile:
         # If any additional profiles are added, we need a base profile as well so add one if
         # one wasn't explicitly provided
         opts.profile = 'ID=main'
 
+    if opts.pcrsig and not opts.join_pcrsig:
+        raise ValueError('--pcrsig requires --join-pcrsig')
+    if opts.join_pcrsig and not opts.pcrsig:
+        raise ValueError('--join-pcrsig requires --pcrsig')
+    if opts.pcrsig and (
+        opts.linux
+        or opts.initrd
+        or opts.profile
+        or opts.join_profiles
+        or opts.microcode
+        or opts.sbat
+        or opts.uname
+        or opts.os_release
+        or opts.cmdline
+        or opts.hwids
+        or opts.splash
+        or opts.devicetree
+        or opts.devicetree_auto
+        or opts.pcr_private_keys
+        or opts.pcr_public_keys
+        or opts.pcr_certificates
+    ):
+        raise ValueError('--pcrsig and --join-pcrsig cannot be used with other sections')
+    if opts.pcrsig:
+        opts.pcrsig = resolve_at_path(opts.pcrsig)
+        if isinstance(opts.pcrsig, Path):
+            opts.pcrsig = opts.pcrsig.read_text()
+
     if opts.verb == 'build' and opts.output is None:
         if opts.linux is None:
             raise ValueError('--output= must be specified when building a PE addon')
@@ -2203,6 +2528,12 @@ def finalize_options(opts: argparse.Namespace) -> None:
 
 def parse_args(args: Optional[list[str]] = None) -> argparse.Namespace:
     opts = create_parser().parse_args(args)
+
+    # argparse puts some unknown options in opts.positional. Make sure we don't
+    # try to interpret something that is an option as a positional argument.
+    if any((bad_opt := o).startswith('-') for o in opts.positional):
+        raise ValueError(f'Unknown option: {bad_opt.partition("=")[0]}')
+
     apply_config(opts)
     finalize_options(opts)
     return opts
