@@ -1,20 +1,16 @@
 /* SPDX-License-Identifier: LGPL-2.1-or-later */
 
-#include <errno.h>
-#include <stdio.h>
-#include <unistd.h>
+#include <stdlib.h>
+#include <sys/stat.h>
 
+#include "alloc-util.h"
 #include "dirent-util.h"
 #include "fd-util.h"
 #include "generator.h"
 #include "glyph-util.h"
 #include "hashmap.h"
 #include "log.h"
-#include "main-func.h"
-#include "nulstr-util.h"
 #include "path-lookup.h"
-#include "stat-util.h"
-#include "string-util.h"
 #include "strv.h"
 #include "xdg-autostart-service.h"
 
@@ -77,7 +73,7 @@ static int enumerate_xdg_autostart(Hashmap *all_services) {
         STRV_FOREACH(path, autostart_dirs) {
                 _cleanup_closedir_ DIR *d = NULL;
 
-                log_debug("Scanning autostart directory \"%s\"%s", *path, special_glyph(SPECIAL_GLYPH_ELLIPSIS));
+                log_debug("Scanning autostart directory \"%s\"%s", *path, glyph(GLYPH_ELLIPSIS));
                 d = opendir(*path);
                 if (!d) {
                         log_full_errno(errno == ENOENT ? LOG_DEBUG : LOG_WARNING, errno,
