@@ -2,6 +2,8 @@
 # SPDX-License-Identifier: LGPL-2.1-or-later
 set -euxo pipefail
 
+. /etc/os-release
+
 DM_NAME="integrity_test"
 DM_NODE="/dev/mapper/${DM_NAME}"
 DM_SERVICE="systemd-integritysetup@${DM_NAME}.service"
@@ -87,7 +89,7 @@ test_one() {
     udevadm wait --timeout=30 --settle "${DM_NODE}"
     mkfs.ext4 -U "${FS_UUID}" "${DM_NODE}"
     # Wait for synthetic events being processed.
-    udevadm settle
+    udevadm settle --timeout=30
     integritysetup close "${DM_NAME}"
     udevadm wait --timeout=30 --settle --removed "${DM_NODE}"
 
@@ -117,6 +119,18 @@ EOF
 }
 
 for a in crc32c crc32 xxhash64 sha1 sha256; do
+    if [[ "$a" == crc32 && "${ID_LIKE:-}" == alpine ]]; then
+        # crc32 is not supported on alpine/postmarketos ??
+        # --------
+        # [   22.419458] TEST-67-INTEGRITY.sh[3085]: + integritysetup format /dev/loop0 --batch-mode -I crc32 ''
+        # [   22.433168] kernel: device-mapper: table: 253:0: integrity: Invalid internal hash (-ENOENT)
+        # [   22.433220] TEST-67-INTEGRITY.sh[3475]: device-mapper: reload ioctl on temporary-cryptsetup-6b3b80ef-6854-4102-8239-6360f15af0c3 (253:0) failed: No such file or directory
+        # [   22.433220] TEST-67-INTEGRITY.sh[3475]: Cannot format integrity for device /dev/loop0.
+        # [   22.433835] kernel: device-mapper: ioctl: error adding target to table
+        # --------
+        continue;
+    fi
+
     test_one "$a" 0
     test_one "$a" 1
 done
